@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -40,6 +40,7 @@ target_values = {
     'nagiosaddconf'     : "Snippet to add to nagios.cfg",
     'nagios_auth_name'  : "HTTP Basic AuthName for Nagios",
     'nagios_binary'     : "Absolute path to Nagios binary itself",
+    'nagios_version'    : "Nagios version",
     'nagios_config_file': "Absolute path to nagios.cfg",
     'nagios_startscript': "Nagios startskript (usually in /etc/init.d)",
     'nagios_status_file': "Absolute path to Nagios' status.dat",
@@ -437,6 +438,12 @@ try:
         # Path to executable
         result['nagios_binary'] = process_executable(pid)
 
+        # Nagios version
+        result['nagios_version'] = ""
+        for line in os.popen(result["nagios_binary"]+ " --version 2>/dev/null"):
+            if line.startswith("Nagios Core") or line.startswith("Icinga Core"):
+                result['nagios_version'] = line.split()[2]
+
         # Path to startscript
         for path in [ '/etc/init.d/nagios', '/etc/init.d/nagios3', '/etc/init.d/icinga' ]:
             if os.path.exists(path):
