@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -24,8 +24,7 @@
 # to the Free Software Foundation, Inc., 51 Franklin St,  Fifth Floor,
 # Boston, MA 02110-1301 USA.
 
-import config, re, pprint, time
-import weblib, htmllib
+import config, re, pprint, time, views
 from lib import *
 
 
@@ -79,8 +78,8 @@ CRIT = 2
 UNKNOWN = 3
 UNAVAIL = 4
 
-service_state_names = { OK:"OK", WARN:"WARN", CRIT:"CRIT", UNKNOWN:"UNKNOWN", PENDING:"PENDING", UNAVAIL:"UNAVAILABLE"}
-host_state_names = { 0:"UP", 1:"DOWN", 2:"UNREACHABLE" }
+service_state_names = { OK:_("OK"), WARN:_("WARN"), CRIT:_("CRIT"), UNKNOWN:_("UNKNOWN"), PENDING:_("PENDING"), UNAVAIL:_("UNAVAILABLE")}
+host_state_names = { 0:_("UP"), 1:_("DOWN"), 2:_("UNREACHABLE") }
 
 AGGR_HOST  = 0
 AGGR_MULTI = 1
@@ -254,6 +253,9 @@ def compile_forest(user, only_hosts = None, only_groups = None):
         used_cache = True
         return # In this case simply skip further compilations
 
+    if not config.aggregations and not config.host_aggregations:
+        return # nothing to do, BI not used
+
     # If we have previously only partly compiled and now there is no
     # filter, then throw away partly compiled data.
     if (cache["compiled_hosts"] or cache["compiled_groups"]) \
@@ -315,8 +317,17 @@ def compile_forest(user, only_hosts = None, only_groups = None):
     single_affected_hosts = []
     for aggr_type, aggregations in aggr_list:
         for entry in aggregations:
+            if entry[0] == config.DISABLED:
+                continue
+
+            if entry[0] == config.HARD_STATES:
+                use_hard_states = True
+                entry = entry[1:]
+            else:
+                use_hard_states = False
+
             if len(entry) < 3:
-                raise MKConfigError(_("<h1>Invalid aggregation <tt>%s</tt>'</h1>"
+                raise MKConfigError(_("<h1>Invalid aggregation <tt>%s</tt></h1>"
                                       "Must have at least 3 entries (has %d)") % (entry, len(entry)))
 
             if type(entry[0]) == list:
@@ -337,6 +348,7 @@ def compile_forest(user, only_hosts = None, only_groups = None):
 
             for this_entry in new_entries:
                 remove_empty_nodes(this_entry)
+                this_entry["use_hard_states"] = use_hard_states
 
             new_entries = [ e for e in new_entries if len(e["nodes"]) > 0 ]
 
@@ -518,7 +530,8 @@ def find_matching_services(aggr_type, what, calllist):
         if middle in g_services_by_hostname:
             entries = [ ((e[0], host_re), e[1]) for e in g_services_by_hostname[middle] ]
             host_re = "(.*)"
-    elif not honor_site and not '*' in host_re and not '$' in host_re and not '|' in host_re:
+    elif not honor_site and not '*' in host_re and not '$' in host_re \
+         and not '|' in host_re and not '[' in host_re:
         # Exact host match
         entries = [ ((e[0], host_re), e[1]) for e in g_services_by_hostname.get(host_re, []) ]
 
@@ -562,14 +575,14 @@ def find_matching_services(aggr_type, what, calllist):
         if host_matches != None:
             if what == config.FOREACH_CHILD:
                 list_of_matches  = [ host_matches + (child,) for child in childs ]
-            if what == config.FOREACH_PARENT:
+            elif what == config.FOREACH_PARENT:
                 list_of_matches  = [ host_matches + (parent,) for parent in parents ]
             else:
                 list_of_matches = [ host_matches ]
 
-            for host_matches in list_of_matches:
+            for matched_host in list_of_matches:
                 if service_re == config.HOST_STATE:
-                    matches.add(host_matches)
+                    matches.add(matched_host)
                 else:
                     for service in services:
                         mo = (service_re, service)
@@ -578,7 +591,7 @@ def find_matching_services(aggr_type, what, calllist):
                         m = regex(service_re).match(service)
                         if m:
                             svc_matches = tuple(m.groups())
-                            matches.add(host_matches + svc_matches)
+                            matches.add(matched_host + svc_matches)
                         else:
                             service_nomatch_cache.add(mo)
 
@@ -694,14 +707,24 @@ def compile_aggregation_rule(aggr_type, rule, args, lvl):
         global g_remaining_refs
         g_remaining_refs = []
 
+    # Convert new dictionary style rule into old tuple based
+    # format
+    if type(rule) == dict:
+        rule = (
+            rule.get("title", _("Untitled BI rule")),
+            rule.get("params", []),
+            rule.get("aggregation", "worst"),
+            rule.get("nodes", [])
+        )
+
     if len(rule) != 4:
-        raise MKConfigError(_("<h3>Invalid aggregation rule</h1>"
+        raise MKConfigError(_("<b>Invalid aggregation rule</b><br><br>"
                 "Aggregation rules must contain four elements: description, argument list, "
                 "aggregation function and list of nodes. Your rule has %d elements: "
                 "<pre>%s</pre>") % (len(rule), pprint.pformat(rule)))
 
     if lvl == 50:
-        raise MKConfigError(_("<h3>Depth limit reached</h3>"
+        raise MKConfigError(_("<b>Depth limit reached</b><br><br>"
                 "The nesting level of aggregations is limited to 50. You either configured "
                 "too many levels or built an infinite recursion. This happened in rule <pre>%s</pre>")
                   % pprint.pformat(rule))
@@ -710,7 +733,7 @@ def compile_aggregation_rule(aggr_type, rule, args, lvl):
 
     # check arguments and convert into dictionary
     if len(arglist) != len(args):
-        raise MKConfigError(_("<h1>Invalid rule usage</h1>"
+        raise MKConfigError(_("<b>Invalid rule usage</b><br><br>"
                 "The rule '%s' needs %d arguments: <tt>%s</tt><br>"
                 "You've specified %d arguments: <tt>%s</tt>") % (
                     description, len(arglist), repr(arglist), len(args), repr(args)))
@@ -872,7 +895,8 @@ def match_host_tags(have_tags, required_tags):
 def compile_leaf_node(host_re, service_re = config.HOST_STATE):
     found = []
     honor_site = SITE_SEP in host_re
-    if not honor_site and not '*' in host_re and not '$' in host_re and not '|' in host_re:
+    if not honor_site and not '*' in host_re and not '$' in host_re \
+        and not '|' in host_re and '[' not in host_re:
         entries = [ ((e[0], host_re), e[1]) for e in g_services_by_hostname.get(host_re, []) ]
 
     else:
@@ -946,20 +970,6 @@ def compile_leaf_node(host_re, service_re = config.HOST_STATE):
 
 service_nomatch_cache = set([])
 
-regex_cache = {}
-def regex(r):
-    rx = regex_cache.get(r)
-    if rx:
-        return rx
-    try:
-        rx = re.compile(r)
-    except Exception, e:
-        raise MKConfigError(_("Invalid regular expression '%s': %s") % (r, e))
-    regex_cache[r] = rx
-    return rx
-
-
-
 
 #     _____                     _   _
 #    | ____|_  _____  ___ _   _| |_(_) ___  _ __
@@ -984,19 +994,20 @@ def regex(r):
 # Execution of the trees. Returns a tree object reflecting
 # the states of all nodes
 def execute_tree(tree, status_info = None):
+    use_hard_states = tree["use_hard_states"]
     if status_info == None:
         required_hosts = tree["reqhosts"]
         status_info = get_status_info(required_hosts)
-    return execute_node(tree, status_info)
+    return execute_node(tree, status_info, use_hard_states)
 
-def execute_node(node, status_info):
+def execute_node(node, status_info, use_hard_states):
     if node["type"] == NT_LEAF:
-        return execute_leaf_node(node, status_info)
+        return execute_leaf_node(node, status_info, use_hard_states)
     else:
-        return execute_rule_node(node, status_info)
+        return execute_rule_node(node, status_info, use_hard_states)
 
 
-def execute_leaf_node(node, status_info):
+def execute_leaf_node(node, status_info, use_hard_states):
 
     site, host = node["host"]
     service = node.get("service")
@@ -1004,8 +1015,13 @@ def execute_leaf_node(node, status_info):
     # Get current state of host and services
     status = status_info.get((site, host))
     if status == None:
-        return ({ "state" : MISSING, "output" : _("Host %s not found") % host}, None, node)
-    host_state, host_output, service_state = status
+        return ({
+            "state"               : MISSING,
+            "output"              : _("Host %s not found") % host,
+            "in_downtime"         : False,
+            "acknowledged"        : False,
+        }, None, node)
+    host_state, host_hard_state, host_output, host_in_downtime, host_acknowledged, service_state = status
 
     # Get state assumption from user
     if service:
@@ -1018,32 +1034,64 @@ def execute_leaf_node(node, status_info):
     if service:
         for entry in service_state: # list of all services of that host
             if entry[0] == service:
-                state, has_been_checked, output = entry[1:]
+                state, has_been_checked, output, hard_state, attempt, max_attempts, downtime_depth, acknowledged = entry[1:9]
                 if has_been_checked == 0:
                     output = _("This service has not been checked yet")
                     state = PENDING
-                state = {"state":state, "output":output}
+                if use_hard_states:
+                    st = hard_state
+                else:
+                    st = state
+                state = {
+                    "state"        : st,
+                    "output"       : output,
+                    "in_downtime"  : downtime_depth > 0,
+                    "acknowledged" : not not acknowledged,
+                }
                 if state_assumption != None:
-                    assumed_state = {"state":state_assumption,
-                                     "output" : _("Assumed to be %s") % service_state_names[state_assumption]}
+                    assumed_state = {
+                        "state"        : state_assumption,
+                        "output"       : _("Assumed to be %s") % service_state_names[state_assumption],
+                        "in_downtime"  : downtime_depth > 0,
+                        "acknowledged" : not not acknowledged,
+                    }
+
                 else:
                     assumed_state = None
                 return (state, assumed_state, node)
 
-        return ({"state":MISSING, "output": _("This host has no such service")}, None, node)
+        return ({
+                "state"        : MISSING,
+                "output"       : _("This host has no such service"),
+                "in_downtime"  : False,
+                "acknowledged" : False,
+            }, None, node)
 
     else:
-        aggr_state = {0:OK, 1:CRIT, 2:UNKNOWN, -1:PENDING}[host_state]
-        state = {"state":aggr_state, "output" : host_output}
+        if use_hard_states:
+            st = host_hard_state
+        else:
+            st = host_state
+        aggr_state = {0:OK, 1:CRIT, 2:UNKNOWN, -1:PENDING}[st]
+        state = {
+            "state"        : aggr_state,
+            "output"       : host_output,
+            "in_downtime"  : host_in_downtime,
+            "acknowledged" : host_acknowledged,
+            }
         if state_assumption != None:
-            assumed_state = {"state": state_assumption,
-                             "output" : _("Assumed to be %s") % host_state_names[state_assumption]}
+            assumed_state = {
+                "state"        : state_assumption,
+                "output"       : _("Assumed to be %s") % host_state_names[state_assumption],
+                "in_downtime"  : host_in_downtime,
+                "acknowledged" : host_acknowledged,
+                }
         else:
             assumed_state = None
         return (state, assumed_state, node)
 
 
-def execute_rule_node(node, status_info):
+def execute_rule_node(node, status_info, use_hard_states):
     # get aggregation function
     funcspec = node["func"]
     parts = funcspec.split('!')
@@ -1058,11 +1106,23 @@ def execute_rule_node(node, status_info):
     subtrees = []
     node_states = []
     assumed_states = []
+    downtime_states = []
+    ack_states = [] # Needed for computing the acknowledgement of non-OK nodes
     one_assumption = False
     for n in node["nodes"]:
-        result = execute_node(n, status_info) # state, assumed_state, node [, subtrees]
+        result = execute_node(n, status_info, use_hard_states) # state, assumed_state, node [, subtrees]
         subtrees.append(result)
 
+        # Assume items in downtime as CRIT when computing downtime state
+        downtime_states.append(({"state": result[0]["in_downtime"] and 2 or 0, "output" : ""}, result[2]))
+
+        # Assume non-OK nodes that are acked as OK
+        if result[0]["acknowledged"]:
+            acked_state = 0
+        else:
+            acked_state = result[0]["state"]
+        ack_states.append(({"state": acked_state, "output" : ""}, result[2]))
+
         node_states.append((result[0], result[2]))
         if result[1] != None:
             assumed_states.append((result[1], result[2]))
@@ -1071,9 +1131,19 @@ def execute_rule_node(node, status_info):
             # no assumption, take real state into assumption array
             assumed_states.append(node_states[-1])
 
+    downtime_state = func(*([downtime_states] + funcargs))
     state = func(*([node_states] + funcargs))
+    state["in_downtime"] = downtime_state["state"] >= 2
+    if state["state"] > 0: # Non-OK-State -> compute acknowledgedment
+        ack_state = func(*([ack_states] + funcargs))
+        state["acknowledged"] = ack_state["state"] == 0 # would be OK if acked problems would be OK
+    else:
+        state["acknowledged"] = False
+
     if one_assumption:
         assumed_state = func(*([assumed_states] + funcargs))
+        assumed_state["in_downtime"] = state["in_downtime"]
+        assumed_state["acknowledged"] = state["acknowledged"]
     else:
         assumed_state = None
     return (state, assumed_state, node, subtrees)
@@ -1101,7 +1171,7 @@ def get_status_info(required_hosts):
         html.live.set_auth_domain('bi')
         data = html.live.query(
                 "GET hosts\n"
-                "Columns: name state plugin_output services_with_info\n"
+                "Columns: name state hard_state plugin_output scheduled_downtime_depth acknowledged services_with_fullstate\n"
                 + filter)
         html.live.set_auth_domain('read')
         tuples += [((site, e[0]), e[1:]) for e in data]
@@ -1111,17 +1181,17 @@ def get_status_info(required_hosts):
 # This variant of the function is configured not with a list of
 # hosts but with a livestatus filter header and a list of columns
 # that need to be fetched in any case
-def get_status_info_filtered(filter_header, only_sites, limit, add_columns, fetch_parents = True):
-    columns = [ "name", "state", "plugin_output", "services_with_info", "parents" ] + add_columns
+def get_status_info_filtered(filter_header, only_sites, limit, add_columns, fetch_parents = True, bygroup=False):
+    columns = [ "name", "host_name", "state", "hard_state", "plugin_output", "scheduled_downtime_depth",
+                "acknowledged", "services_with_fullstate", "parents" ] + add_columns
 
     html.live.set_only_sites(only_sites)
     html.live.set_prepend_site(True)
 
-    query = "GET hosts\n"
+    query = "GET hosts%s\n" % (bygroup and "bygroup" or "")
     query += "Columns: " + (" ".join(columns)) + "\n"
     query += filter_header
 
-
     if config.debug_livestatus_queries \
             and html.output_format == "html" and 'W' in html.display_options:
         html.write('<div class="livestatus message" onmouseover="this.style.display=\'none\';">'
@@ -1147,10 +1217,10 @@ def get_status_info_filtered(filter_header, only_sites, limit, add_columns, fetc
     if fetch_parents:
         parent_filter = []
         for row in data:
-            parent_filter += [ 'Filter: name = %s\n' % p for p in row[5] ]
+            parent_filter += [ 'Filter: name = %s\n' % p for p in row[8] ]
         parent_filter_txt = ''.join(parent_filter)
         parent_filter_txt += 'Or: %d\n' % len(parent_filter)
-        for row in  get_status_info_filtered(filter_header, only_sites, limit, add_columns, False):
+        for row in  get_status_info_filtered(filter_header, only_sites, limit, add_columns, False, bygroup):
             if row['name'] not in hostnames:
                 rows.append(row)
 
@@ -1199,9 +1269,14 @@ def x_best_state(l, x):
 
     return ll[n-1][1]
 
-def aggr_nth_state(nodelist, n, worst_state):
-    states = [ i[0]["state"] for i in nodelist ]
-    state = x_best_state(states, n)
+def aggr_nth_state(nodelist, n, worst_state, ignore_states = None):
+    states = [ i[0]["state"] for i in nodelist if not ignore_states or i[0]["state"] not in ignore_states ]
+    # In case of the ignored states it might happen that the states list is empty. Use the
+    # OK state in this case.
+    if not states:
+        state = OK
+    else:
+        state = x_best_state(states, n)
 
     # limit to worst state
     if state_weight(state) > state_weight(worst_state):
@@ -1209,11 +1284,11 @@ def aggr_nth_state(nodelist, n, worst_state):
 
     return { "state" : state, "output" : "" }
 
-def aggr_worst(nodes, n = 1, worst_state = CRIT):
-    return aggr_nth_state(nodes, -int(n), int(worst_state))
+def aggr_worst(nodes, n = 1, worst_state = CRIT, ignore_states = None):
+    return aggr_nth_state(nodes, -int(n), int(worst_state), ignore_states)
 
-def aggr_best(nodes, n = 1, worst_state = CRIT):
-    return aggr_nth_state(nodes, int(n), int(worst_state))
+def aggr_best(nodes, n = 1, worst_state = CRIT, ignore_states = None):
+    return aggr_nth_state(nodes, int(n), int(worst_state), ignore_states)
 
 config.aggregation_functions["worst"] = aggr_worst
 config.aggregation_functions["best"]  = aggr_best
@@ -1226,18 +1301,33 @@ def aggr_countok_convert(num, count):
 
 def aggr_countok(nodes, needed_for_ok=2, needed_for_warn=1):
     states = [ i[0]["state"] for i in nodes ]
-    num_ok = len([s for s in states if s == 0 ])
+    num_ok      = len([s for s in states if s == 0 ])
+    num_nonok   = len([s for s in states if s > 0 ])
+    num_pending = len(states) - num_ok - num_nonok
+    num_nodes   = num_ok + num_nonok
+
+    # We need to handle the special case "PENDING" separately.
+    # Example: count is set to 50%. You have 10 nodes, all of
+    # which are PENDING, then the outcome must be PENDING, not
+    # CRIT.
+    if num_nodes == 0: # All are pending
+        return { "state": -1, "output": "" }
 
     # counts can be specified as integer (e.g. '2') or
     # as percentages (e.g. '70%').
+    ok_count = aggr_countok_convert(needed_for_ok, num_nodes)
+    warn_count = aggr_countok_convert(needed_for_warn, num_nodes)
 
+    # Enough nodes are OK -> state is OK
+    if num_ok >= ok_count:
+        return { "state": 0, "output": "" }
+
+    # Enough nodes OK in order to trigger warn level -> WARN
+    elif num_ok >= warn_count:
+        return { "state": 1, "output": "" }
 
-    if num_ok >= aggr_countok_convert(needed_for_ok, len(states)):
-        return { "state" : 0, "output" : "" }
-    elif num_ok >= aggr_countok_convert(needed_for_warn, len(states)):
-        return { "state" : 1, "output" : "" }
     else:
-        return { "state" : 2, "output" : "" }
+        return { "state": 2, "output": "" }
 
 config.aggregation_functions["count_ok"] = aggr_countok
 
@@ -1315,23 +1405,23 @@ def ajax_set_assumption():
     save_assumptions()
 
 def ajax_save_treestate():
-    path_id = html.var("path")
+    path_id = html.var_utf8("path")
     current_ex_level, path = path_id.split(":", 1)
     current_ex_level = int(current_ex_level)
 
     saved_ex_level = load_ex_level()
 
     if saved_ex_level != current_ex_level:
-        weblib.set_tree_states('bi', {})
-    weblib.set_tree_state('bi', path, html.var("state") == "open")
-    weblib.save_tree_states()
+        html.set_tree_states('bi', {})
+    html.set_tree_state('bi', path, html.var("state") == "open")
+    html.save_tree_states()
 
     save_ex_level(current_ex_level)
 
 def ajax_render_tree():
-    aggr_group = html.var("group")
+    aggr_group = html.var_utf8("group")
     reqhosts = [ tuple(sitehost.split('#')) for sitehost in html.var("reqhosts").split(',') ]
-    aggr_title = html.var("title")
+    aggr_title = html.var_utf8("title")
     omit_root = not not html.var("omit_root")
     boxes = not not html.var("boxes")
     only_problems = not not html.var("only_problems")
@@ -1367,11 +1457,11 @@ def ajax_render_tree():
 
 def render_tree_foldable(row, boxes, omit_root, expansion_level, only_problems, lazy):
     saved_expansion_level = load_ex_level()
-    treestate = weblib.get_tree_states('bi')
+    treestate = html.get_tree_states('bi')
     if expansion_level != saved_expansion_level:
         treestate = {}
-        weblib.set_tree_states('bi', treestate)
-        weblib.save_tree_states()
+        html.set_tree_states('bi', treestate)
+        html.save_tree_states()
 
     def render_subtree(tree, path, show_host):
         is_leaf = len(tree) == 3
@@ -1444,7 +1534,7 @@ def render_tree_foldable(row, boxes, omit_root, expansion_level, only_problems,
 
             h += aggr_render_node(tree, tree[2]["title"], mc, show_host)
             if not is_empty:
-                h += '<ul id="%d:%s" %sclass="subtree">' % (expansion_level, path_id, style)
+                h += '<ul id="%d:%s" %sclass="subtree">' % (expansion_level or 0, path_id, style)
                 if not omit_content:
                     for node in tree[3]:
                         estate = node[1] != None and node[1] or node[0]
@@ -1462,7 +1552,7 @@ def render_tree_foldable(row, boxes, omit_root, expansion_level, only_problems,
     affected_hosts = row["aggr_hosts"]
     title = row["aggr_tree"]["title"]
     group = row["aggr_group"]
-    url_id = htmllib.urlencode_vars([
+    url_id = html.urlencode_vars([
         ( "group", group ),
         ( "title", title ),
         ( "omit_root", omit_root and "yes" or ""),
@@ -1471,12 +1561,13 @@ def render_tree_foldable(row, boxes, omit_root, expansion_level, only_problems,
         ( "reqhosts", ",".join('%s#%s' % sitehost for sitehost in affected_hosts) ),
     ])
 
-    htmlcode = '<div id="%s" class=bi_tree_container>' % htmllib.attrencode(url_id) + \
+    htmlcode = '<div id="%s" class=bi_tree_container>' % html.attrencode(url_id) + \
                render_subtree(tree, [tree[2]["title"]], len(affected_hosts) > 1) + \
                '</div>'
     return "aggrtree" + (boxes and "_box" or ""), htmlcode
 
 def aggr_render_node(tree, title, mousecode, show_host):
+
     # Check if we have an assumed state: comparing assumed state (tree[1]) with state (tree[0])
     if tree[1] and tree[0] != tree[1]:
         addclass = " " + _("assumed")
@@ -1485,6 +1576,14 @@ def aggr_render_node(tree, title, mousecode, show_host):
         addclass = ""
         effective_state = tree[0]
 
+    if tree[0]["in_downtime"]:
+        title = ('<img class="icon bi" src="images/icon_downtime.png" title="%s">' % \
+            _("This element is currently in a scheduled downtime")) + title
+
+    if tree[0]["acknowledged"]:
+        title = ('<img class="icon bi" src="images/icon_ack.png" title="%s">' % \
+            _("This problem has been acknowledged")) + title
+
     h = '<span class="content state state%d%s">%s</span>\n' \
          % (effective_state["state"], addclass, render_bi_state(effective_state["state"]))
     if mousecode:
@@ -1516,7 +1615,7 @@ def render_assume_icon(site, host, service):
          (_("Assume another state for this item (reload page to activate)"),
          # MIST: DAS HIER MUSS verfünftig für Javascript encodiert werden.
          # Das Ausgangsmaterial sind UTF-8 kodierte str-Objekte.
-          site, host, service != None and service or '')
+          site, host, service != None and service.replace('\\', '\\\\') or '')
     current = str(ass).lower()
     return u'<img state="%s" class=assumption %s src="images/assume_%s.png">\n' % (current, mousecode, current)
 
@@ -1535,23 +1634,18 @@ def aggr_render_leaf(tree, show_host, bare = False):
     # (4) CPU load                  (show_host == False, service != None)
 
     if show_host or not service:
-        host_url = html.makeuri([("view_name", "hoststatus"), ("site", site), ("host", host)], filename="view.py")
+        host_url = html.makeuri_contextless([("view_name", "hoststatus"), ("site", site), ("host", host)], filename="view.py")
 
     if service:
-        service_url = html.makeuri([("view_name", "service"), ("site", site), ("host", host), ("service", service)], filename="view.py")
+        service_url = html.makeuri_contextless([("view_name", "service"), ("site", site), ("host", host), ("service", service)], filename="view.py")
 
     if show_host:
         content += '<a href="%s">%s</a><b class=bullet>&diams;</b>' % (host_url, host.replace(" ", "&nbsp;"))
 
-    if tree[1] and tree[0] != tree[1]:
-        addclass = ' class="state assumed"'
-    else:
-        addclass = ""
-
     if not service:
-        content += '<a href="%s"%s>%s</a>' % (host_url, addclass, _("Host&nbsp;status"))
+        content += '<a href="%s">%s</a>' % (host_url, _("Host&nbsp;status"))
     else:
-        content += '<a href="%s"%s>%s</a>' % (service_url, addclass, service.replace(" ", "&nbsp;"))
+        content += '<a href="%s">%s</a>' % (service_url, service.replace(" ", "&nbsp;"))
 
     if bare:
         return content
@@ -1585,6 +1679,28 @@ def filter_tree_only_problems(tree):
     return state, assumed_state, node, new_subtrees
 
 
+def page_timeline():
+    aggr_group = html.var("av_aggr_group")
+    aggr_name = html.var("av_aggr_name")
+
+    # First compile the required BI aggregates.
+    if config.bi_precompile_on_demand:
+        compile_forest(config.user_id, only_groups = [ aggr_group ])
+    else:
+        compile_forest(config.user_id)
+
+    # In the resulting collection of BI aggregates find
+    # our tree
+    for tree in g_user_cache["forest"][aggr_group]:
+        if tree["title"] == aggr_name:
+            break
+    else:
+        raise MKGeneralException("No aggregation with the name %s" %
+            aggr_name)
+    row = { "aggr_tree" : tree, "aggr_group" : aggr_group }
+    views.render_bi_availability(aggr_name, [row])
+
+
 
 #    ____        _
 #   |  _ \  __ _| |_ __ _ ___  ___  _   _ _ __ ___ ___  ___
@@ -1672,12 +1788,15 @@ def table(columns, add_headers, only_sites, limit, filters):
 
 # Table of all host aggregations, i.e. aggregations using data from exactly one host
 def hostname_table(columns, add_headers, only_sites, limit, filters):
-    return singlehost_table(columns, add_headers, only_sites, limit, filters, True)
+    return singlehost_table(columns, add_headers, only_sites, limit, filters, True, bygroup=False)
+
+def hostname_by_group_table(columns, add_headers, only_sites, limit, filters):
+    return singlehost_table(columns, add_headers, only_sites, limit, filters, True, bygroup=True)
 
 def host_table(columns, add_headers, only_sites, limit, filters):
-    return singlehost_table(columns, add_headers, only_sites, limit, filters, False)
+    return singlehost_table(columns, add_headers, only_sites, limit, filters, False, bygroup=False)
 
-def singlehost_table(columns, add_headers, only_sites, limit, filters, joinbyname):
+def singlehost_table(columns, add_headers, only_sites, limit, filters, joinbyname, bygroup):
     log("--------------------------------------------------------------------\n")
     log("* Starting to compute singlehost_table (joinbyname = %s)\n" % joinbyname)
     load_assumptions() # user specific, always loaded
@@ -1694,7 +1813,7 @@ def singlehost_table(columns, add_headers, only_sites, limit, filters, joinbynam
 
     log("* Getting status information about hosts...\n")
     host_columns = filter(lambda c: c.startswith("host_"), columns)
-    hostrows = get_status_info_filtered(filter_code, only_sites, limit, host_columns, config.bi_precompile_on_demand)
+    hostrows = get_status_info_filtered(filter_code, only_sites, limit, host_columns, config.bi_precompile_on_demand, bygroup)
     log("* Got %d host rows\n" % len(hostrows))
 
     # if limit:
@@ -1758,16 +1877,22 @@ def singlehost_table(columns, add_headers, only_sites, limit, filters, joinbynam
                         row = rows_by_host[sitehost]
                         status_info[sitehost] = [
                              row["state"],
+                             row["hard_state"],
                              row["plugin_output"],
-                             row["services_with_info"] ]
+                             not not hostrow["acknowledged"],
+                             hostrow["scheduled_downtime_depth"] > 0,
+                             row["services_with_fullstate"] ]
                 if status_info == None:
                     break
         else:
             aggrs = g_user_cache["host_aggregations"].get((site, host), [])
             status_info = { (site, host) : [
                 hostrow["state"],
+                hostrow["hard_state"],
                 hostrow["plugin_output"],
-                hostrow["services_with_info"] ] }
+                not not hostrow["acknowledged"],
+                hostrow["scheduled_downtime_depth"] > 0,
+                hostrow["services_with_fullstate"] ] }
 
         for group, aggregation in aggrs:
             row = hostrow.copy()
@@ -1782,8 +1907,11 @@ def singlehost_table(columns, add_headers, only_sites, limit, filters, joinbynam
                     if this_row:
                         status_info[(site, host)] = [
                             this_row['state'],
+                            this_row['hard_state'],
                             this_row['plugin_output'],
-                            this_row['services_with_info'],
+                            not not this_row["acknowledged"],
+                            this_row["scheduled_downtime_depth"] > 0,
+                            this_row['services_with_fullstate'],
                         ]
 
             row.update(create_aggregation_row(aggregation, status_info))
