@@ -6,7 +6,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -34,7 +34,7 @@ function getAllPsuFiles($path) {
   $files = array();
   if($h = opendir($path)) {
 		while(($file = readdir($h)) !== false) {
-      if(preg_match('/^PSU_[0-9]+\.rrd$/', $file, $aRet))
+      if(preg_match('/^PSU_[0-9]+_output\.rrd$/', $file, $aRet))
         $files[] = $aRet[0];
     }
 		natcasesort($files);
