@@ -6,7 +6,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -23,9 +23,19 @@
 # to the Free Software Foundation, Inc., 51 Franklin St,  Fifth Floor,
 # Boston, MA 02110-1301 USA.
 
-$maxmem = $MAX[1] / 1024.0;
+// Make data sources available via names
+$RRD = array();
+foreach ($NAME as $i => $n) {
+    $RRD[$n] = "$RRDFILE[$i]:$DS[$i]:MAX";
+    $WARN[$n] = $WARN[$i];
+    $CRIT[$n] = $CRIT[$i];
+    $MIN[$n]  = $MIN[$i];
+    $MAX[$n]  = $MAX[$i];
+}
+
+$maxmem = $MAX["memory"] / 1024.0;
 $maxmemprint  = sprintf("%5.2f", $maxmem);
-$maxpage = $MAX[2] / 1024.0;
+$maxpage = $MAX["pagefile"] / 1024.0;
 $maxpageprint = sprintf("%5.2f", $maxpage);
 
 $opt[1] = " --vertical-label 'Gigabytes' -X0 "
@@ -34,29 +44,40 @@ $opt[1] = " --vertical-label 'Gigabytes' -X0 "
         . " --title \"Memory and page file usage $hostname\" ";
 
 
-$def[1] = "DEF:mem=$RRDFILE[1]:$DS[1]:MAX " 
+$def[1] = "DEF:mem=$RRD[memory] "
         . "CDEF:memgb=mem,1024,/ "
-        . "DEF:page=$RRDFILE[2]:$DS[2]:MAX " 
+        . "DEF:page=$RRD[pagefile] "
         . "CDEF:pagegb=page,1024,/ "
         . "CDEF:mpagegb=pagegb,-1,* "
-        
-        . "AREA:$maxmem#a0f8c0:\"$maxmemprint GB RAM      \" " 
-        . "AREA:memgb#20d060 " 
-        . "GPRINT:memgb:LAST:\"%5.2lf GB last\" " 
-        . "GPRINT:memgb:AVERAGE:\"%5.2lf GB avg\" " 
-        . "GPRINT:memgb:MAX:\"%5.2lf GB max\" " 
-        . "HRULE:".($WARN[1]/1024)."#FFFF00:\"Warn\" "
-        . "HRULE:".($CRIT[1]/1024)."#FF0000:\"Crit\\n\" "
 
-        . "AREA:\"-$maxpage\"#a0d0e8:\"$maxpageprint GB page file\" " 
-        . "AREA:mpagegb#3040d0 " 
-        . "GPRINT:pagegb:LAST:\"%5.2lf GB last\" " 
-        . "GPRINT:pagegb:AVERAGE:\"%5.2lf GB avg\" " 
-        . "GPRINT:pagegb:MAX:\"%5.2lf GB max\" " 
-        . "HRULE:".(-$WARN[2]/1024)."#FFFF00:\"Warn\" "
-        . "HRULE:".(-$CRIT[2]/1024)."#FF0000:\"Crit\\n\" "
-        
+        . "AREA:$maxmem#b0ffe0:\"$maxmemprint GB RAM      \" "
+        . "AREA:memgb#40f090 "
+        . "GPRINT:memgb:LAST:\"%5.2lf GB last\" "
+        . "GPRINT:memgb:AVERAGE:\"%5.2lf GB avg\" "
+        . "GPRINT:memgb:MAX:\"%5.2lf GB max\" "
+        . "HRULE:".($WARN["memory"]/1024)."#FFFF00:\"Warn\" "
+        . "HRULE:".($CRIT["memory"]/1024)."#FF0000:\"Crit\\n\" "
 
+        . "AREA:\"-$maxpage\"#b0e0f0:\"$maxpageprint GB page file\" "
+        . "AREA:mpagegb#90b0ff "
+        . "GPRINT:pagegb:LAST:\"%5.2lf GB last\" "
+        . "GPRINT:pagegb:AVERAGE:\"%5.2lf GB avg\" "
+        . "GPRINT:pagegb:MAX:\"%5.2lf GB max\" "
+        . "HRULE:".(-$WARN["pagefile"]/1024)."#FFFF00:\"Warn\" "
+        . "HRULE:".(-$CRIT["pagefile"]/1024)."#FF0000:\"Crit\\n\" "
         ;
 
+# If averaging is enabled then we get two further metrics
+if (isset($RRD["memory_avg"])) {
+    $def[1] .= ""
+        . "DEF:memavg=$RRD[memory_avg] "
+        . "CDEF:memavggb=memavg,1024,/ "
+        . "LINE:memavggb#006000:\"Memory Average     \" "
+        . "DEF:pageavg=$RRD[pagefile_avg] "
+        . "CDEF:pageavggb=pageavg,1024,/ "
+        . "CDEF:mpageavggb=pageavggb,-1,* "
+        . "LINE:mpageavggb#000060:\"Pagefile Average\\n\" "
+        ;
+}
+
 ?>
