@@ -6,7 +6,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -25,8 +25,8 @@
 
 $opt[1] = "--vertical-label 'Througput (MByte/s)' -l0  -u 1 --title \"Disk throughput $hostname / $servicedesc\" ";
 
-$def[1]  = "DEF:kb=$RRDFILE[1]:$DS[1]:AVERAGE " ;
-$def[1] .= "CDEF:mb=kb,1024,/ " ;
+$def[1]  = "DEF:bytes=$RRDFILE[1]:$DS[1]:AVERAGE " ;
+$def[1] .= "CDEF:mb=bytes,1048576,/ " ;
 $def[1] .= "AREA:mb#40c080 " ;
 $def[1] .= "GPRINT:mb:LAST:\"%6.1lf MByte/s last\" " ;
 $def[1] .= "GPRINT:mb:AVERAGE:\"%6.1lf MByte/s avg\" " ;
