@@ -6,7 +6,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -25,14 +25,17 @@
 
 $opt[1] = "--vertical-label 'MEMORY(MB)' --upper-limit " . ($MAX[1] * 120 / 100) . " -l0  --title \"Memory usage $hostname\" ";
 
+$maxgb = sprintf("%.1f", $MAX[1] / 1024.0);
+
 $def[1] =  "DEF:ram=$RRDFILE[1]:$DS[1]:AVERAGE " ;
 $def[1] .= "DEF:swap=$RRDFILE[2]:$DS[2]:AVERAGE " ;
 $def[1] .= "DEF:virt=$RRDFILE[3]:$DS[3]:AVERAGE " ;
 $def[1] .= "HRULE:$MAX[3]#000080:\"RAM+SWAP installed\" ";
-$def[1] .= "HRULE:$MAX[1]#2040d0:\"RAM installed\" ";
+$def[1] .= "HRULE:$MAX[1]#2040d0:\"$maxgb GB RAM installed\" ";
 $def[1] .= "HRULE:$WARN[3]#FFFF00:\"Warning\" ";
 $def[1] .= "HRULE:$CRIT[3]#FF0000:\"Critical\\n\" ";
 
+$def[1] .= "'COMMENT:\\n' " ;
 $def[1] .= "AREA:ram#80ff40:\"RAM used     \" " ;
 $def[1] .= "GPRINT:ram:LAST:\"%6.0lf MB last\" " ;
 $def[1] .= "GPRINT:ram:AVERAGE:\"%6.0lf MB avg\" " ;
