@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -24,27 +24,38 @@
 # to the Free Software Foundation, Inc., 51 Franklin St,  Fifth Floor,
 # Boston, MA 02110-1301 USA.
 
-# The following debug code can be used to tackle a mod_python
-# problem when using the local hierarchy for plugins that refer
-# to the config module.
-# import time, os
-# fn = "/tmp/config.load.%d" % os.getpid()
-# if os.path.exists(fn):
-#     raise Exception("Mist: config zweimal geladen!!")
-#
-# file(fn, "a").write("[%d] Geladen: %s\n" % (os.getpid(), time.time()))
-
 import os, pprint, glob
 from lib import *
-import defaults
+
+# In case we start standalone and outside a Check_MK enviroment,
+# we have another path for the defaults.
+# FIXME: Do we need thus rubbish anymore?
+try:
+    import defaults
+except:
+    import defaults_standalone as defaults
 
 # Python 2.3 does not have 'set' in normal namespace.
 # But it can be imported from 'sets'
+# FIXME: We should officially drop Python 2.3 support
 try:
     set()
 except NameError:
     from sets import Set as set
 
+#.
+#   .--Declarations--------------------------------------------------------.
+#   |       ____            _                 _   _                        |
+#   |      |  _ \  ___  ___| | __ _ _ __ __ _| |_(_) ___  _ __  ___        |
+#   |      | | | |/ _ \/ __| |/ _` | '__/ _` | __| |/ _ \| '_ \/ __|       |
+#   |      | |_| |  __/ (__| | (_| | | | (_| | |_| | (_) | | | \__ \       |
+#   |      |____/ \___|\___|_|\__,_|_|  \__,_|\__|_|\___/|_| |_|___/       |
+#   |                                                                      |
+#   +----------------------------------------------------------------------+
+#   |  Declarations of global variables and constants                      |
+#   '----------------------------------------------------------------------'
+
+
 user = None
 user_id = None
 builtin_role_ids = [ "user", "admin", "guest" ] # hard coded in various permissions
@@ -66,9 +77,10 @@ except:
 
 # Global table of available permissions. Plugins may add their own
 # permissions by calling declare_permission()
-permissions_by_name  = {}
-permissions_by_order = []
-permission_sections  = {}
+permissions_by_name              = {}
+permissions_by_order             = []
+permission_sections              = {}
+permission_declaration_functions = []
 
 # Constants for BI
 ALL_HOSTS = '(.*)'
@@ -79,6 +91,8 @@ class FOREACH_CHILD: pass
 class FOREACH_PARENT: pass
 class FOREACH_SERVICE: pass
 class REMAINING: pass
+class DISABLED: pass
+class HARD_STATES: pass
 
 # Has to be declared here once since the functions can be assigned in
 # bi.py and also in multisite.mk. "Double" declarations are no problem
@@ -87,7 +101,8 @@ class REMAINING: pass
 aggregation_functions = {}
 
 
-#   .----------------------------------------------------------------------.
+#.
+#   .--Functions-----------------------------------------------------------.
 #   |             _____                 _   _                              |
 #   |            |  ___|   _ _ __   ___| |_(_) ___  _ __  ___              |
 #   |            | |_ | | | | '_ \ / __| __| |/ _ \| '_ \/ __|             |
@@ -122,7 +137,7 @@ def load_config():
     modification_timestamps = []
 
     # Set default values for all user-changable configuration settings
-    load_default_values(globals())
+    load_plugins()
 
     # First load main file
     include("multisite.mk")
@@ -141,14 +156,27 @@ def load_config():
         include(p)
 
 
-# -------------------------------------------------------------------
-#    ____                     _         _
-#   |  _ \ ___ _ __ _ __ ___ (_)___ ___(_) ___  _ __  ___
-#   | |_) / _ \ '__| '_ ` _ \| / __/ __| |/ _ \| '_ \/ __|
-#   |  __/  __/ |  | | | | | | \__ \__ \ | (_) | | | \__ \
-#   |_|   \___|_|  |_| |_| |_|_|___/___/_|\___/|_| |_|___/
-#
-# -------------------------------------------------------------------
+def reporting_available():
+    try:
+        # Check the existance of one arbitrary config variable from the
+        # reporting module
+        reporting_filename
+        return True
+    except:
+        return False
+
+
+#.
+#   .--Permissions---------------------------------------------------------.
+#   |        ____                     _         _                          |
+#   |       |  _ \ ___ _ __ _ __ ___ (_)___ ___(_) ___  _ __  ___          |
+#   |       | |_) / _ \ '__| '_ ` _ \| / __/ __| |/ _ \| '_ \/ __|         |
+#   |       |  __/  __/ |  | | | | | | \__ \__ \ | (_) | | | \__ \         |
+#   |       |_|   \___|_|  |_| |_| |_|_|___/___/_|\___/|_| |_|___/         |
+#   |                                                                      |
+#   +----------------------------------------------------------------------+
+#   |  Handling of users, permissions and roles                            |
+#   '----------------------------------------------------------------------'
 
 def declare_permission(name, title, description, defaults):
     perm = { "name" : name, "title" : title, "description" : description, "defaults" : defaults }
@@ -169,10 +197,25 @@ def declare_permission(name, title, description, defaults):
 
     permissions_by_name[name] = perm
 
-def declare_permission_section(name, title, prio = 0):
+def declare_permission_section(name, title, prio = 0, do_sort = False):
     # Prio can be a number which is used for sorting. Higher numbers will
     # be listed first, e.g. in the edit dialogs
-    permission_sections[name] = (prio, title)
+    permission_sections[name] = (prio, title, do_sort)
+
+# Some module have a non-fixed list of permissions. For example for
+# each user defined view there is also a permission. This list is
+# not known at the time of the loading of the module - though. For
+# that purpose module can register functions. These functions should
+# just call declare_permission(). They are being called in the correct
+# situations.
+def declare_dynamic_permissions(func):
+    permission_declaration_functions.append(func)
+
+# This function needs to be called by all code that needs access
+# to possible dynamic permissions
+def load_dynamic_permissions():
+    for func in permission_declaration_functions:
+        func()
 
 # Compute permissions for HTTP user and set in
 # global variables. Also store user.
@@ -255,6 +298,8 @@ def roles_of_user(user):
         return [ "user" ]
     elif os.path.exists(config_dir + "/" + user + "/automation.secret"):
         return [ "guest" ] # unknown user with automation account
+    elif 'roles' in default_user_profile:
+        return default_user_profile['roles']
     elif default_user_role:
         return [ default_user_role ]
     else:
@@ -293,6 +338,8 @@ def may_with_roles(some_role_ids, pname):
                 base_role_id = role["basedon"]
             else:
                 base_role_id = role_id
+            if pname not in permissions_by_name:
+                return False # Permission unknown. Assume False. Functionality might be missing
             perm = permissions_by_name[pname]
             he_may = base_role_id in perm["defaults"]
         if he_may:
@@ -308,8 +355,8 @@ def may(pname):
     user_permissions[pname] = he_may
     return he_may
 
-def user_may(u, pname):
-    return may_with_roles(roles_of_user(u), pname)
+def user_may(user_id, pname):
+    return may_with_roles(roles_of_user(user_id), pname)
 
 def need_permission(pname):
     if not may(pname):
@@ -344,30 +391,50 @@ def get_role_permissions():
                 role_permissions[role_id].append(perm['name'])
     return role_permissions
 
+
+def load_stars():
+    return set(load_user_file("favorites", []))
+
+def save_stars(stars):
+    save_user_file("favorites", list(stars))
+
 # Helper functions
-def load_user_file(name, deflt):
+def load_user_file(name, deflt, lock = False):
     path = user_confdir + "/" + name + ".mk"
     try:
+        if lock:
+            aquire_lock(path)
+
         return eval(file(path).read())
     except:
         return deflt
 
-def save_user_file(name, content):
+def save_user_file(name, content, unlock = False):
     path = user_confdir + "/" + name + ".mk"
     try:
         write_settings_file(path, content)
-    except Exception, e:
-        raise MKConfigError(_("Cannot save %s options for user <b>%s</b> into <b>%s</b>: %s") % \
-                (name, user, path, e))
 
-# -------------------------------------------------------------------
-#    ____  _ _
-#   / ___|(_) |_ ___  ___
-#   \___ \| | __/ _ \/ __|
-#    ___) | | ||  __/\__ \
-#   |____/|_|\__\___||___/
-#
-# -------------------------------------------------------------------
+        if unlock:
+            release_lock(path)
+    except Exception, e:
+        # Error while writing file -> release lock
+        release_lock(path)
+        if debug:
+            raise
+        raise MKConfigError(_("Cannot save %s options for user <b>%s</b> into <b>%s</b>: %s") % \
+                (name, user_id, path, e))
+
+#.
+#   .--Sites---------------------------------------------------------------.
+#   |                        ____  _ _                                     |
+#   |                       / ___|(_) |_ ___  ___                          |
+#   |                       \___ \| | __/ _ \/ __|                         |
+#   |                        ___) | | ||  __/\__ \                         |
+#   |                       |____/|_|\__\___||___/                         |
+#   |                                                                      |
+#   +----------------------------------------------------------------------+
+#   |  The config module provides some helper functions for sites.         |
+#   '----------------------------------------------------------------------'
 
 sites = { "": {} }
 use_siteicons = False
@@ -378,7 +445,16 @@ def sitenames():
 def allsites():
     return dict( [(name, site(name))
                   for name in sitenames()
-                  if not site(name).get("disabled", False)] )
+                  if not site(name).get("disabled", False)
+                     and site(name)['socket'] != 'disabled' ] )
+
+def sorted_sites():
+    sitenames = []
+    for sitename, site in allsites().iteritems():
+        sitenames.append((sitename, site['alias']))
+    sitenames = sorted(sitenames, key=lambda k: k[1], cmp = lambda a,b: cmp(a.lower(), b.lower()))
+
+    return sitenames
 
 def site(name):
     s = sites.get(name, {})
@@ -398,6 +474,8 @@ def site_is_local(name):
     sock = s.get("socket")
     return not sock or sock == "unix:" + defaults.livestatus_unix_socket
 
+# FIXME: Should this return True even if all sites but one are disabled?
+# -> should we use allsites() instead of "sites" directly?
 def is_multisite():
     if len(sites) > 1:
         return True
@@ -414,210 +492,20 @@ def read_site_config():
 def save_site_config():
     save_user_file("siteconfig", user_siteconf)
 
+#.
+#   .--Plugins-------------------------------------------------------------.
+#   |                   ____  _             _                              |
+#   |                  |  _ \| |_   _  __ _(_)_ __  ___                    |
+#   |                  | |_) | | | | |/ _` | | '_ \/ __|                   |
+#   |                  |  __/| | |_| | (_| | | | | \__ \                   |
+#   |                  |_|   |_|\__,_|\__, |_|_| |_|___/                   |
+#   |                                 |___/                                |
+#   +----------------------------------------------------------------------+
+#   |  Handling of our own plugins. In plugins other software pieces can   |
+#   |  declare defaults for configuration variables.                       |
+#   '----------------------------------------------------------------------'
 
-# This function is used for loading default values
-# into *global* variables. We need this here, since
-# we need to re-initialize these values every time
-# that we read multisite.mk (because of mod_python
-# makes those persist during HTTP requests).
-def load_default_values(into):
-    #    ____       _
-    #   |  _ \ ___ | | ___  ___
-    #   | |_) / _ \| |/ _ \/ __|
-    #   |  _ < (_) | |  __/\__ \
-    #   |_| \_\___/|_|\___||___/
-    #
-    into["roles"] = {} # User supplied roles
-
-    # define default values for all settings
-    into["debug"]             = False
-    into["profile"]           = False
-    into["users"]             = []
-    into["admin_users"]       = []
-    into["guest_users"]       = []
-    into["default_user_role"] = "user"
-
-    # New style, used by WATO
-    into["multisite_users"] = {}
-
-    #    ____  _     _      _
-    #   / ___|(_) __| | ___| |__   __ _ _ __
-    #   \___ \| |/ _` |/ _ \ '_ \ / _` | '__|
-    #    ___) | | (_| |  __/ |_) | (_| | |
-    #   |____/|_|\__,_|\___|_.__/ \__,_|_|
-    #
-
-    into["sidebar"] = \
-    [('tactical_overview', 'open'),
-     ('search',            'open'),
-     ('views',             'open'),
-     ('bookmarks',         'open'),
-     ('admin',             'open'),
-     ('master_control',    'closed')]
-
-    #    _     _           _ _
-    #   | |   (_)_ __ ___ (_) |_ ___
-    #   | |   | | '_ ` _ \| | __/ __|
-    #   | |___| | | | | | | | |_\__ \
-    #   |_____|_|_| |_| |_|_|\__|___/
-    #
-
-    into["soft_query_limit"] = 1000
-    into["hard_query_limit"] = 5000
-
-    #    ____                        _
-    #   / ___|  ___  _   _ _ __   __| |___
-    #   \___ \ / _ \| | | | '_ \ / _` / __|
-    #    ___) | (_) | |_| | | | | (_| \__ \
-    #   |____/ \___/ \__,_|_| |_|\__,_|___/
-    #
-
-    into["sound_url"] = "sounds/"
-    into["enable_sounds"] = False
-    into["sounds"] = [
-     ( "down",     "down.wav" ),
-     ( "critical", "critical.wav" ),
-     ( "unknown",  "unknown.wav" ),
-     ( "warning",  "warning.wav" ),
-    # ( None,       "ok.wav" ),
-    ]
-
-
-    #   __     ___                             _   _
-    #   \ \   / (_) _____      __   ___  _ __ | |_(_) ___  _ __  ___
-    #    \ \ / /| |/ _ \ \ /\ / /  / _ \| '_ \| __| |/ _ \| '_ \/ __|
-    #     \ V / | |  __/\ V  V /  | (_) | |_) | |_| | (_) | | | \__ \
-    #      \_/  |_|\___| \_/\_/    \___/| .__/ \__|_|\___/|_| |_|___/
-    #                                   |_|
-
-    into["view_option_refreshes"] = [ 30, 60, 90, 0 ]
-    into["view_option_columns"]   = [ 1, 2, 3, 4, 5, 6, 8 ]
-
-    # MISC
-    into["doculink_urlformat"] = "http://mathias-kettner.de/checkmk_%s.html";
-
-
-    #   ____          _                    _     _       _
-    #  / ___|   _ ___| |_ ___  _ __ ___   | |   (_)_ __ | | _____
-    # | |  | | | / __| __/ _ \| '_ ` _ \  | |   | | '_ \| |/ / __|
-    # | |__| |_| \__ \ || (_) | | | | | | | |___| | | | |   <\__ \
-    #  \____\__,_|___/\__\___/|_| |_| |_| |_____|_|_| |_|_|\_\___/
-    #
-
-    into["custom_links"] = {}
-
-    #  __     __         _
-    #  \ \   / /_ _ _ __(_) ___  _   _ ___
-    #   \ \ / / _` | '__| |/ _ \| | | / __|
-    #    \ V / (_| | |  | | (_) | |_| \__ \
-    #     \_/ \__,_|_|  |_|\___/ \__,_|___/
-    #
-
-    into["debug_livestatus_queries"] = False
-
-    # Show livestatus errors in multi site setup if some sites are
-    # not reachable.
-    into["show_livestatus_errors"] = True
-
-    # Set this to a list in order to globally control which views are
-    # being displayed in the sidebar snapin "Views"
-    into["visible_views"] = None
-
-    # Set this list in order to actively hide certain views
-    into["hidden_views"] = None
-
-    # Custom user stylesheet to load (resides in htdocs/)
-    into["custom_style_sheet"] = None
-
-    # URL for start page in main frame (welcome page)
-    into["start_url"] = "dashboard.py"
-
-    # Page heading for main frame set
-    into["page_heading"] = "Check_MK %s"
-
-    # Timeout for rescheduling of host- and servicechecks
-    into["reschedule_timeout"] = 10.0
-
-    # Interval of snapin updates in seconds
-    into["sidebar_update_interval"] = 30.0
-
-    # Number of columsn in "Filter" form
-    into["filter_columns"] = 2
-
-    # Default language for l10n
-    into["default_language"] = None
-
-    # Hide these languages from user selection
-    into['hide_languages'] = []
-
-    # Default timestamp format to be used in multisite
-    into["default_ts_format"] = 'mixed'
-
-    # Default authentication type. Can be changed to e.g. "cookie" for
-    # using the cookie auth
-    into["auth_type"] = 'basic'
-
-    # Show only most used buttons, set to None if you want
-    # always all buttons to be shown
-    into["context_buttons_to_show"] = 5
-
-    # Buffering of HTML output stream
-    into["buffered_http_stream"] = True
-
-    # Maximum livetime of unmodified selections
-    into["selection_livetime"]  = 3600
-
-    #     ____ ___
-    #    | __ )_ _|
-    #    |  _ \| |
-    #    | |_) | |
-    #    |____/___|
-    #
-    into["aggregation_rules"] = {}
-    into["aggregations"] = []
-    into["host_aggregations"] = []
-    into['bi_compile_log'] = None
-    into['bi_precompile_on_demand'] = False
-
-    #    __        ___  _____ ___
-    #    \ \      / / \|_   _/ _ \
-    #     \ \ /\ / / _ \ | || | | |
-    #      \ V  V / ___ \| || |_| |
-    #       \_/\_/_/   \_\_| \___/
-    #
-
-    into["wato_enabled"] = True
-    into["wato_host_tags"] = []
-    into["wato_aux_tags"] = []
-    into["wato_hide_filenames"] = True
-    into["wato_hide_hosttags"] = False
-    into["wato_hide_varnames"] = True
-    into["wato_max_snapshots"] = 50
-    into["wato_num_hostspecs"] = 12
-    into["wato_num_itemspecs"] = 15
-    into["wato_activation_method"] = 'restart'
-    into["wato_write_nagvis_auth"] = False
-    into["wato_hidden_users"] = []
-
-    #     _   _               ____  ____
-    #    | | | |___  ___ _ __|  _ \| __ )
-    #    | | | / __|/ _ \ '__| | | |  _ \
-    #    | |_| \__ \  __/ |  | |_| | |_) |
-    #     \___/|___/\___|_|  |____/|____/
-    #
-
-    into["user_connectors"]      = ['htpasswd']
-    into["ldap_connection"]      = {}
-    into["ldap_userspec"]        = {}
-    into["ldap_groupspec"]       = {}
-    into["ldap_active_plugins"]  = {'email': {}, 'alias': {}, 'auth_expire': {}}
-    into["ldap_cache_livetime"]  = 300
-    into['ldap_debug_log']       = None
-    into["default_user_profile"] = {
-        'roles': ['user'],
-    }
-
-# Make sure, we have all values set right now - until
-# the configuration will be loaded
-load_default_values(globals())
+def load_plugins():
+    load_web_plugins("config", globals())
 
+load_plugins()
