@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -26,9 +26,9 @@ var browser         = navigator.userAgent.toLowerCase();
 var weAreIEF__k     = ((browser.indexOf("msie") != -1) && (browser.indexOf("opera") == -1));
 var weAreOpera      = browser.indexOf("opera") != -1;
 var weAreFirefox    = browser.indexOf("firefox") != -1 || browser.indexOf("namoroka") != -1;
-var contentLocation = null;
-if(contentFrameAccessible())
-    var contentLocation = parent.frames[1].document.location;
+var g_content_loc   = null;
+
+var sidebar_folded = false;
 
 //
 // Sidebar styling and scrolling stuff
@@ -106,15 +106,6 @@ var snapinOffset   = [ 0, 0 ];
 var snapinStartPos = [ 0, 0 ];
 var snapinScrollTop = 0;
 
-function getButton(event) {
-  if (event.which == null)
-    /* IE case */
-    return (event.button < 2) ? "LEFT" : ((event.button == 4) ? "MIDDLE" : "RIGHT");
-  else
-    /* All others */
-    return (event.which < 2) ? "LEFT" : ((event.which == 2) ? "MIDDLE" : "RIGHT");
-}
-
 function snapinStartDrag(event) {
   if (!event)
     event = window.event;
@@ -159,8 +150,7 @@ function snapinDrag(event) {
   // Drag the snapin
   snapinDragging.style.position = 'absolute';
   var newTop = event.clientY  - snapinOffset[0] - snapinScrollTop;
-  /*if (weAreIEF__k)
-      newTop += document.getElementById('side_content').scrollTop;*/
+  newTop += document.getElementById('side_content').scrollTop;
   snapinDragging.style.top      = newTop + 'px';
   snapinDragging.style.left     = (event.clientX - snapinOffset[1]) + 'px';
   snapinDragging.style.zIndex   = 200;
@@ -283,7 +273,8 @@ function getSnapinList() {
 }
 
 function getSnapinCoords(obj) {
-  var snapinTop = snapinDragging.offsetTop + document.getElementById('side_content').scrollTop;
+  var snapinTop = snapinDragging.offsetTop;
+  // + document.getElementById('side_content').scrollTop;
 
   var bottomOffset = obj.offsetTop + obj.clientHeight - snapinTop;
   if (bottomOffset < 0)
@@ -360,8 +351,49 @@ function contentFrameAccessible() {
     }
 }
 
+function update_content_location() {
+    // init the original frameset title
+    if (typeof(window.parent.orig_title) == 'undefined') {
+        window.parent.orig_title = window.parent.document.title;
+    }
+
+    var content_frame = window.parent.frames[1];
+
+    // Change the title to add the right frame title to reflect the
+    // title of the content URL in the framesets title (window title or tab title)
+    if (content_frame.document.title != '') {
+        var page_title = window.parent.orig_title + ' - ' + content_frame.document.title;
+    } else {
+        var page_title = window.parent.orig_title;
+    }
+    window.parent.document.title = page_title;
+
+    // Construct the URL to be called on page reload
+    var parts = window.parent.location.pathname.split('/')
+    parts.pop();
+    var cmk_path = parts.join('/');
+    var rel_url = content_frame.location.pathname + content_frame.location.search + content_frame.location.hash
+    var index_url = cmk_path + '/index.py?start_url=' + encodeURIComponent(rel_url);
+
+    if (window.parent.history.replaceState) {
+        if (rel_url && rel_url != 'blank') {
+            // Update the URL to be called on reload, e.g. via F5, to make the
+            // frameset switch to exactly this URL
+            window.parent.history.replaceState({}, page_title, index_url);
+
+            // only update the internal flag var if the url was not blank and has been updated
+            //otherwise try again on next scheduler run
+            g_content_loc = parent.frames[1].document.location.href;
+        }
+    } else {
+        // Only a browser without history.replaceState support reaches this. Sadly
+        // we have no F5/reload fix for them...
+        g_content_loc = parent.frames[1].document.location.href;
+    }
+}
+
 function debug(s) {
-  window.parent.frames[1].document.write(s+'<br />');
+    window.parent.frames[1].document.write(s+'<br />');
 }
 
 
@@ -370,30 +402,30 @@ function debug(s) {
 // to be treated here.
 var g_just_resizing = 0;
 function setSidebarHeight() {
-  var oHeader  = document.getElementById('side_header');
-  var oContent = document.getElementById('side_content');
-  var oFooter  = document.getElementById('side_footer');
-  var height   = pageHeight();
+    var oHeader  = document.getElementById('side_header');
+    var oContent = document.getElementById('side_content');
+    var oFooter  = document.getElementById('side_footer');
+    var height   = pageHeight();
 
-  // Resize sidebar frame on Chrome (and other webkit browsers)
-  if (isWebkit()) {
-      var oldcols = parent.document.body.cols.split(",");
-      var oldwidth = parseInt(oldcols[0]);
-      var width = oHeader.clientWidth;
-      var target_width = oldwidth * 280.0 / width;
-      var newcols = target_width.toString() + ",*";
-      parent.document.body.cols = newcols;
-  }
+    // Resize sidebar frame on Chrome (and other webkit browsers)
+    if (isWebkit()) {
+        var oldcols = parent.document.body.cols.split(",");
+        var oldwidth = parseInt(oldcols[0]);
+        var width = oHeader.clientWidth;
+        var target_width = parseInt(oldwidth * 280.0 / width);
+        var newcols = target_width.toString() + ",*";
+        parent.document.body.cols = newcols;
+    }
 
-  // Don't handle zero heights
-  if (height == 0)
-    return;
+    // Don't handle zero heights
+    if (height == 0)
+      return;
 
-  oContent.style.height = (height - oHeader.clientHeight - oFooter.clientHeight + 4) + 'px';
+    oContent.style.height = (height - oHeader.clientHeight - oFooter.clientHeight + 4) + 'px';
 
-  oFooter = null;
-  oContent = null;
-  oHeader = null;
+    oFooter = null;
+    oContent = null;
+    oHeader = null;
 }
 
 var scrolling = true;
@@ -421,10 +453,19 @@ function startDragScroll(event) {
   if (!event)
     event = window.event;
 
+  if (sidebar_folded) {
+      unfoldSidebar();
+      return false;
+  }
+  else if (!sidebar_folded && event.clientX < 10) {
+      foldSidebar();
+      return false;
+  }
+
+
   var target = getTarget(event);
   var button = getButton(event);
 
-  // Evtl. auch nur mit Shift Taste: (e.button == 0 && (e["shiftKey"])
   if (dragging === false && button == 'LEFT'
       && target.tagName != 'A'
       && target.tagName != 'INPUT'
@@ -482,6 +523,34 @@ function dragScroll(event) {
   return false;
 }
 
+function foldSidebar()
+{
+    sidebar_folded = true;
+    document.getElementById('check_mk_sidebar').style.position = "relative";
+    document.getElementById('check_mk_sidebar').style.left = "-265px";
+    if (isWebkit()) {
+        var oldcols = parent.document.body.cols.split(",");
+        var oldwidth = parseInt(oldcols[0]);
+        var new_width = 10.0 / 280.0 * oldwidth;
+        parent.document.body.cols = new_width.toString() + ",*";
+    }
+    else
+        parent.document.body.cols = "10,*";
+    get_url('sidebar_fold.py?fold=yes');
+}
+
+
+function unfoldSidebar()
+{
+    document.getElementById('check_mk_sidebar').style.position = "";
+    document.getElementById('check_mk_sidebar').style.left = "0";
+    parent.document.body.cols = "280,*";
+    sidebar_folded = false;
+    get_url('sidebar_fold.py?fold=');
+}
+
+
+
 /************************************************
  * Mausrad scrollen
  *************************************************/
@@ -555,7 +624,7 @@ function removeSnapin(id, code) {
   }
 
   // reload main frame if it is just displaying the "add snapin" page
-  var href = escape(parent.frames[1].location);
+  var href = encodeURIComponent(parent.frames[1].location);
   if (href.indexOf("sidebar_add_snapin.py") > -1)
       parent.frames[1].location.reload();
 
@@ -618,12 +687,24 @@ function switch_site(switchvar) {
        everything is affected by the switch */
 }
 
+var g_seconds_to_update = null;
+
+function refresh_single_snapin(name) {
+    var url = 'sidebar_snapin.py?names=' + name;
+    var ids = [ 'snapin_' + name ];
+    get_url(url, bulkUpdateContents, ids);
+}
+
 function sidebar_scheduler() {
-    var timestamp = Date.parse(new Date()) / 1000;
+    if (g_seconds_to_update == null)
+        g_seconds_to_update = sidebar_update_interval;
+    else
+        g_seconds_to_update -= 1;
+
     var newcontent = "";
     var to_be_updated = [];
 
-    for (var i in refresh_snapins) {
+    for (var i = 0; i < refresh_snapins.length; i++) {
         var name = refresh_snapins[i][0];
         if (refresh_snapins[i][1] != '') {
             // Special handling for snapins like the nagvis maps snapin which request
@@ -631,7 +712,7 @@ function sidebar_scheduler() {
             // from this url
             var url = refresh_snapins[i][1];
 
-            if (timestamp % sidebar_update_interval == 0) {
+            if (g_seconds_to_update <= 0) {
                 get_url(url, updateContents, "snapin_" + name);
             }
         } else {
@@ -642,7 +723,7 @@ function sidebar_scheduler() {
 
     // Are there any snapins to be bulk updates?
     if(to_be_updated.length > 0) {
-        if (timestamp % sidebar_update_interval == 0) {
+        if (g_seconds_to_update <= 0) {
             var url = 'sidebar_snapin.py?names=' + to_be_updated.join(',');
             if (sidebar_restart_time !== null)
                 url += '&since=' + sidebar_restart_time;
@@ -656,19 +737,31 @@ function sidebar_scheduler() {
         }
     }
 
+    if (g_sidebar_notify_interval !== null) {
+        var timestamp = Date.parse(new Date()) / 1000;
+        if (timestamp % g_sidebar_notify_interval == 0) {
+            update_messages();
+        }
+    }
+
     // Detect page changes and re-register the mousemove event handler
     // in the content frame. another bad hack ... narf
-    if (contentFrameAccessible() && contentLocation != parent.frames[1].document.location) {
+    if(contentFrameAccessible() && g_content_loc != parent.frames[1].document.location.href) {
         registerEdgeListeners(parent.frames[1]);
-        contentLocation = parent.frames[1].document.location;
+        update_content_location();
     }
+
+    if (g_seconds_to_update <= 0)
+        g_seconds_to_update = sidebar_update_interval;
+
     setTimeout(function(){sidebar_scheduler();}, 1000);
 }
 
 function addBookmark() {
     href = parent.frames[1].location;
     title = parent.frames[1].document.title;
-    get_url("add_bookmark.py?title=" + escape(title) + "&href=" + escape(href), updateContents, "snapin_bookmarks");
+    get_url("add_bookmark.py?title=" + encodeURIComponent(title)
+            + "&href=" + encodeURIComponent(href), updateContents, "snapin_bookmarks");
 }
 
 /************************************************
@@ -678,7 +771,7 @@ function addBookmark() {
 function setCookie(cookieName, value,expiredays) {
     var exdate = new Date();
     exdate.setDate(exdate.getDate() + expiredays);
-    document.cookie = cookieName + "=" + escape(value) +
+    document.cookie = cookieName + "=" + encodeURIComponent(value) +
         ((expiredays == null) ? "" : ";expires=" + exdate.toUTCString());
 }
 
@@ -694,7 +787,7 @@ function getCookie(cookieName) {
     var cookieEnd = document.cookie.indexOf(";", cookieStart);
     if(cookieEnd == -1)
         cookieEnd = document.cookie.length;
-    return unescape(document.cookie.substring(cookieStart, cookieEnd));
+    return decodeURIComponent(document.cookie.substring(cookieStart, cookieEnd));
 }
 
 function initScrollPos() {
@@ -708,19 +801,75 @@ function storeScrollPos() {
     setCookie('sidebarScrollPos', document.getElementById('side_content').scrollTop, null);
 }
 
+/************************************************
+ * WATO Folders snapin handling
+ *************************************************/
+
+// FIXME: Make this somehow configurable - use the start url?
+g_last_view   = 'dashboard.py?name=main';
+g_last_folder = '';
+
+// highlight the followed link (when both needed snapins are available)
+function highlight_link(link_obj, container_id) {
+    var this_snapin = document.getElementById(container_id);
+    if (container_id == 'snapin_container_wato_folders')
+        var other_snapin = document.getElementById('snapin_container_views');
+    else
+        var other_snapin = document.getElementById('snapin_container_wato_folders');
+
+    if (this_snapin && other_snapin) {
+        if (this_snapin.getElementsByClassName)
+            var links = this_snapin.getElementsByClassName('link');
+        else
+            var links = document.getElementsByClassName('link', this_snapin);
+
+        for (var i = 0; i < links.length; i++) {
+            links[i].style.fontWeight = 'normal';
+        }
+
+        link_obj.style.fontWeight = 'bold';
+    }
+}
+
+function wato_folders_clicked(link_obj, folderpath) {
+    g_last_folder = folderpath;
+    highlight_link(link_obj, 'snapin_container_wato_folders');
+    parent.frames[1].location = g_last_view + '&wato_folder=' + encodeURIComponent(g_last_folder);
+}
+
+function wato_views_clicked(link_obj) {
+    g_last_view = link_obj.href;
+
+    highlight_link(link_obj, 'snapin_container_views');
+    highlight_link(link_obj, 'snapin_container_dashboards');
+
+    if (g_last_folder != '') {
+        // Navigate by using javascript, cancel following the default link
+        parent.frames[1].location = g_last_view + '&wato_folder=' + encodeURIComponent(g_last_folder);
+        return false;
+    } else {
+        // Makes use the url stated in href attribute
+        return true;
+    }
+}
+
+/************************************************
+ * WATO Foldertree (Standalone) snapin handling
+ *************************************************/
+
 /* Foldable Tree in snapin */
-function wato_tree_click(folderpath) {
+function wato_tree_click(link_obj, folderpath) {
     var topic  = document.getElementById('topic').value;
     var target = document.getElementById('target_' + topic).value;
 
     if(target.substr(0, 9) == 'dashboard') {
         dashboard_name = target.substr(10, target.length);
-        href = 'dashboard.py?name=' + escape(dashboard_name);
+        href = 'dashboard.py?name=' + encodeURIComponent(dashboard_name);
     } else {
-        href = 'view.py?view_name=' + escape(target);
+        href = 'view.py?view_name=' + encodeURIComponent(target);
     }
 
-    href += '&wato_folder=' + escape(folderpath);
+    href += '&wato_folder=' + encodeURIComponent(folderpath);
 
     parent.frames[1].location = href;
 }
@@ -743,7 +892,7 @@ function wato_tree_topic_changed(topic_field) {
     }
 
     // Then send the info to python code via ajax call for persistance
-    get_url('ajax_set_foldertree.py?topic=' + escape(topic) + '&target=');
+    get_url('ajax_set_foldertree.py?topic=' + encodeURIComponent(topic) + '&target=');
 }
 
 function wato_tree_target_changed(target_field) {
@@ -751,7 +900,7 @@ function wato_tree_target_changed(target_field) {
     var target = target_field.value;
 
     // Send the info to python code via ajax call for persistance
-    get_url('ajax_set_foldertree.py?topic=' + escape(topic) + '&target=' + escape(target));
+    get_url('ajax_set_foldertree.py?topic=' + encodeURIComponent(topic) + '&target=' + encodeURIComponent(target));
 }
 
 // adds a variable to a GET url, but tries to remove that
@@ -763,10 +912,111 @@ function add_html_var(url, varname, value) {
     var re = new RegExp('&' + varname + '=[^&]*');
     var new_url = url.replace(re, "");
     if (new_url.indexOf('?') != '-1')
-        new_url += "&" + varname + "=" + escape(value);
+        new_url += "&" + varname + "=" + encodeURIComponent(value);
     else
-        new_url += "?" + varname + "=" + escape(value);
+        new_url += "?" + varname + "=" + encodeURIComponent(value);
     return new_url;
 }
 
 
+/************************************************
+ * Popup Message Handling
+ *************************************************/
+
+// integer representing interval in seconds or <null> when disabled.
+var g_sidebar_notify_interval;
+
+function init_messages(interval) {
+    g_sidebar_notify_interval = interval;
+
+    // Are there pending messages? Render the initial state of
+    // trigger button
+    update_message_trigger();
+}
+
+function handle_update_messages(_unused, code) {
+    // add new messages to container
+    var c = document.getElementById('messages');
+    if (c) {
+        c.innerHTML = code;
+        executeJSbyObject(c);
+        update_message_trigger();
+    }
+}
+
+function update_messages() {
+    // Remove all pending messages from container
+    var c = document.getElementById('messages');
+    if (c) {
+        c.innerHTML = '';
+    }
+
+    // retrieve new messages
+    get_url('sidebar_get_messages.py', handle_update_messages);
+}
+
+function get_hint_messages(c) {
+    var hints;
+    if (c.getElementsByClassName)
+        hints = c.getElementsByClassName('popup_msg');
+    else
+        hints = document.getElementsByClassName('popup_msg', c);
+    return hints;
+}
+
+function update_message_trigger() {
+    var c = document.getElementById('messages');
+    if (c) {
+        var b = document.getElementById('msg_button');
+        var hints = get_hint_messages(c);
+        if (hints.length > 0) {
+            // are there pending messages? make trigger visible
+            b.style.display = 'inline';
+
+            // Create/Update a blinking number label
+            var l = document.getElementById('msg_label');
+            if (!l) {
+                var l = document.createElement('span');
+                l.setAttribute('id', 'msg_label');
+                b.appendChild(l);
+            }
+
+            l.innerHTML = '' + hints.length;
+        } else {
+            // no messages: hide the trigger
+            b.style.display = 'none';
+        }
+    }
+}
+
+function mark_message_read(msg_id) {
+    get_url('sidebar_message_read.py?id=' + msg_id);
+
+    // Update the button state
+    update_message_trigger();
+}
+
+function read_message() {
+    var c = document.getElementById('messages');
+    if (!c)
+        return;
+
+    // extract message from teh message container
+    var hints = get_hint_messages(c);
+    var msg = hints[0];
+    c.removeChild(msg);
+
+    // open the next message in a window
+    c.parentNode.appendChild(msg);
+
+    // tell server that the message has been read
+    var msg_id = msg.id.replace('message-', '');
+    mark_message_read(msg_id);
+}
+
+function message_close(msg_id) {
+    var m = document.getElementById('message-' + msg_id);
+    if (m) {
+        m.parentNode.removeChild(m);
+    }
+}
