@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -29,7 +29,7 @@
 function getElementsByClass(cl) {
     var items = new Array();
     var elements = document.getElementsByTagName('*');
-    for(var i = 0; i < elements.length; i++)
+    for (var i = 0; i < elements.length; i++)
         if (elements[i].className == cl)
             items.push(elements[i]);
     return items;
@@ -86,7 +86,8 @@ function wato_fix_visibility() {
 
     var oTable = oHostTags.childNodes[0]; /* tbody */
     oHostTags = null;
-    for (var i in oTable.childNodes) {
+
+    for (var i = 0; i < oTable.childNodes.length; i++) {
         var oTr = oTable.childNodes[i];
         if (oTr.tagName == 'TR') {
             var oTdLegend = oTr.childNodes[0];
@@ -97,7 +98,7 @@ function wato_fix_visibility() {
             /* If the Checkbox is unchecked try to get a value from the inherited_tags */
             var oCheckbox = oTdLegend.childNodes[1].childNodes[0];
             if (oCheckbox.checked == false ){
-                var attrname = oCheckbox.parentNode.parentNode.id;
+                var attrname = 'attr_' + oCheckbox.name.replace('_change_', '');
                 if(attrname in inherited_tags && inherited_tags[attrname] !== null){
                     currentTags = currentTags.concat(inherited_tags[attrname].split("|"));
                 }
@@ -119,7 +120,7 @@ function wato_fix_visibility() {
        stored in the global variable wato_depends_on_tags, which is filled
        during the creation of the web page. */
 
-    for (var i in wato_check_attributes) {
+    for (var i = 0; i < wato_check_attributes.length; i++) {
         var attrname = wato_check_attributes[i];
         /* Now comes the tricky part: decide whether that attribute should
            be visible or not: */
@@ -132,8 +133,8 @@ function wato_fix_visibility() {
 
         // Visibility depends on roles
         if( display == "" && attrname in wato_depends_on_roles){
-            for (var i in wato_depends_on_roles[attrname]) {
-                var role = wato_depends_on_roles[attrname][i];
+            for (var index = 0; index < wato_depends_on_roles[attrname].length; index++) {
+                var role = wato_depends_on_roles[attrname][index];
                 var negate = role[0] == '!';
                 var rolename = negate ? role.substr(1) : role;
                 var have_role = user_roles.indexOf(rolename) != -1;
@@ -146,8 +147,8 @@ function wato_fix_visibility() {
 
         // Visibility depends on tags
         if( display == "" && attrname in wato_depends_on_tags){
-            for (var i in wato_depends_on_tags[attrname]) {
-                var tag = wato_depends_on_tags[attrname][i];
+            for (var index = 0; index < wato_depends_on_tags[attrname].length; index++) {
+                var tag = wato_depends_on_tags[attrname][index];
                 var negate = tag[0] == '!';
                 var tagname = negate ? tag.substr(1) : tag;
                 var have_tag = currentTags.indexOf(tagname) != -1;
@@ -199,9 +200,9 @@ function wato_fix_visibility() {
 
     // FIXME: use generic identifier for each form
     var available_forms = [ "form_edithost", "form_editfolder", "form_bulkedit" ];
-    for ( var try_form in available_forms ){
-            var my_form = document.getElementById(available_forms[try_form]);
-            if (my_form != null) {
+    for (var try_form = 0; try_form < available_forms.length; try_form++) {
+        var my_form = document.getElementById(available_forms[try_form]);
+        if (my_form != null) {
             for (var child in my_form.childNodes){
                 oTr = my_form.childNodes[child];
                 if (oTr.className == "nform"){
@@ -260,36 +261,53 @@ function progress_handle_error(data, code) {
     progress_handle_response(data, '', code);
 }
 
-function progress_handle_response(data, code, http_code) {
+function progress_handle_response(data, code, http_code)
+{
     var mode = data[0];
     var item = data[1];
 
     var header = null;
     var body = null;
-    if(http_code !== undefined) {
+    if (http_code !== undefined) {
         // If the request failed report the item as failed
+        // Note: the item is either a plain item (single-item-mode), or
+        // a list of items which are separated with ;. Also it is possible
+        // that additional parameters are prefixed to the item separated
+        // by pipes. The bulk inventory uses that format for doing a couple
+        // of hosts at the same time. So here we detect both variants.
+
+        var parts = data[1].split("|");
+        var last_part = parts[parts.length-1];
+        var items = last_part.split(";");
+        var num_failed = items.length;
+
         // - Report failed state
         // - Update the total count (item 0 = 1)
         // - Update the failed stats
-        header = [ 'failed', 1 ];
-        for(var i = 1; i <= Math.max.apply(Math, progress_fail_stats); i++) {
-            if(progress_fail_stats.indexOf(i) !== -1) {
-                header.push(1);
-            } else {
+        header = [ 'failed', num_failed ];
+        for (var i = 1; i <= Math.max.apply(Math, progress_fail_stats); i++) {
+            if (progress_fail_stats.indexOf(i) !== -1) {
+                header.push(num_failed);
+            }
+            else {
                 header.push(0);
             }
         }
-        body = 'Inventory of ' + item + ' failed\n'
-              +'<div class=exc><h1>HTTP-Request failed</h1>'
-              +'HTTP-Code: ' + http_code + '<br />'
-              +'Parameters: ' + data + '</div>\n';
-    } else {
+
+        body = '';
+        for (var i=0; i < items.length; i++)
+        {
+            body += items[i] + ' failed: HTTP-Request failed with code ' + http_code + '<br>';
+        }
+    }
+    else {
         // Regular response processing
         try {
             var header = eval(code.split("\n", 1)[0]);
             if (header === null)
                 alert('Header is null!');
-        } catch(err) {
+        }
+        catch(err) {
             alert('Invalid response: ' + code);
         }
 
@@ -342,6 +360,7 @@ function progress_retry() {
     document.getElementById('progress_abort').style.display    = '';
     progress_clean_log();
     clear_progress_stats();
+    // Note: no bulksize limit is applied here
     progress_items = failed_items;
     failed_items = Array();
     progress_scheduler(progress_mode, progress_url, progress_timeout, [], "", "");
@@ -432,11 +451,13 @@ function progress_clean_log() {
     log = null;
 }
 
-function progress_scheduler(mode, url_prefix, timeout, items, end_url, success_stats, fail_stats, term_url, finished_txt) {
+function progress_scheduler(mode, url_prefix, timeout, items, transids, end_url, success_stats, fail_stats, term_url, finished_txt) {
     // Initialize
     if (progress_items === null) {
+        total_num_items        = items.length;
         progress_items         = items;
         failed_items           = Array();
+        progress_transids      = transids;
         progress_total_num     = items.length;
         progress_end_url       = end_url;
         progress_term_url      = term_url;
@@ -455,11 +476,17 @@ function progress_scheduler(mode, url_prefix, timeout, items, end_url, success_s
     // Regular processing when not paused and not already running
     if (!progress_paused && !progress_running) {
         if (progress_items.length > 0) {
+            num_items_left = progress_items.length;
+            perc = Math.round(100 - (100.0 * num_items_left / total_num_items));
+            // title = progress_items.length + " Items, " + progress_success_stats.length + " Stats, " + progress_fail_stats.length + " Failed.";
+            title = perc + "%";
             // Progressing
             progress_running = true;
             // Remove leading pipe signs (when having no folder set)
-            update_progress_title(progress_items[0].replace(/^\|*/g, ''));
-            get_url(url_prefix + '&_transid=-1&_item=' + escape(progress_items[0]),
+            // update_progress_title(percentage + "%");
+            update_progress_title(title);
+            use_transid = progress_transids.shift();
+            get_url(url_prefix + '&_transid=' + use_transid + '&_item=' + encodeURIComponent(progress_items[0]),
                 progress_handle_response,    // regular handler (http code 200)
                 [ mode, progress_items[0] ], // data to hand over to handlers
                 progress_handle_error        // error handler
@@ -496,7 +523,11 @@ function wato_do_activation(est) {
 
     // Hide the activate changes button
     var button = document.getElementById('act_changes_button');
-    if(button) {
+    if (button) {
+        button.style.display = 'none';
+    }
+    button = document.getElementById('discard_changes_button');
+    if (button) {
         button.style.display = 'none';
         button = null;
     }
@@ -515,23 +546,26 @@ function wato_activation_result(siteid, code) {
         oState.innerHTML = "<div class='repprogress ok' style='width: 160px;'>OK</div>";
         oMsg.innerHTML = code.substr(3);
 
-        // Reload page after 2 secs
+        // Reload page after 1 secs
         setTimeout(wato_replication_finish, 1000);
     } else {
         oState.innerHTML = '';
         oMsg.innerHTML = code;
-
-        // Show the activate changes button again
-        var button = document.getElementById('act_changes_button');
-        if(button) {
-            button.style.display = '';
-            button = null;
-        }
+        wato_hide_changes_button();
     }
     oState = null;
     oMsg = null;
 }
 
+function wato_hide_changes_button()
+{
+    var button = document.getElementById('act_changes_button');
+    if (button) {
+        button.style.display = 'none';
+        button = null;
+    }
+}
+
 //   +----------------------------------------------------------------------+
 //   |           ____            _ _           _   _                        |
 //   |          |  _ \ ___ _ __ | (_) ___ __ _| |_(_) ___  _ __             |
@@ -581,10 +615,8 @@ function wato_replication_result(siteid, code) {
 function wato_replication_finish() {
     // check if we have a sidebar-main frame setup
     if (this.parent && parent && parent.frames[1] == this)
-        parent.frames[0].location.reload(); // reload sidebar
+        reload_sidebar();
 
-    // always reload main
-    // this.location.reload();
     var oDiv = document.getElementById("act_changes_button");
     oDiv.style.display = "none";
     oDiv = null
@@ -607,6 +639,70 @@ function wato_randomize_secret(id, len) {
     oInput.value = secret;
 }
 
+// .-Profile Repl----------------------------------------------------------.
+// |          ____             __ _ _        ____            _             |
+// |         |  _ \ _ __ ___  / _(_) | ___  |  _ \ ___ _ __ | |            |
+// |         | |_) | '__/ _ \| |_| | |/ _ \ | |_) / _ \ '_ \| |            |
+// |         |  __/| | | (_) |  _| | |  __/ |  _ <  __/ |_) | |            |
+// |         |_|   |_|  \___/|_| |_|_|\___| |_| \_\___| .__/|_|            |
+// |                                                  |_|                  |
+// +-----------------------------------------------------------------------+
+
+var profile_replication_progress = new Array();
+
+function wato_do_profile_replication(siteid, est, progress_text) {
+    get_url("wato_ajax_profile_repl.py?site=" + siteid,
+            wato_profile_replication_result, siteid);
+    profile_replication_progress[siteid] = 20; // 10 of 10 10ths
+    setTimeout("profile_replication_step('"+siteid+"', "+est+", '"+progress_text+"');", est/20);
+}
+
+function profile_replication_set_status(siteid, image, text) {
+    var oImg = document.getElementById("site-" + siteid).childNodes[0];
+    oImg.title = text;
+    oImg.src = 'images/icon_'+image+'.png';
+}
+
+function profile_replication_step(siteid, est, progress_text) {
+    if (profile_replication_progress[siteid] > 0) {
+        profile_replication_progress[siteid]--;
+        var perc = (20.0 - profile_replication_progress[siteid]) * 100 / 20;
+        var img;
+        if (perc >= 75)
+            img = 'repl_75';
+        else if (perc >= 50)
+            img = 'repl_50';
+        else if (perc >= 25)
+            img = 'repl_25';
+        else
+            img = 'repl_pending';
+        profile_replication_set_status(siteid, img, progress_text);
+        setTimeout("profile_replication_step('"+siteid+"',"+est+", '"+progress_text+"');", est/20);
+    }
+}
+
+// g_num_replsites is set by the page code in wato.py to the number async jobs started
+// in total
+function wato_profile_replication_result(siteid, code) {
+    profile_replication_progress[siteid] = 0;
+    var oDiv = document.getElementById("site-" + siteid);
+    if (code[0] == "0")
+        profile_replication_set_status(siteid, 'repl_success', code.substr(2));
+    else
+        profile_replication_set_status(siteid, 'repl_failed', code.substr(2));
+    g_num_replsites--;
+
+    if (0 == g_num_replsites) {
+        setTimeout(wato_profile_replication_finish, 1000);
+    }
+}
+
+function wato_profile_replication_finish() {
+    // check if we have a sidebar-main frame setup
+    if (this.parent && parent && parent.frames[1] == this)
+        reload_sidebar();
+}
+
 // ----------------------------------------------------------------------------
 // Folderlist
 // ----------------------------------------------------------------------------
@@ -696,3 +792,98 @@ function wato_toggle_move_folder(event, oButton) {
         event.returnValue = false;
     return false;
 }
+
+// .--Host Diag-----------------------------------------------------------.
+// |              _   _           _     ____  _                           |
+// |             | | | | ___  ___| |_  |  _ \(_) __ _  __ _               |
+// |             | |_| |/ _ \/ __| __| | | | | |/ _` |/ _` |              |
+// |             |  _  | (_) \__ \ |_  | |_| | | (_| | (_| |              |
+// |             |_| |_|\___/|___/\__| |____/|_|\__,_|\__, |              |
+// |                                                  |___/               |
+// +----------------------------------------------------------------------+
+
+function handle_host_diag_result(ident, response_text) {
+    var img   = document.getElementById(ident + '_img');
+    var log   = document.getElementById(ident + '_log');
+    var retry = document.getElementById(ident + '_retry');
+
+    if (response_text[0] == "0") {
+        img.src = "images/icon_success.png";
+        log.className = "log diag_success";
+    }
+    else {
+        img.src = "images/icon_failed.png";
+        log.className = "log diag_failed";
+    }
+
+    log.innerHTML = response_text.substr(1).replace(/\n/g, "<br>\n");
+
+    retry.src = "images/icon_retry.gif";
+    retry.style.display = 'inline';
+}
+
+function start_host_diag_test(ident, hostname) {
+    var log   = document.getElementById(ident + '_log');
+    var img   = document.getElementById(ident + '_img');
+    var retry = document.getElementById(ident + '_retry');
+
+    retry.style.display = 'none';
+
+    var vars = '';
+    vars = '&ipaddress=' + encodeURIComponent(document.getElementsByName('vs_host_p_ipaddress')[0].value);
+    vars += '&snmp_community=' + encodeURIComponent(document.getElementsByName('vs_host_p_snmp_community')[0].value);
+    vars += '&agent_port=' + encodeURIComponent(document.getElementsByName('vs_rules_p_agent_port')[0].value);
+    vars += '&snmp_timeout=' + encodeURIComponent(document.getElementsByName('vs_rules_p_snmp_timeout')[0].value);
+    vars += '&snmp_retries=' + encodeURIComponent(document.getElementsByName('vs_rules_p_snmp_retries')[0].value);
+    vars += '&datasource_program=' + encodeURIComponent(document.getElementsByName('vs_rules_p_datasource_program')[0].value);
+
+    img.src = "images/icon_loading.gif";
+    log.innerHTML = "...";
+    get_url("wato_ajax_diag_host.py?host=" + encodeURIComponent(hostname)
+            + "&_test=" + encodeURIComponent(ident) + vars,
+              handle_host_diag_result, ident);
+}
+
+// .-Active Checks---------------------------------------------------------.
+// |       _        _   _              ____ _               _              |
+// |      / \   ___| |_(_)_   _____   / ___| |__   ___  ___| | _____       |
+// |     / _ \ / __| __| \ \ / / _ \ | |   | '_ \ / _ \/ __| |/ / __|      |
+// |    / ___ \ (__| |_| |\ V /  __/ | |___| | | |  __/ (__|   <\__ \      |
+// |   /_/   \_\___|\__|_| \_/ \___|  \____|_| |_|\___|\___|_|\_\___/      |
+// |                                                                       |
+// '-----------------------------------------------------------------------'
+
+function execute_active_check(site, hostname, checktype, item, divid)
+{
+    var oDiv = document.getElementById(divid);
+    var url = "wato_ajax_execute_check.py?" +
+           "site="       + encodeURIComponent(site) +
+           "&host="      + encodeURIComponent(hostname)  +
+           "&checktype=" + encodeURIComponent(checktype) +
+           "&item="      + encodeURIComponent(item);
+    get_url(url, handle_execute_active_check, oDiv);
+}
+
+
+function handle_execute_active_check(oDiv, response_text)
+{
+    // Response is STATUS\nOUTPUT
+    var parts = response_text.split("\n", 3);
+    var state = parts[0];
+    if (state == "-1")
+        state = "p"; // Pending
+    var statename = parts[1];
+    var output = parts[2];
+    oDiv.innerHTML = output;
+
+    // Change name and class of status columns
+    var oTr = oDiv.parentNode.parentNode;
+    if (hasClass(oTr, "even0"))
+        var rowtype = "even";
+    else
+        var rowtype = "odd";
+    oTr.className = "data " + rowtype + state;
+    var oTdState = oTr.childNodes[1]; // 0 is a text node due to a \n
+    oTdState.className = "state statesvc state" + state;
+    oTdState.innerHTML = statename;
+}
