@@ -6,7 +6,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -24,7 +24,7 @@
 # Boston, MA 02110-1301 USA.
 
 
-VERSION=1.2.2p2
+VERSION=1.2.6p4
 NAME=check_mk
 LANG=
 LC_ALL=
@@ -225,6 +225,7 @@ HOMEBASEDIR=$HOME/$NAME
 
 ask_title "Installation directories of check_mk"
 
+
 ask_dir bindir /usr/bin $HOMEBASEDIR/bin $OMD_ROOT/local/bin "Executable programs" \
   "Directory where to install executable programs such as check_mk itself.
 This directory should be in your search path (\$PATH). Otherwise you
@@ -250,8 +251,8 @@ ask_dir checkmandir /usr/share/doc/$NAME/checks $HOMEBASEDIR/doc/checks $OMD_ROO
   "Directory for manuals for the various checks. The manuals can be viewed
 with check_mk -M <CHECKNAME>"
 
-ask_dir vardir /var/lib/$NAME $HOMEBASEDIR/var $OMD_ROOT/var/check_mk "working directory of check_mk" \
-  "check_mk will create caches files, automatically created checks and
+ask_dir vardir /var/lib/$NAME $HOMEBASEDIR/var $OMD_ROOT/var/check_mk "working directory of Check_MK" \
+  "Check_MK will create log files, automatically created checks and
 other files into this directory. The setup will create several subdirectories
 and makes them writable by the Nagios process"
 
@@ -375,6 +376,14 @@ Those templates make the history graphs look nice. PNP4Nagios
 expects such templates in the directory pnp/templates in your
 document root for static web pages"
 
+ask_dir rrd_path /var/lib/nagios/rrd $HOMEBASEDIR/var/nagios/rrd $OMD_ROOT/var/pnp4nagios/perfdata "RRD files" \
+    "Configure the directory PNP4Nagios stores the RRD database files in"
+
+ask_dir rrdcached_socket /tmp/rrdcached.sock $HOMEBASEDIR/var/rrdcached.sock $OMD_ROOT/tmp/run/rrdached.sock "rrdcached socket" \
+    "If you use the rrdcached to process performance data from Nagios,
+you can configure the socket of the rrdcached here to make the prediction
+feature use it"
+
 # -------------------------------------------------------------------
 ask_title "Check_MK Livestatus Module"
 # -------------------------------------------------------------------
@@ -398,6 +407,10 @@ C++ compiler installed in order to do this"
 
 if [ "$enable_livestatus" = yes ]
 then
+  ask_dir -d nagios_version "3.5.0" "3.5.0" "OMD Monitoring Site $OMD_SITE" "Nagios / Icinga version" \
+   "The version is required for the compilation of the livestatus module.
+Depending on the major version (3 or 4) different nagios headers are included"
+
   ask_dir libdir /usr/lib/$NAME $HOMEBASEDIR/lib $OMD_ROOT/local/lib/mk-livestatus "check_mk's binary modules" \
    "Directory for architecture dependent binary libraries and plugins
 of check_mk"
@@ -445,6 +458,7 @@ fi
 
 checksdir=$sharedir/checks
 notificationsdir=$sharedir/notifications
+inventorydir=$sharedir/inventory
 modulesdir=$sharedir/modules
 web_dir=$sharedir/web
 localedir=$sharedir/locale
@@ -469,12 +483,14 @@ check_mk_configdir          = '$confdir/conf.d'
 share_dir                   = '$sharedir'
 checks_dir                  = '$checksdir'
 notifications_dir           = '$notificationsdir'
+inventory_dir               = '$inventorydir'
 check_manpages_dir          = '$checkmandir'
 modules_dir                 = '$modulesdir'
 locale_dir                  = '$localedir'
 agents_dir                  = '$agentsdir'
-var_dir                     = '$vardir'
 lib_dir                     = '$libdir'
+var_dir                     = '$vardir'
+log_dir                     = '$vardir/log'
 snmpwalks_dir               = '$vardir/snmpwalks'
 autochecksdir               = '$vardir/autochecks'
 precompiled_hostchecks_dir  = '$vardir/precompiled'
@@ -483,6 +499,8 @@ tcp_cache_dir		    = '$vardir/cache'
 tmp_dir		            = '$vardir/tmp'
 logwatch_dir                = '$vardir/logwatch'
 nagios_objects_file         = '$nagconfdir/check_mk_objects.cfg'
+rrd_path                    = '$rrd_path'
+rrddcached_socket           = '$rrdcached_socket'
 nagios_command_pipe_path    = '$nagpipe'
 check_result_path           = '$check_result_path'
 nagios_status_file          = '$nagios_status_file'
@@ -550,7 +568,14 @@ compile_livestatus ()
    mkdir -p $D
    tar xvzf $SRCDIR/livestatus.tar.gz -C $D
    pushd $D
-   ./configure --libdir=$libdir --bindir=$bindir &&
+
+   local CONFIGURE_OPTS=""
+   if [ -n "$nagios_version" ] ; then
+        if [ ${nagios_version:0:1} == 4 ] ; then
+           CONFIGURE_OPTS="--with-nagios4"
+        fi
+   fi
+   ./configure --libdir=$libdir --bindir=$bindir $CONFIGURE_OPTS &&
    make clean &&
    cat <<EOF > src/livestatus.h &&
 #ifndef livestatus_h
@@ -561,7 +586,7 @@ EOF
    make -j 8  2>&1 &&
    strip src/livestatus.o &&
    mkdir -p $DESTDIR$libdir &&
-   install -m 755 src/livecheck src/livestatus.o $DESTDIR$libdir &&
+   install -m 755 src/livestatus.o $DESTDIR$libdir &&
    mkdir -p $DESTDIR$bindir &&
    install -m 755 src/unixcat $DESTDIR$bindir &&
    popd
@@ -602,6 +627,7 @@ VARDIR=$mkeventdstatedir
 CONFDIR=$confdir
 MKEVENTD_SYSLOG=off
 RUNUSER=$nagiosuser
+LIVESTATUS=$livesock
 # DEBUG="--debug --foreground"
 
 PIDFILE=\$SOCKETDIR/pid
@@ -783,6 +809,8 @@ do
 	   tar xzf $SRCDIR/checks.tar.gz -C $DESTDIR$checksdir &&
 	   mkdir -p $DESTDIR$notificationsdir &&
 	   tar xzf $SRCDIR/notifications.tar.gz -C $DESTDIR$notificationsdir &&
+	   mkdir -p $DESTDIR$inventorydir &&
+	   tar xzf $SRCDIR/inventory.tar.gz -C $DESTDIR$inventorydir &&
 	   mkdir -p $DESTDIR$web_dir &&
 	   tar xzf $SRCDIR/web.tar.gz -C $DESTDIR$web_dir &&
 	   cp $DESTDIR$modulesdir/defaults $DESTDIR$web_dir/htdocs/defaults.py &&
@@ -801,15 +829,17 @@ do
 	       sed -ri 's@^export MK_LIBDIR="(.*)"@export MK_LIBDIR="'"$agentslibdir"'"@' $agent
 	       sed -ri 's@^export MK_CONFDIR="(.*)"@export MK_CONFDIR="'"$agentsconfdir"'"@' $agent
 	   done &&
-	   mkdir -p $DESTDIR$vardir/{autochecks,counters,precompiled,cache,logwatch,web,wato} &&
+	   mkdir -p $DESTDIR$vardir/{autochecks,counters,precompiled,cache,logwatch,web,wato,notify,log} &&
 	   if [ -z "$DESTDIR" ] && id "$nagiosuser" > /dev/null 2>&1 && [ $UID = 0 ] ; then
-	     chown -R $nagiosuser $DESTDIR$vardir/{counters,cache,logwatch}
-	     chown $nagiosuser $DESTDIR$vardir/web
+	     chown -R $nagiosuser $DESTDIR$vardir/{counters,cache,logwatch,notify}
+	     chown $nagiosuser $DESTDIR$vardir/{web,log}
            fi &&
 	   mkdir -p $DESTDIR$confdir/conf.d &&
 	   if [ -z "$DESTDIR" ] ; then
 	     chgrp -R $wwwgroup $DESTDIR$vardir/web &&
 	     chmod -R g+w $DESTDIR$vardir/web &&
+	     chgrp -R $wwwgroup $DESTDIR$vardir/log &&
+	     chmod -R g+w $DESTDIR$vardir/log &&
 	     chgrp -R $wwwgroup $DESTDIR$vardir/wato &&
 	     chmod -R g+w $DESTDIR$vardir/wato
              mkdir -p $DESTDIR$vardir/tmp &&
@@ -925,8 +955,19 @@ and change the path there. Restart Apache afterwards."
        Allow from all
        Satisfy any
   </Location>
+
+  # Trigger cron jobs. This is done without authentication
+  <Location "${url_prefix}check_mk/run_cron.py">
+      Order deny,allow
+      Deny from all
+      Allow from 127.0.0.1
+      Satisfy any
+  </Location>
+
 </IfModule>
 
+
+
 <IfModule !mod_python.c>
   Alias ${url_prefix}check_mk $web_dir/htdocs
   <Directory $web_dir/htdocs>
@@ -993,10 +1034,10 @@ EOF
 	   done &&
            # make htpasswd writable by apache, since we need this for
            # WATO. Also create an empty and Apache-writable auth.serials
-           serials_file=${htpasswd_file%/*}/auth.serials &&
+           serials_file=$DESTDIR${htpasswd_file%/*}/auth.serials &&
            touch "$serials_file" &&
-           chown $wwwuser "$serials_file" &&
-           chown $wwwuser "$htpasswd_file" &&
+           (chown $wwwuser "$serials_file" || true) &&
+           (chown $wwwuser "$htpasswd_file" || true) &&
 	   create_sudo_configuration &&
            if [ "$enable_mkeventd" = yes ]
            then
