@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -37,7 +37,7 @@ def render_bi_groups():
     html.write("<ul>")
     for group in bi.aggregation_groups():
         bulletlink(group, "view.py?view_name=aggr_group&aggr_group=%s" %
-              htmllib.urlencode(group))
+              html.urlencode(group))
     html.write("</ul>")
 
 sidebar_snapins["biaggr_groups"] = {
