@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -25,7 +25,6 @@
 # Boston, MA 02110-1301 USA.
 
 import views, time, defaults, dashboard
-import weblib
 from lib import *
 
 # Python 2.3 does not have 'set' in normal namespace.
@@ -67,31 +66,22 @@ sidebar_snapins["about"] = {
 #      \_/  |_|\___| \_/\_/ |___/
 #
 # --------------------------------------------------------------
-visible_views = [ "allhosts", "searchsvc" ]
 
-def views_by_topic():
-    s = [ (view.get("topic", _("Other")), view["title"], name)
-          for name, view
-          in html.available_views.items()
-          if not view["hidden"] and not view.get("mobile")]
-
-    # Add all the dashboards to the views list
-    s += [ (_('Dashboards'), d['title'] and d['title'] or d_name, d_name)
-           for d_name, d
-           in dashboard.dashboards.items()
-    ]
+def visuals_by_topic(permitted_visuals,
+        default_order = [ _("Overview"), _("Hosts"), _("Host Groups"), _("Services"), _("Service Groups"),
+                         _("Business Intelligence"), _("Problems"), _("Addons") ]):
+    s = [ (_u(visual.get("topic") or _("Other")), _u(visual.get("title")), name, 'painters' in visual)
+          for name, visual
+          in permitted_visuals
+          if not visual["hidden"] and not visual.get("mobile")]
 
     s.sort()
 
-    # Enforce a certain order on the topics
-    known_topics = [ _('Dashboards'), _("Hosts"), _("Hostgroups"), _("Services"), _("Servicegroups"),
-                     _("Business Intelligence"), _("Problems"), _("Addons") ]
-
     result = []
-    for topic in known_topics:
+    for topic in default_order:
         result.append((topic, s))
 
-    rest = list(set([ t for (t, _t, _v) in s if t not in known_topics ]))
+    rest = list(set([ t for (t, _t, _v, _i) in s if t not in default_order ]))
     rest.sort()
     for topic in rest:
         if topic:
@@ -100,25 +90,28 @@ def views_by_topic():
     return result
 
 def render_views():
+    views.load_views()
+    dashboard.load_dashboards()
+
     def render_topic(topic, s):
         first = True
-        for t, title, name in s:
-            if config.visible_views and name not in config.visible_views:
+        for t, title, name, is_view in s:
+            if is_view and config.visible_views and name not in config.visible_views:
                 continue
-            if config.hidden_views and name in config.hidden_views:
+            if is_view and config.hidden_views and name in config.hidden_views:
                 continue
             if t == topic:
                 if first:
                     html.begin_foldable_container("views", topic, False, topic, indent=True)
                     first = False
-                if topic == _('Dashboards'):
-                    bulletlink(title, 'dashboard.py?name=%s' % name)
+                if is_view:
+                    bulletlink(title, "view.py?view_name=%s" % name, onclick = "return wato_views_clicked(this)")
                 else:
-                    bulletlink(title, "view.py?view_name=%s" % name)
+                    bulletlink(title, 'dashboard.py?name=%s' % name, onclick = "return wato_views_clicked(this)")
         if not first: # at least one item rendered
             html.end_foldable_container()
 
-    for topic, s in views_by_topic():
+    for topic, s in visuals_by_topic(views.permitted_views().items() + dashboard.permitted_dashboards().items()):
         render_topic(topic, s)
 
     links = []
@@ -130,11 +123,67 @@ def render_views():
 
 sidebar_snapins["views"] = {
     "title" : _("Views"),
-    "description" : _("Links to all views"),
+    "description" : _("Links to global views and dashboards"),
     "render" : render_views,
     "allowed" : [ "user", "admin", "guest" ],
 }
 
+#   .--Dashboards----------------------------------------------------------.
+#   |        ____            _     _                         _             |
+#   |       |  _ \  __ _ ___| |__ | |__   ___   __ _ _ __ __| |___         |
+#   |       | | | |/ _` / __| '_ \| '_ \ / _ \ / _` | '__/ _` / __|        |
+#   |       | |_| | (_| \__ \ | | | |_) | (_) | (_| | | | (_| \__ \        |
+#   |       |____/ \__,_|___/_| |_|_.__/ \___/ \__,_|_|  \__,_|___/        |
+#   |                                                                      |
+#   +----------------------------------------------------------------------+
+#   |                                                                      |
+#   '----------------------------------------------------------------------'
+
+def render_dashboards():
+    dashboard.load_dashboards()
+
+    def render_topic(topic, s, foldable = True):
+        first = True
+        for t, title, name, is_view in s:
+            if t == topic:
+                if first:
+                    if foldable:
+                        html.begin_foldable_container("dashboards", topic, False, topic, indent=True)
+                    else:
+                        html.write('<ul>')
+                    first = False
+                bulletlink(title, 'dashboard.py?name=%s' % name, onclick = "return wato_views_clicked(this)")
+
+        if not first: # at least one item rendered
+            if foldable:
+                html.end_foldable_container()
+            else:
+                html.write('<ul>')
+
+    by_topic = visuals_by_topic(dashboard.permitted_dashboards().items(), default_order = [ _('Overview') ])
+    topics = [ topic for topic, entry in by_topic ]
+
+    if len(topics) < 2:
+        render_topic(by_topic[0][0], by_topic[0][1], foldable = False)
+
+    else:
+        for topic, s in by_topic:
+            render_topic(topic, s)
+
+    links = []
+    if config.may("general.edit_dashboards"):
+        if config.debug:
+            links.append((_("EXPORT"), "export_dashboards.py"))
+        links.append((_("EDIT"), "edit_dashboards.py"))
+        footnotelinks(links)
+
+sidebar_snapins["dashboards"] = {
+    "title"       : _("Dashboards"),
+    "description" : _("Links to all dashboards"),
+    "render"      : render_dashboards,
+    "allowed"     : [ "user", "admin", "guest" ],
+}
+
 # --------------------------------------------------------------
 #    ____                  _                     __
 #   / ___|  ___ _ ____   _(_) ___ ___           / /
@@ -156,7 +205,7 @@ def render_groups(what):
     groups.sort() # sort by Alias in lowercase
     html.write('<ul>')
     for alias_lower, alias, name in groups:
-        url = "view.py?view_name=%sgroup&%sgroup=%s" % (what, what, htmllib.urlencode(name))
+        url = "view.py?view_name=%sgroup&%sgroup=%s" % (what, what, html.urlencode(name))
         bulletlink(alias or name, url)
     html.write('</ul>')
 
@@ -194,8 +243,17 @@ def render_hosts(mode):
         query += "Filter: custom_variable_names < _REALNAME\n"
 
     if mode == "problems":
-        query += "Filter: state > 0\nFilter: worst_service_state > 0\nOr: 2\n"
         view = "problemsofhost"
+        # Exclude hosts and services in downtime
+        svc_query = "GET services\nColumns: host_name\n"\
+                    "Filter: state > 0\nFilter: scheduled_downtime_depth = 0\n"\
+                    "Filter: host_scheduled_downtime_depth = 0\nAnd: 3"
+        problem_hosts = set(map(lambda x: x[1], html.live.query(svc_query)))
+
+        query += "Filter: state > 0\nFilter: scheduled_downtime_depth = 0\nAnd: 2\n"
+        for host in problem_hosts:
+            query += "Filter: name = %s\n" % host
+        query += "Or: %d\n" % (len(problem_hosts) + 1)
 
     hosts = html.live.query(query)
     html.live.set_prepend_site(False)
@@ -227,7 +285,7 @@ def render_hosts(mode):
         else:
             statecolor = 0
         html.write('<div class="statebullet state%d">&nbsp;</div> ' % statecolor)
-        html.write(link(host, target + ("&host=%s&site=%s" % (htmllib.urlencode(host), htmllib.urlencode(site)))))
+        html.write(link(host, target + ("&host=%s&site=%s" % (html.urlencode(host), html.urlencode(site)))))
         html.write("</td>")
         if col == num_columns:
             html.write("</tr>\n")
@@ -264,7 +322,7 @@ sidebar_snapins["summary_hosts"] = {
 
 sidebar_snapins["problem_hosts"] = {
     "title" : _("Problem hosts"),
-    "description" : _("A summary state of all hosts that have problem, with links to problems of those hosts"),
+    "description" : _("A summary state of all hosts that have a problem, with links to problems of those hosts"),
     "render" : lambda: render_hosts("problems"),
     "allowed" : [ "user", "admin", "guest" ],
     "refresh" : True,
@@ -334,7 +392,7 @@ def render_hostmatrix():
             s = 1
         else:
             s = 0
-        url = "view.py?view_name=host&site=%s&host=%s" % (htmllib.urlencode(site), htmllib.urlencode(host))
+        url = "view.py?view_name=host&site=%s&host=%s" % (html.urlencode(site), html.urlencode(host))
         html.write('<td class="state state%s"><a href="%s" title="%s" target="main" style="width:%spx;height:%spx;"></a></td>' %
                                                                                            (s, url, host, cell_size, cell_size))
         if col == n or (row == rows and n == lastcols):
@@ -374,13 +432,7 @@ def render_sitestatus():
     if config.is_multisite():
         html.write("<table cellspacing=0 class=sitestate>")
 
-        # Sort the list of sitenames by sitealias
-        sitenames = []
-        for sitename, site in config.allsites().iteritems():
-            sitenames.append((sitename, site['alias']))
-        sitenames = sorted(sitenames, key=lambda k: k[1], cmp = lambda a,b: cmp(a.lower(), b.lower()))
-
-        for sitename, sitealias in sitenames:
+        for sitename, sitealias in config.sorted_sites():
             site = config.site(sitename)
             if sitename not in html.site_status or "state" not in html.site_status[sitename]:
                 state = "missing"
@@ -581,7 +633,6 @@ def render_performance():
 
     data = html.live.query("GET status\nColumns: service_checks_rate host_checks_rate "
                            "external_commands_rate connections_rate forks_rate "
-                           "livechecks_rate livecheck_overflows_rate "
                            "log_messages_rate cached_log_messages\n")
     for what, col, format in \
         [("Service checks",        0, "%.2f/s"),
@@ -589,10 +640,8 @@ def render_performance():
         ("External commands",      2, "%.2f/s"),
         ("Livestatus-conn.",       3, "%.2f/s"),
         ("Process creations",      4, "%.2f/s"),
-        ("Livechecks",             5, "%.2f/s"),
-        ("Livecheck overflows",    6, "%.2f/s"),
-        ("New log messages",       7, "%.2f/s"),
-        ("Cached log messages",    8, "%d")]:
+        ("New log messages",       5, "%.2f/s"),
+        ("Cached log messages",    6, "%d")]:
         write_line(what + ":", format % sum([row[col] for row in data]))
 
     if len(config.allsites()) == 1:
@@ -682,7 +731,7 @@ function show_speed(percentage) {
     context.shadowOffsetX = 2;
     context.shadowOffsetY = 2;
     context.shadowBlur = 2;
-    context.stroStyle = "#000000";
+    context.strokeStyle = "#000000";
     context.stroke();
     context = null;
 }
@@ -744,11 +793,11 @@ speedometer_show_speed(0, 0, 0);
 
 
 sidebar_snapins["speedometer"] = {
-    "title" : _("Speed-O-Meter"),
-    "description" : _("A gadget that shows your current check rate in relation to "
+    "title" : _("Service Speed-O-Meter"),
+    "description" : _("A gadget that shows your current service check rate in relation to "
                       "the scheduled check rate. If the Speed-O-Meter shows a speed "
-                      "of 100 percent, then all checks are being executed in exactly "
-                      "the rate that is configured (via check_interval)"),
+                      "of 100 percent, all service checks are being executed in exactly "
+                      "the rate that is desired."),
     "render" : render_speedometer,
     "allowed" : [ "admin", ],
     "styles" : """
@@ -888,6 +937,7 @@ def render_master_control():
         ( "enable_notifications",     _("Notifications" )),
         ( "execute_service_checks",   _("Service checks" )),
         ( "execute_host_checks",      _("Host checks" )),
+        ( "enable_flap_detection",    _("Flap Detection" )),
         ( "enable_event_handlers",    _("Event handlers" )),
         ( "process_performance_data", _("Performance data" )),
         ]
@@ -900,8 +950,13 @@ def render_master_control():
         if siteid:
             sitealias = html.site_status[siteid]["site"]["alias"]
             html.begin_foldable_container("master_control", siteid, True, sitealias)
+        is_cmc = html.site_status[siteid]["program_version"].startswith("Check_MK ")
         html.write("<table class=master_control>\n")
         for i, (colname, title) in enumerate(items):
+            # Do not show event handlers on Check_MK Micro Core
+            if is_cmc and colname == 'enable_event_handlers':
+                continue
+
             colvalue = siteline[i + 1]
             url = defaults.url_prefix + ("check_mk/switch_master_state.py?site=%s&switch=%s&state=%d" % (siteid, colname, 1 - colvalue))
             onclick = "get_url('%s', updateContents, 'snapin_master_control')" % url
@@ -972,8 +1027,8 @@ def render_bookmarks():
     n = 0
     for title, href in bookmarks:
         html.write("<div class=bookmark id=\"bookmark_%d\">" % n)
-        iconbutton(_("delete"), "del_bookmark.py?num=%d" % n, "side", "updateContents", 'snapin_bookmarks', css_class = 'bookmark')
-        iconbutton(_("edit"), "edit_bookmark.py?num=%d" % n, "main", css_class = 'bookmark')
+        iconbutton("delete", "del_bookmark.py?num=%d" % n, "side", "updateContents", 'snapin_bookmarks', css_class = 'bookmark')
+        iconbutton("edit", "edit_bookmark.py?num=%d" % n, "main", css_class = 'bookmark')
         html.write(link(title, href))
         html.write("</div>")
         n += 1
@@ -1018,7 +1073,7 @@ def render_custom_links():
         return
 
     def render_list(ids, links):
-        states = weblib.get_tree_states('customlinks')
+        states = html.get_tree_states('customlinks')
         n = 0
         for entry in links:
             n += 1
@@ -1058,3 +1113,469 @@ sidebar_snapins["custom_links"] = {
 }
 """
 }
+
+
+
+#Example Sidebar:
+#Heading1:
+#   * [[link1]]
+#   * [[link2]]
+#
+#----
+#
+#Heading2:
+#   * [[link3]]
+#   * [[link4]]
+
+def render_wiki():
+    filename = defaults.omd_root + '/var/dokuwiki/data/pages/sidebar.txt'
+    html.javascript("""
+    function wiki_search()
+    {
+        var oInput = document.getElementById('wiki_search_field');
+        top.frames["main"].location.href =
+           "/%s/wiki/doku.php?do=search&id=" + escape(oInput.value);
+    }
+    """ % defaults.omd_site)
+
+    html.write('<form id="wiki_search" onSubmit="wiki_search()">')
+    html.write('<input id="wiki_search_field" type="text" name="wikisearch"></input>\n')
+    html.icon_button("#", _("Search"), "wikisearch", onclick="wiki_search();")
+    html.write('</form>')
+    html.write('<div id="wiki_side_clear"></div>')
+
+    start_ul = True
+    ul_started = False
+    try:
+        title = None
+        for line in file(filename).readlines():
+            line = line.strip()
+            if line == "":
+                if ul_started == True:
+                    html.end_foldable_container()
+                    start_ul = True
+                    ul_started = False
+            elif line.endswith(":"):
+                title = line[:-1]
+            elif line == "----":
+                pass
+                # html.write("<br>")
+
+            elif line.startswith("*"):
+                if start_ul == True:
+                    if title:
+                         html.begin_foldable_container("wikisnapin", title, True, title, indent=True)
+                    else:
+                        html.write('<ul>')
+                    start_ul = False
+                    ul_started = True
+
+                erg = re.findall('\[\[(.*)\]\]', line)
+                if len(erg) == 0:
+                    continue
+                erg = erg[0].split('|')
+                if len(erg) > 1:
+                    link = erg[0]
+                    name = erg[1]
+                else:
+                    link = erg[0]
+                    name = erg[0]
+
+                if link.startswith("http://") or link.startswith("https://"):
+                    simplelink(name, link, "_blank")
+                else:
+                    erg = name.split(':')
+                    if len(erg) > 0:
+                        name = erg[-1]
+                    else:
+                        name = erg[0]
+                    bulletlink(name, "/%s/wiki/doku.php?id=%s" % (defaults.omd_site, link))
+
+            else:
+                html.write(line)
+
+        if ul_started == True:
+            html.write("</ul>")
+    except IOError:
+        html.write("<p>To get a navigation menu, you have to create a <a href='/%s/wiki/doku.php?id=%s' "
+                   "target='main'>sidebar</a> in your wiki first.</p>" % (defaults.omd_site, _("sidebar")))
+
+if defaults.omd_root:
+    sidebar_snapins["wiki"] = {
+        "title" : _("Wiki"),
+        "description" : _("Shows the Wiki Navigation of the OMD Site"),
+        "render" : render_wiki,
+        "allowed" : [ "admin", "user", "guest" ],
+        "styles" : """
+        #snapin_container_wiki div.content {
+            font-weight: bold;
+            color: white;
+        }
+
+        #snapin_container_wiki div.content p {
+            font-weight: normal;
+        }
+
+        #wiki_navigation {
+            text-align: left;
+        }
+
+        #wiki_search {
+            width: 232px;
+            padding: 0;
+        }
+
+        #wiki_side_clear {
+            clear: both;
+        }
+
+        #wiki_search img.iconbutton {
+            width: 33px;
+            height: 26px;
+            margin-top: -25px;
+            left: 196px;
+            float: left;
+            position: relative;
+            z-index:100;
+        }
+
+        #wiki_search input {
+            margin:  0;
+            padding: 0px 5px;
+            font-size: 8pt;
+            width: 194px;
+            height: 25px;
+            background-image: url("images/quicksearch_field_bg.png");
+            background-repeat: no-repeat;
+            -moz-border-radius: 0px;
+            border-style: none;
+            float: left;
+        }
+        """
+    }
+
+#   .--Virt. Host Tree-----------------------------------------------------.
+#   |  __     ___      _       _   _           _     _____                 |
+#   |  \ \   / (_)_ __| |_    | | | | ___  ___| |_  |_   _| __ ___  ___    |
+#   |   \ \ / /| | '__| __|   | |_| |/ _ \/ __| __|   | || '__/ _ \/ _ \   |
+#   |    \ V / | | |  | |_ _  |  _  | (_) \__ \ |_    | || | |  __/  __/   |
+#   |     \_/  |_|_|   \__(_) |_| |_|\___/|___/\__|   |_||_|  \___|\___|   |
+#   |                                                                      |
+#   '----------------------------------------------------------------------'
+
+def compute_tag_tree(taglist):
+    html.live.set_prepend_site(True)
+    query = "GET hosts\n" \
+            "Columns: host_name state num_services_ok num_services_warn num_services_crit num_services_unknown custom_variables"
+    hosts = html.live.query(query)
+    html.live.set_prepend_site(False)
+    hosts.sort()
+
+    def get_tag_group_value(groupentries, tags):
+        for entry in groupentries:
+            if entry[0] in tags:
+                return entry[0], entry[1] # tag, title
+        # Not found -> try empty entry
+        for entry in groupentries:
+            if entry[0] == None:
+                return None, entry[1]
+
+        # No empty entry found -> get default (i.e. first entry)
+        return groupentries[0][:2]
+
+    # Prepare list of host tag groups and topics
+    taggroups = {}
+    topics = {}
+    for entry in config.wato_host_tags:
+        grouptitle           = entry[1]
+        if '/' in grouptitle:
+            topic, grouptitle = grouptitle.split("/", 1)
+            topics.setdefault(topic, []).append(entry)
+
+        groupname            = entry[0]
+        group                = entry[2]
+        taggroups[groupname] = group
+
+    tree = {}
+    for site, host_name, state, num_ok, num_warn, num_crit, num_unknown, custom_variables in hosts:
+        # make state reflect the state of the services + host
+        have_svc_problems = False
+        if state:
+            state += 1 # shift 1->2 (DOWN->CRIT) and 2->3 (UNREACH->UNKNOWN)
+        if num_crit:
+            state = 2
+            have_svc_problems = True
+        elif num_unknown:
+            if state != 2:
+                state = 3
+            have_svc_problems = True
+        elif num_warn:
+            if not state:
+                state = 1
+            have_svc_problems = True
+
+        tags = custom_variables.get("TAGS", []).split()
+
+        tree_entry = tree # Start at top node
+
+        # Now go through the levels of the tree. Each level may either be
+        # - a tag group id, or
+        # - "topic:" plus the name of a tag topic. That topic should only contain
+        #   checkbox tags.
+        # The problem with the "topic" entries is, that a host may appear several
+        # times!
+
+        current_branches = [ tree ]
+
+        for tag in taglist:
+            new_current_branches = []
+            for tree_entry in current_branches:
+                if tag.startswith("topic:"):
+                    topic = tag[6:]
+                    if topic in topics: # Could have vanished
+                        # Iterate over all host tag groups with that topic
+                        for entry in topics[topic]:
+                            grouptitle  = entry[1].split("/", 1)[1]
+                            group       = entry[2]
+                            for tagentry in group:
+                                tag_value, tag_title = tagentry[:2]
+                                if tag_value in tags:
+                                    new_current_branches.append(tree_entry.setdefault((tag_title, tag_value), {}))
+
+                else:
+                    if tag not in taggroups:
+                        continue # Configuration error. User deleted tag group after configuring his tree
+                    tag_value, tag_title = get_tag_group_value(taggroups[tag], tags)
+                    new_current_branches.append(tree_entry.setdefault((tag_title, tag_value), {}))
+
+            current_branches = new_current_branches
+
+        for tree_entry in new_current_branches:
+            if not tree_entry:
+                tree_entry.update({
+                    "_num_hosts" : 0,
+                    "_state"     : 0,
+                })
+            tree_entry["_num_hosts"] += 1
+            tree_entry["_svc_problems"] = tree_entry.get("_svc_problems", False) or have_svc_problems
+            if state == 2 or tree_entry["_state"] == 2:
+                tree_entry["_state"] = 2
+            else:
+                tree_entry["_state"] = max(state, tree_entry["_state"])
+
+    return tree
+
+def tag_tree_worst_state(tree):
+    if "_state" in tree:
+        return tree["_state"]
+    else:
+        states = map(tag_tree_worst_state, tree.values())
+        for x in states:
+            if x == 2:
+                return 2
+        return max(states)
+
+def tag_tree_has_svc_problems(tree):
+    if "_svc_problems" in tree:
+        return tree["_svc_problems"]
+    else:
+        for x in tree.values():
+            if tag_tree_has_svc_problems(x):
+                return True
+        return False
+
+def tag_tree_url(taggroups, taglist, viewname):
+    urlvars = [("view_name", viewname), ("filled_in", "filter")]
+    if viewname == "svcproblems":
+        urlvars += [ ("st1", "on"), ("st2", "on"), ("st3", "on") ]
+
+    for nr, (group, tag) in enumerate(zip(taggroups, taglist)):
+        if group.startswith("topic:"):
+            # Find correct tag group for this tag
+            for entry in config.wato_host_tags:
+                for tagentry in entry[2]:
+                    if tagentry[0] == tag: # Found our tag
+                        taggroup = entry[0]
+                        urlvars.append(("host_tag_%d_grp" % nr, taggroup))
+                        urlvars.append(("host_tag_%d_op" % nr, "is"))
+                        urlvars.append(("host_tag_%d_val" % nr, tag))
+                        break
+        else:
+            urlvars.append(("host_tag_%d_grp" % nr, group))
+            urlvars.append(("host_tag_%d_op" % nr, "is"))
+            urlvars.append(("host_tag_%d_val" % nr, tag or ""))
+    return html.makeuri_contextless(urlvars, "view.py")
+
+def tag_tree_bullet(state, path, leaf):
+    code = '<div class="tagtree %sstatebullet state%d">&nbsp;</div>' % ((leaf and "leaf " or ""), state)
+    if not leaf:
+        code = '<a title="%s" href="javascript:virtual_host_tree_enter(%r);">' % \
+           (_("Display the tree only below this node"), "|".join(path)) + code + "</a>"
+    return code + " "
+
+
+def is_tag_subdir(path, cwd):
+    if not cwd:
+        return True
+    elif not path:
+        return False
+    elif path[0] != cwd[0]:
+        return False
+    else:
+        return is_tag_subdir(path[1:], cwd[1:])
+
+def render_tag_tree_level(taggroups, path, cwd, title, tree):
+    if not is_tag_subdir(path, cwd) and not is_tag_subdir(cwd, path):
+        return
+
+    if path != cwd and is_tag_subdir(path, cwd):
+        bullet = tag_tree_bullet(tag_tree_worst_state(tree), path, False)
+        if tag_tree_has_svc_problems(tree):
+            # We cannot use html.plug() here, since this is not (yet)
+            # reentrant and it is used by the sidebar snapin updater.
+            # So we need to duplicate the code of icon_button here:
+            bullet += ('<a target="main" onfocus="if (this.blur) this.blur();" href="%s">'
+                       '<img align=absmiddle class=iconbutton title="%s" src="images/button_svc_problems_lo.png" '
+                       'onmouseover="hilite_icon(this, 1)" onmouseout="hilite_icon(this, 0)"></a>' % (
+                        tag_tree_url(taggroups, path, "svcproblems"),
+                       _("Show the service problems contained in this branch")))
+
+
+        if path:
+            html.begin_foldable_container("tag-tree", ".".join(map(str, path)), False, bullet + title)
+
+    items = tree.items()
+    items.sort()
+
+    for nr, ((title, tag), subtree) in enumerate(items):
+        subpath = path + [tag or ""]
+        url = tag_tree_url(taggroups, subpath, "allhosts")
+        if "_num_hosts" in subtree:
+            title += " (%d)" % subtree["_num_hosts"]
+        href = '<a target=main href="%s">%s</a>' % (url, html.attrencode(title))
+        if "_num_hosts" in subtree:
+
+            if is_tag_subdir(path, cwd):
+                html.write(tag_tree_bullet(subtree["_state"], subpath, True))
+                if subtree.get("_svc_problems"):
+                    url = tag_tree_url(taggroups, subpath, "svcproblems")
+                    html.icon_button(url, _("Show the service problems contained in this branch"),
+                            "svc_problems", target="main")
+                html.write(href)
+                html.write("<br>")
+        else:
+            render_tag_tree_level(taggroups, subpath, cwd, href, subtree)
+
+    if path and path != cwd and is_tag_subdir(path, cwd):
+        html.end_foldable_container()
+
+virtual_host_tree_js = """
+function virtual_host_tree_changed(field)
+{
+    var tree_conf = field.value;
+    // Then send the info to python code via ajax call for persistance
+    get_url_sync('sidebar_ajax_tag_tree.py?conf=' + escape(tree_conf));
+    refresh_single_snapin("tag_tree");
+}
+
+function virtual_host_tree_enter(path)
+{
+    get_url_sync('sidebar_ajax_tag_tree_enter.py?path=' + escape(path));
+    refresh_single_snapin("tag_tree");
+}
+"""
+
+def render_tag_tree():
+    if not config.virtual_host_trees:
+        url = 'wato.py?varname=virtual_host_trees&mode=edit_configvar'
+        html.write(_('You have not defined any virtual host trees. You can '
+                     'do this in the global settings for <a target=main href="%s">Multisite</a>.') % url)
+        return
+
+    tree_conf = config.load_user_file("virtual_host_tree", {"tree": 0, "cwd": {}})
+    if type(tree_conf) == int:
+        tree_conf = {"tree": tree_conf, "cwd":{}} # convert from old style
+
+
+    choices = [ (str(i), v[0]) for i, v in enumerate(config.virtual_host_trees)]
+    html.begin_form("vtree")
+
+    # Give chance to change one level up, if we are in a subtree
+    cwd = tree_conf["cwd"].get(tree_conf["tree"])
+    if cwd:
+        upurl = "javascript:virtual_host_tree_enter(%r)" % "|".join(cwd[:-1])
+        html.icon_button(upurl, _("Go up one tree level"), "back")
+
+    html.select("vtree", choices, str(tree_conf["tree"]), onchange = 'virtual_host_tree_changed(this)')
+    html.write("<br>")
+    html.end_form()
+    html.final_javascript(virtual_host_tree_js)
+
+    title, taggroups = config.virtual_host_trees[tree_conf["tree"]]
+
+    tree = compute_tag_tree(taggroups)
+    render_tag_tree_level(taggroups, [], cwd, _("Virtual Host Tree"), tree)
+
+sidebar_snapins["tag_tree"] = {
+    "title" : _("Virtual Host Tree"),
+    "description" : _("This snapin shows tree views of your hosts based on their tag classifications. You "
+                      "can configure which tags to use in your global settings of Multisite."),
+    "render" : render_tag_tree,
+    "refresh" : True,
+    "allowed" : [ "admin", "user", "guest" ],
+    "styles" : """
+
+#snapin_tag_tree img.iconbutton {
+}
+
+#snapin_tag_tree select {
+    background-color: #6DA1B8;
+    border-color: #123A4A;
+    color: #FFFFFF;
+    font-size: 8pt;
+    height: 19px;
+    margin-bottom: 2px;
+    margin-top: -2px;
+    padding: 0;
+    width: 230px;
+}
+
+#snapin_tag_tree div.statebullet {
+    position: relative;
+    top: 3px;
+    left: 1px;
+    float: none;
+    display: inline-block;
+    width: 8px;
+    height: 8px;
+    margin-right: 0px;
+    box-shadow: 0px 0px 0.7px #284850;
+}
+
+#snapin_tag_tree ul > div.statebullet.leaf {
+    margin-left: 16px;
+}
+#snapin_tag_tree b {
+    font-weight: normal;
+}
+
+#snapin_tag_tree {
+    position: relative;
+    top: 0px;
+    left: 0px;
+}
+#snapin_tag_tree form img.iconbutton {
+    width: 16px;
+    height: 16px;
+    float: none;
+    display: inline-box;
+    position: absolute;
+    top: 9px;
+    left: 14px;
+}
+#snapin_tag_tree select {
+    width: 198px;
+    margin-left: 17px;
+}
+"""
+}
