@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -26,20 +26,24 @@
 
 # This module is needed only for SNMP based checks
 
-OID_END    =  0
-OID_STRING = -1
-OID_BIN    = -2
+import subprocess
 
-def strip_snmp_value(value):
+OID_END              =  0  # Suffix-part of OID that was not specified
+OID_STRING           = -1  # Complete OID as string ".1.3.6.1.4.1.343...."
+OID_BIN              = -2  # Complete OID as binary string "\x01\x03\x06\x01..."
+OID_END_BIN          = -3  # Same, but just the end part
+OID_END_OCTET_STRING = -4  # yet same, but omit first byte (assuming that is the length byte)
+
+def strip_snmp_value(value, hex_plain = False):
     v = value.strip()
     if v.startswith('"'):
         v = v[1:-1]
         if len(v) > 2 and is_hex_string(v):
-            return convert_from_hex(v)
+            return not hex_plain and convert_from_hex(v) or value
         else:
             return v.strip()
     else:
-        return v.strip()
+        return v
 
 def is_hex_string(value):
     # as far as I remember, snmpwalk puts a trailing space within
@@ -69,53 +73,6 @@ def convert_from_hex(value):
 def oid_to_bin(oid):
     return u"".join([ unichr(int(p)) for p in oid.strip(".").split(".") ])
 
-
-def snmpwalk_on_suboid(hostname, ip, oid):
-    portspec = snmp_port_spec(hostname)
-    command = snmp_walk_command(hostname) + \
-             " -OQ -OU -On -Ot %s%s %s 2>/dev/null" % (ip, portspec, oid)
-    if opt_debug:
-        sys.stderr.write('   Running %s\n' % (command,))
-    snmp_process = os.popen(command, "r").xreadlines()
-
-    # Ugly(1): in some cases snmpwalk inserts line feed within one
-    # dataset. This happens for example on hexdump outputs longer
-    # than a few bytes. Those dumps are enclosed in double quotes.
-    # So if the value begins with a double quote, but the line
-    # does not end with a double quote, we take the next line(s) as
-    # a continuation line.
-    rowinfo = []
-    try:
-        while True: # walk through all lines
-            line = snmp_process.next().strip()
-            parts = line.split('=', 1)
-            if len(parts) < 2:
-                continue # broken line, must contain =
-            oid = parts[0].strip()
-            value = parts[1].strip()
-            # Filter out silly error messages from snmpwalk >:-P
-            if value.startswith('No more variables') or value.startswith('End of MIB') \
-               or value.startswith('No Such Object available') or value.startswith('No Such Instance currently exists'):
-                continue
-
-            if len(value) > 0 and value[0] == '"' and value[-1] != '"': # to be continued
-                while True: # scan for end of this dataset
-                    nextline = snmp_process.next().strip()
-                    value += " " + nextline
-                    if value[-1] == '"':
-                        break
-            rowinfo.append((oid, strip_snmp_value(value)))
-
-    except StopIteration:
-        pass
-
-    exitstatus = snmp_process.close()
-    if exitstatus:
-        if opt_verbose:
-            sys.stderr.write(tty_red + tty_bold + "ERROR: " + tty_normal + "SNMP error\n")
-	raise MKSNMPError("SNMP Error on %s" % ip)
-    return rowinfo
-
 def extract_end_oid(prefix, complete):
     return complete[len(prefix):].lstrip('.')
 
@@ -129,7 +86,7 @@ def oid_to_intlist(oid):
 def cmp_oids(o1, o2):
     return cmp(oid_to_intlist(o1), oid_to_intlist(o2))
 
-def get_snmp_table(hostname, ip, oid_info):
+def get_snmp_table(hostname, ip, check_type, oid_info):
     # oid_info is either ( oid, columns ) or
     # ( oid, suboids, columns )
     # suboids is a list if OID-infixes that are put between baseoid
@@ -172,18 +129,31 @@ def get_snmp_table(hostname, ip, oid_info):
             # in later. If the column in OID_STRING or OID_BIN we do something
             # similar: we fill in the complete OID of the entry, either as
             # string or as binary UTF-8 encoded number string
-            if column in [ OID_END, OID_STRING, OID_BIN ]:
+            if column in [ OID_END, OID_STRING, OID_BIN, OID_END_BIN, OID_END_OCTET_STRING ]:
+                if index_column >= 0 and index_column != colno:
+                    raise MKGeneralException("Invalid SNMP OID specification in implementation of check. "
+                        "You can only use one of OID_END, OID_STRING, OID_BIN, OID_END_BIN and OID_END_OCTET_STRING.")
                 index_column = colno
                 columns.append((fetchoid, []))
                 index_format = column
                 continue
 
-
             if opt_use_snmp_walk or is_usewalk_host(hostname):
                 rowinfo = get_stored_snmpwalk(hostname, fetchoid)
+            elif has_inline_snmp and use_inline_snmp:
+                rowinfo = inline_snmpwalk_on_suboid(hostname, check_type, fetchoid)
             else:
                 rowinfo = snmpwalk_on_suboid(hostname, ip, fetchoid)
 
+            # I've seen a broken device (Mikrotik Router), that broke after an
+            # update to RouterOS v6.22. It would return 9 time the same OID when
+            # .1.3.6.1.2.1.1.1.0 was being walked. We try to detect these situations
+            # by removing any duplicate OID information
+            if len(rowinfo) > 1 and rowinfo[0][0] == rowinfo[1][0]:
+                if opt_verbose:
+                    sys.stderr.write("Detected broken SNMP agent. Ignoring duplicate OID %s.\n" % rowinfo[0][0])
+                rowinfo = rowinfo[:1]
+
             columns.append((fetchoid, rowinfo))
             number_rows = len(rowinfo)
             if len(rowinfo) > max_len:
@@ -196,12 +166,16 @@ def get_snmp_table(hostname, ip, oid_info):
             fetchoid, max_column  = columns[max_len_col]
             for o, value in max_column:
                 if index_format == OID_END:
-		    eo = extract_end_oid(fetchoid, o)
-                    index_rows.append((o, eo))
+                    index_rows.append((o, extract_end_oid(fetchoid, o)))
                 elif index_format == OID_STRING:
                     index_rows.append((o, o))
-                else:
+                elif index_format == OID_BIN:
                     index_rows.append((o, oid_to_bin(o)))
+                elif index_format == OID_END_BIN:
+                    index_rows.append((o, oid_to_bin(extract_end_oid(fetchoid, o))))
+                else: # OID_END_OCTET_STRING:
+                    index_rows.append((o, oid_to_bin(extract_end_oid(fetchoid, o))[1:]))
+
             columns[index_column] = fetchoid, index_rows
 
 
@@ -333,121 +307,93 @@ def get_stored_snmpwalk(hostname, oid):
 
     path = snmpwalks_dir + "/" + hostname
 
-    if opt_debug:
-        sys.stderr.write("Getting %s from %s\n" % (oid, path))
-    if not os.path.exists(path):
-        raise MKGeneralException("No snmpwalk file %s\n" % path)
-
     rowinfo = []
 
-    use_new = True
-    if use_new:
-        # New implementation: use binary search
-        def to_bin_string(oid):
-            try:
-                return tuple(map(int, oid.strip(".").split(".")))
-            except:
-                raise MKGeneralException("Invalid OID %s" % oid)
+    # New implementation: use binary search
+    def to_bin_string(oid):
+        try:
+            return tuple(map(int, oid.strip(".").split(".")))
+        except:
+            raise MKGeneralException("Invalid OID %s" % oid)
 
-        def compare_oids(a, b):
-            aa = to_bin_string(a)
-            bb = to_bin_string(b)
-            if len(aa) <= len(bb) and bb[:len(aa)] == aa:
-                result = 0
-            else:
-                result = cmp(aa, bb)
-            return result
-
-        if hostname in g_walk_cache:
-            lines = g_walk_cache[hostname]
+    def compare_oids(a, b):
+        aa = to_bin_string(a)
+        bb = to_bin_string(b)
+        if len(aa) <= len(bb) and bb[:len(aa)] == aa:
+            result = 0
         else:
+            result = cmp(aa, bb)
+        return result
+
+    if hostname in g_walk_cache:
+        lines = g_walk_cache[hostname]
+    else:
+        try:
             lines = file(path).readlines()
-            g_walk_cache[hostname] = lines
+        except IOError:
+            raise MKGeneralException("No snmpwalk file %s\n" % path)
+        g_walk_cache[hostname] = lines
 
-        begin = 0
-        end = len(lines)
-        hit = None
-        while end - begin > 0:
-            current = (begin + end) / 2
-            parts = lines[current].split(None, 1)
-            comp = parts[0]
-            hit = compare_oids(oid_prefix, comp)
-            if hit == 0:
-                break
-            elif hit == 1: # we are too low
-                begin = current + 1
-            else:
-                end = current
-
-        if hit != 0:
-            return [] # not found
-
-
-        def collect_until(index, direction):
-            rows = []
-            # Handle case, where we run after the end of the lines list
-            if index >= len(lines):
-                if direction > 0:
-                    return []
-                else:
-                    index -= 1
-            while True:
-                line = lines[index]
-                parts = line.split(None, 1)
-                o = parts[0]
-                if o.startswith('.'):
-                    o = o[1:]
-                if o == oid or o.startswith(oid_prefix + "."):
-                    if len(parts) > 1:
-                        value = parts[1]
-                        if agent_simulator:
-                            value = agent_simulator_process(value)
-                    else:
-                        value = ""
-                    # Fix for missing starting oids
-                    rows.append(('.'+o, strip_snmp_value(value)))
-                    index += direction
-                    if index < 0 or index >= len(lines):
-                        break
-                else:
-                    break
-            return rows
-
-
-        rowinfo = collect_until(current, -1)
-        rowinfo.reverse()
-        rowinfo += collect_until(current + 1, 1)
-        # import pprint ; pprint.pprint(rowinfo)
-
-        if dot_star:
-            return [ rowinfo[0] ]
-        else:
-            return rowinfo
-
-
-
-    # Old implementation
-    hot = False
-    for line in file(path):
-        parts = line.split(None, 1)
-        o = parts[0]
-        if o.startswith('.'):
-            o = o[1:]
-        if o == oid or o.startswith(oid_prefix + "."):
-            hot = True
-            if len(parts) > 1:
-                value = parts[1]
-                if agent_simulator:
-                    value = agent_simulator_process(value)
-            else:
-                value = ""
-            rowinfo.append((o, strip_snmp_value(value))) # return pair of OID and value
-            if dot_star:
-                break
-        elif hot: # end of interesting part, no point in further search
+    begin = 0
+    end = len(lines)
+    hit = None
+    while end - begin > 0:
+        current = (begin + end) / 2
+        parts = lines[current].split(None, 1)
+        comp = parts[0]
+        hit = compare_oids(oid_prefix, comp)
+        if hit == 0:
             break
+        elif hit == 1: # we are too low
+            begin = current + 1
+        else:
+            end = current
+
+    if hit != 0:
+        return [] # not found
+
+
+    def collect_until(index, direction):
+        rows = []
+        # Handle case, where we run after the end of the lines list
+        if index >= len(lines):
+            if direction > 0:
+                return []
+            else:
+                index -= 1
+        while True:
+            line = lines[index]
+            parts = line.split(None, 1)
+            o = parts[0]
+            if o.startswith('.'):
+                o = o[1:]
+            if o == oid or o.startswith(oid_prefix + "."):
+                if len(parts) > 1:
+                    try:
+                        value = agent_simulator_process(parts[1])
+                    except:
+                        value = parts[1] # agent simulator missing in precompiled mode
+                else:
+                    value = ""
+                # Fix for missing starting oids
+                rows.append(('.'+o, strip_snmp_value(value)))
+                index += direction
+                if index < 0 or index >= len(lines):
+                    break
+            else:
+                break
+        return rows
+
+
+    rowinfo = collect_until(current, -1)
+    rowinfo.reverse()
+    rowinfo += collect_until(current + 1, 1)
     # import pprint ; pprint.pprint(rowinfo)
-    return rowinfo
+
+    if dot_star:
+        return [ rowinfo[0] ]
+    else:
+        return rowinfo
 
 # Helper function to be used in checks.  It applies a user-specified
 # character encoding in order to tranlate e.g. latin1 to utf8
@@ -456,5 +402,72 @@ def snmp_decode_string(text):
     if encoding:
         return text.decode(encoding).encode("utf-8")
     else:
-        return text
+        # Try to determine the current string encoding. In case a UTF-8
+        # decoding fails, we decode latin1 and encode it as UTF-8 again.
+        # When UTF-8 decoding works, we assume the string is already
+        # encoded in UTF-8 as we expect it to be.
+        try:
+            text.decode('utf-8')
+            return text
+        except:
+            return text.decode('latin1').encode('utf-8')
 
+#   .--Classic SNMP--------------------------------------------------------.
+#   |        ____ _               _        ____  _   _ __  __ ____         |
+#   |       / ___| | __ _ ___ ___(_) ___  / ___|| \ | |  \/  |  _ \        |
+#   |      | |   | |/ _` / __/ __| |/ __| \___ \|  \| | |\/| | |_) |       |
+#   |      | |___| | (_| \__ \__ \ | (__   ___) | |\  | |  | |  __/        |
+#   |       \____|_|\__,_|___/___/_|\___| |____/|_| \_|_|  |_|_|           |
+#   |                                                                      |
+#   +----------------------------------------------------------------------+
+#   | Non-inline SNMP handling code. Kept for compatibility.               |
+#   '----------------------------------------------------------------------'
+
+def snmpwalk_on_suboid(hostname, ip, oid, hex_plain = False):
+    portspec = snmp_port_spec(hostname)
+    command = snmp_walk_command(hostname) + \
+             " -OQ -OU -On -Ot %s%s %s" % (ip, portspec, oid)
+    if opt_debug:
+        sys.stderr.write('   Running %s\n' % (command,))
+
+    snmp_process = subprocess.Popen(command, shell = True, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
+
+    # Ugly(1): in some cases snmpwalk inserts line feed within one
+    # dataset. This happens for example on hexdump outputs longer
+    # than a few bytes. Those dumps are enclosed in double quotes.
+    # So if the value begins with a double quote, but the line
+    # does not end with a double quote, we take the next line(s) as
+    # a continuation line.
+    rowinfo = []
+    try:
+        line_iter = snmp_process.stdout.xreadlines()
+        while True:
+            line = line_iter.next().strip()
+            parts = line.split('=', 1)
+            if len(parts) < 2:
+                continue # broken line, must contain =
+            oid = parts[0].strip()
+            value = parts[1].strip()
+            # Filter out silly error messages from snmpwalk >:-P
+            if value.startswith('No more variables') or value.startswith('End of MIB') \
+               or value.startswith('No Such Object available') or value.startswith('No Such Instance currently exists'):
+                continue
+
+            if value == '"' or (len(value) > 1 and value[0] == '"' and (value[-1] != '"')): # to be continued
+                while True: # scan for end of this dataset
+                    nextline = line_iter.next().strip()
+                    value += " " + nextline
+                    if value[-1] == '"':
+                        break
+            rowinfo.append((oid, strip_snmp_value(value, hex_plain)))
+
+    except StopIteration:
+        pass
+
+    error = snmp_process.stderr.read()
+    exitstatus = snmp_process.wait()
+    if exitstatus:
+        if opt_verbose:
+            sys.stderr.write(tty_red + tty_bold + "ERROR: " + tty_normal + "SNMP error: %s\n" % error.strip())
+        raise MKSNMPError("SNMP Error on %s: %s (Exit-Code: %d)" % (ip, error.strip(), exitstatus))
+    return rowinfo
