@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -41,8 +41,6 @@ public:
     void *getNagiosObject(char *name); // return pointer to contact group
     bool isNagiosMember(void *data, void *nagobject);
     bool isEmpty(void *data);
-private:
-    contactgroupsmember *getData(void *);
 };
 
 
