@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -33,23 +33,29 @@
 
 using namespace std;
 class Query;
+class Column;
 
 class Filter
 {
     string _error_message; // Error in constructor
     unsigned _error_code;
+    Column *_column;
 
 protected:
     Query *_query; // needed by TimeOffsetFilter (currently)
     void setError(unsigned code, const char *format, ...);
 
 public:
-    Filter() : _query(0) {}
+    Filter() : _query(0), _column(0) {}
     virtual ~Filter() {}
+    virtual bool isAndingFilter() { return false; }
+    virtual bool isNegatingFilter() { return false; }
     string errorMessage() { return _error_message; }
     unsigned errorCode() { return _error_code; }
     bool hasError() { return _error_message != ""; }
     void setQuery(Query *q) { _query = q; }
+    void setColumn(Column *c) { _column = c; }
+    Column *column() { return _column; }
     virtual bool accepts(void *data) = 0;
     virtual void *indexFilter(const char *columnname __attribute__ ((__unused__))) { return 0; }
     virtual void findIntLimits(const char *columnname __attribute__ ((__unused__)), int *lower __attribute__ ((__unused__)), int *upper __attribute__ ((__unused__))) {}
