@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -37,28 +37,12 @@ using namespace std;
 class LogEntry;
 class Query;
 class LogCache;
+class World;
 
 typedef map<uint64_t, LogEntry *> logfile_entries_t; // key is time_t . lineno
 
 class Logfile
 {
-public:
-	Logfile(const char *path, bool watch);
-    ~Logfile();
-
-    char *path() { return _path; }
-    void load(LogCache *LogCache, time_t since, time_t until, unsigned logclasses);
-    void flush();
-    time_t since() { return _since; }
-    unsigned classesRead() { return _logclasses_read; }
-    long numEntries() { return _entries.size(); }
-    logfile_entries_t* getEntriesFromQuery(Query *query, LogCache *lc, time_t since, time_t until, unsigned);
-    bool answerQuery(Query *query, LogCache *lc, time_t since, time_t until, unsigned);
-    bool answerQueryReverse(Query *query, LogCache *lc, time_t since, time_t until, unsigned);
-
-    long freeMessages(unsigned logclasses);
-
-    unsigned   _logclasses_read; // only these types have been read
 private:
     char      *_path;
     time_t     _since;         // time of first entry
@@ -69,6 +53,28 @@ private:
 
     logfile_entries_t  _entries;
     char       _linebuffer[MAX_LOGLINE];
+    World     *_world;         // CMC: world our references point into
+
+
+public:
+    Logfile(const char *path, bool watch);
+    ~Logfile();
+
+    char *path() { return _path; }
+    char *readIntoBuffer(int *size);
+    void load(LogCache *LogCache, time_t since, time_t until, unsigned logclasses);
+    void flush();
+    time_t since() { return _since; }
+    unsigned classesRead() { return _logclasses_read; }
+    long numEntries() { return _entries.size(); }
+    logfile_entries_t* getEntriesFromQuery(Query *query, LogCache *lc, time_t since, time_t until, unsigned);
+    bool answerQuery(Query *query, LogCache *lc, time_t since, time_t until, unsigned);
+    bool answerQueryReverse(Query *query, LogCache *lc, time_t since, time_t until, unsigned);
+
+    long freeMessages(unsigned logclasses);
+    void updateReferences();
+
+    unsigned   _logclasses_read; // only these types have been read
 
 
 private:
