@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -34,6 +34,8 @@ class NegatingFilter : public Filter
 public:
     NegatingFilter(Filter *filter) : _filter(filter) {}
     ~NegatingFilter() { delete _filter; }
+    bool isNegatingFilter() { return true; }
+    Filter *subfilter() { return _filter; }
     bool accepts(void *data) { return !_filter->accepts(data); }
 };
 
