@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -23,6 +23,9 @@
 // Boston, MA 02110-1301 USA.
 
 #include <string.h>
+#include <math.h>
+#include <stdlib.h>
+
 #include "strutil.h"
 #include "logger.h"
 #include "Query.h"
