@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -35,7 +35,7 @@
 #include "Query.h"
 #include "Filter.h"
 #include "Column.h"
-#include "EmptyColumn.h"
+#include "NullColumn.h"
 #include "OutputBuffer.h"
 #include "InputBuffer.h"
 #include "StatsColumn.h"
@@ -65,6 +65,8 @@ Query::Query(InputBuffer *input, OutputBuffer *output, Table *table) :
     _need_ds_separator(false),
     _output_format(OUTPUT_FORMAT_CSV),
     _limit(-1),
+    _time_limit(-1),
+    _time_limit_timeout(0),
     _current_line(0),
     _timezone_offset(0)
 {
@@ -111,6 +113,9 @@ Query::Query(InputBuffer *input, OutputBuffer *output, Table *table) :
         else if (!strncmp(buffer, "Limit:", 6))
             parseLimitLine(lstrip(buffer + 6));
 
+        else if (!strncmp(buffer, "Timelimit:", 10))
+            parseTimelimitLine(lstrip(buffer + 10));
+
         else if (!strncmp(buffer, "AuthUser:", 9))
             parseAuthUserHeader(lstrip(buffer + 9));
 
@@ -182,7 +187,7 @@ Query::~Query()
 
 Column *Query::createDummyColumn(const char *name)
 {
-    Column *col = new EmptyColumn(name, "Dummy column");
+    Column *col = new NullColumn(name, "Non existing column");
     _dummy_columns.push_back(col);
     return col;
 }
@@ -193,8 +198,9 @@ void Query::addColumn(Column *column)
     _columns.push_back(column);
 }
 
-void Query::setError(int error_code, const char* msg){
-	_output->setError(error_code, msg);
+void Query::setError(int error_code, const char* msg)
+{
+    _output->setError(error_code, msg);
 }
 
 bool Query::hasNoColumns()
@@ -246,8 +252,10 @@ Filter *Query::createFilter(Column *column, int operator_id, char *value)
         delete filter;
         filter = 0;
     }
-    else
+    else {
         filter->setQuery(this);
+        filter->setColumn(column);
+    }
     return filter;
 }
 
@@ -255,6 +263,13 @@ Filter *Query::createFilter(Column *column, int operator_id, char *value)
 void Query::parseAndOrLine(char *line, int andor, bool filter)
 {
     char *value = next_field(&line);
+    if (!value) {
+        _output->setError(RESPONSE_CODE_INVALID_HEADER, "Missing value for %s%s: need non-zero integer number",
+                filter ? "" : "WaitCondition",
+                andor == ANDOR_OR ? "Or" : "And");
+        return;
+    }
+
     int number = atoi(value);
     if (!isdigit(value[0]) || number <= 0) {
         _output->setError(RESPONSE_CODE_INVALID_HEADER, "Invalid value for %s%s: need non-zero integer number",
@@ -303,6 +318,11 @@ void Query::parseNegateLine(char *line, bool filter)
 void Query::parseStatsAndOrLine(char *line, int andor)
 {
     char *value = next_field(&line);
+    if (!value) {
+        _output->setError(RESPONSE_CODE_INVALID_HEADER, "Missing value for Stats%s: need non-zero integer number",
+                andor == ANDOR_OR ? "Or" : "And");
+    }
+
     int number = atoi(value);
     if (!isdigit(value[0]) || number <= 0) {
         _output->setError(RESPONSE_CODE_INVALID_HEADER, "Invalid value for Stats%s: need non-zero integer number",
@@ -511,8 +531,12 @@ void Query::parseColumnsLine(char *line)
         if (column)
             _columns.push_back(column);
         else {
-            _output->setError(RESPONSE_CODE_INVALID_HEADER,
-                   "Table '%s' has no column '%s'", _table->name(), column_name);
+            logger(LOG_WARNING, "Replacing non-existing column '%s' with null column", column_name);
+            // Do not fail any longer. We might want to make this configurable.
+            // But not failing has the advantage that an updated GUI, that expects new columns,
+            // will be able to keep compatibility with older Livestatus versions.
+            // _output->setError(RESPONSE_CODE_INVALID_HEADER,
+            //       "Table '%s' has no column '%s'", _table->name(), column_name);
             Column *col = createDummyColumn(column_name);
             _columns.push_back(col);
         }
@@ -551,6 +575,12 @@ void Query::parseSeparatorsLine(char *line)
 void Query::parseOutputFormatLine(char *line)
 {
     char *format = next_field(&line);
+    if (!format) {
+        _output->setError(RESPONSE_CODE_INVALID_HEADER,
+               "Missing output format. Only 'csv' and 'json' are available.");
+        return;
+    }
+
     if (!strcmp(format, "csv"))
         _output_format = OUTPUT_FORMAT_CSV;
     else if (!strcmp(format, "json"))
@@ -565,6 +595,12 @@ void Query::parseOutputFormatLine(char *line)
 void Query::parseColumnHeadersLine(char *line)
 {
     char *value = next_field(&line);
+    if (!value) {
+        _output->setError(RESPONSE_CODE_INVALID_HEADER,
+              "Missing value for ColumnHeaders: must be 'on' or 'off'");
+        return;
+    }
+
     if (!strcmp(value, "on"))
         _show_column_headers = true;
     else if (!strcmp(value, "off"))
@@ -577,6 +613,12 @@ void Query::parseColumnHeadersLine(char *line)
 void Query::parseKeepAliveLine(char *line)
 {
     char *value = next_field(&line);
+    if (!value) {
+        _output->setError(RESPONSE_CODE_INVALID_HEADER,
+                "Missing value for KeepAlive: must be 'on' or 'off'");
+        return;
+    }
+
     if (!strcmp(value, "on"))
         _output->setDoKeepalive(true);
     else if (!strcmp(value, "off"))
@@ -589,6 +631,12 @@ void Query::parseKeepAliveLine(char *line)
 void Query::parseResponseHeaderLine(char *line)
 {
     char *value = next_field(&line);
+    if (!value) {
+        _output->setError(RESPONSE_CODE_INVALID_HEADER,
+                "Missing value for ResponseHeader: must be 'off' or 'fixed16'");
+        return;
+    }
+
     if (!strcmp(value, "off"))
         _output->setResponseHeader(RESPONSE_HEADER_OFF);
     else if (!strcmp(value, "fixed16"))
@@ -614,6 +662,26 @@ void Query::parseLimitLine(char *line)
     }
 }
 
+
+void Query::parseTimelimitLine(char *line)
+{
+    char *value = next_field(&line);
+    if (!value) {
+        _output->setError(RESPONSE_CODE_INVALID_HEADER, "Header Timelimit: missing value");
+    }
+    else {
+        int timelimit = atoi(value);
+        if (!isdigit(value[0]) || timelimit < 0)
+            _output->setError(RESPONSE_CODE_INVALID_HEADER,
+                    "Invalid value for Timelimit: must be non-negative integer (seconds)");
+        else {
+            _time_limit = timelimit;
+            _time_limit_timeout = time(0) + _time_limit;
+        }
+    }
+}
+
+
 void Query::parseWaitTimeoutLine(char *line)
 {
     char *value = next_field(&line);
@@ -764,6 +832,17 @@ void Query::start()
 }
 
 
+bool Query::timelimitReached()
+{
+    if (_time_limit >= 0 && time(0) >= _time_limit_timeout) {
+        logger(LG_INFO, "Maximum query time of %d seconds exceeded!", _time_limit);
+        _output->setError(RESPONSE_CODE_LIMIT_EXCEEDED, "Maximum query time of %d seconds exceeded!", _time_limit);
+        return true;
+    }
+    else
+        return false;
+}
+
 bool Query::processDataset(void *data)
 {
     if (_output->size() > g_max_response_size) {
@@ -779,6 +858,11 @@ bool Query::processDataset(void *data)
         if (_limit >= 0 && (int)_current_line > _limit)
             return false;
 
+        // When we reach the time limit we let the query fail. Otherwise the user will
+        // not know that the answer is incomplete.
+        if (timelimitReached())
+            return false;
+
         if (doStats())
         {
             Aggregator **aggr;
@@ -832,9 +916,10 @@ void Query::finish()
                 it != _stats_groups.end();
                 ++it)
         {
-            // dataset separator after first group
-            if (it != _stats_groups.begin() && _output_format != OUTPUT_FORMAT_CSV)
+            if (_need_ds_separator && _output_format != OUTPUT_FORMAT_CSV)
                 _output->addBuffer(",\n", 2);
+            else
+                _need_ds_separator = true;
 
             outputDatasetBegin();
 
@@ -865,6 +950,10 @@ void Query::finish()
 
     // stats without group column
     else if (doStats()) {
+        if (_need_ds_separator && _output_format != OUTPUT_FORMAT_CSV)
+            _output->addBuffer(",\n", 2);
+        else
+            _need_ds_separator = true;
         outputDatasetBegin();
         for (unsigned i=0; i<_stats_columns.size(); i++)
         {
@@ -963,6 +1052,24 @@ void Query::outputDouble(double value)
     _output->addBuffer(buf, l);
 }
 
+void Query::outputNull()
+{
+    if (_output_format == OUTPUT_FORMAT_CSV) {
+        // output empty cell
+    }
+    else if (_output_format == OUTPUT_FORMAT_PYTHON)
+        _output->addBuffer("None", 4);
+    else
+        _output->addBuffer("null", 4); // JSON
+}
+
+void Query::outputAsciiEscape(char value)
+{
+    char buf[8];
+    snprintf(buf, sizeof(buf), "\\%03o", value);
+    _output->addBuffer(buf, 4);
+}
+
 void Query::outputUnicodeEscape(unsigned value)
 {
     char buf[8];
@@ -986,7 +1093,17 @@ void Query::outputHostService(const char *host_name, const char *service_descrip
     }
 }
 
-void Query::outputString(const char *value)
+void Query::outputBlob(const char *buffer, int size)
+{
+    if (_output_format != OUTPUT_FORMAT_CSV)
+        outputString(buffer, size);
+    else
+        _output->addBuffer(buffer, size);
+}
+
+// len = -1 -> use strlen(), len >= 0: consider
+// output as blob, do not handle UTF-8.
+void Query::outputString(const char *value, int len)
 {
     if (!value) {
         if (_output_format != OUTPUT_FORMAT_CSV)
@@ -999,18 +1116,22 @@ void Query::outputString(const char *value)
 
     else // JSON
     {
-        if (_output_format == OUTPUT_FORMAT_PYTHON)
+        if (_output_format == OUTPUT_FORMAT_PYTHON && len < 0)
             _output->addChar('u'); // mark strings as unicode
         _output->addChar('"');
         const char *r = value;
-        int chars_left = strlen(r);
+        int chars_left = len >= 0 ? len : strlen(r);
         while (*r) {
             // Always escape control characters (1..31)
-            if (*r < 32 && *r >= 0)
-                outputUnicodeEscape((unsigned)*r);
+            if (*r < 32 && *r >= 0) {
+                if (len < 0)
+                    outputUnicodeEscape((unsigned)*r);
+                else
+                    outputAsciiEscape(*r);
+            }
 
             // Output ASCII characters unencoded
-            else if (*r >= 32) {
+            else if (*r >= 32 || len >= 0) {
                 if (*r == '"' || *r == '\\')
                     _output->addChar('\\');
                 _output->addChar(*r);
