@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -32,6 +32,7 @@
 #include "OffsetTimeColumn.h"
 #include "OffsetDoubleColumn.h"
 #include "OffsetTimeperiodColumn.h"
+#include "CustomTimeperiodColumn.h"
 #include "OffsetStringHostMacroColumn.h"
 #include "AttributelistColumn.h"
 #include "HostContactsColumn.h"
@@ -39,16 +40,19 @@
 #include "CustomVarsColumn.h"
 #include "CustomVarsExplicitColumn.h"
 #include "HostlistColumn.h"
+#include "HostFileColumn.h"
 #include "ServicelistColumn.h"
 #include "ServicelistStateColumn.h"
 #include "HostgroupsColumn.h"
 #include "ContactgroupsColumn.h"
 #include "HostSpecialIntColumn.h"
+#include "HostSpecialDoubleColumn.h"
 #include "tables.h"
 #include "auth.h"
 
 extern host *host_list;
 extern hostgroup *hostgroup_list;
+extern char g_mk_inventory_path[];
 
 struct hostbygroup {
     host       _host;
@@ -87,16 +91,25 @@ void TableHosts::addColumns(Table *table, string prefix, int indirect_offset)
                 "An alias name for the host", (char *)(&hst.alias) - ref, indirect_offset));
     table->addColumn(new OffsetStringColumn(prefix + "address",
                 "IP address", (char *)(&hst.address) - ref, indirect_offset));
+#ifdef NAGIOS4
+    table->addColumn(new OffsetStringColumn(prefix + "check_command",
+                "Nagios command for active host check of this host", (char *)(&hst.check_command) - ref, indirect_offset));
+    table->addColumn(new OffsetStringHostMacroColumn(prefix + "check_command_expanded",
+                "Nagios command for active host check of this host with the macros expanded", (char *)(&hst.check_command) - ref, indirect_offset));
+#else
     table->addColumn(new OffsetStringColumn(prefix + "check_command",
                 "Nagios command for active host check of this host", (char *)(&hst.host_check_command) - ref, indirect_offset));
     table->addColumn(new OffsetStringHostMacroColumn(prefix + "check_command_expanded",
                 "Nagios command for active host check of this host with the macros expanded", (char *)(&hst.host_check_command) - ref, indirect_offset));
+#endif
     table->addColumn(new OffsetStringColumn(prefix + "event_handler",
                 "Nagios command used as event handler", (char *)(&hst.event_handler) - ref, indirect_offset));
     table->addColumn(new OffsetStringColumn(prefix + "notification_period",
                 "Time period in which problems of this host will be notified. If empty then notification will be always", (char *)(&hst.notification_period) - ref, indirect_offset));
     table->addColumn(new OffsetStringColumn(prefix + "check_period",
                 "Time period in which this host will be checked. If empty then the host will always be checked.", (char *)(&hst.check_period) - ref, indirect_offset));
+    table->addColumn(new CustomVarsExplicitColumn(prefix + "service_period",
+                "The name of the service period of the host", (char *)(&hst.custom_variables) - ref, indirect_offset, "SERVICE_PERIOD"));
     table->addColumn(new OffsetStringColumn(prefix + "notes",
                 "Optional notes for this host", (char *)(&hst.notes) - ref, indirect_offset));
     table->addColumn(new OffsetStringHostMacroColumn(prefix + "notes_expanded",
@@ -134,8 +147,13 @@ void TableHosts::addColumns(Table *table, string prefix, int indirect_offset)
                 "Whether freshness checks are activated (0/1)", (char *)(&hst.check_freshness) - ref, indirect_offset));
     table->addColumn(new OffsetIntColumn(prefix + "process_performance_data",
                 "Whether processing of performance data is enabled (0/1)", (char *)(&hst.process_performance_data) - ref, indirect_offset));
+    #ifndef NAGIOS4
     table->addColumn(new OffsetIntColumn(prefix + "accept_passive_checks",
                 "Whether passive host checks are accepted (0/1)", (char *)(&hst.accept_passive_host_checks) - ref, indirect_offset));
+    #else
+    table->addColumn(new OffsetIntColumn(prefix + "accept_passive_checks",
+                "Whether passive host checks are accepted (0/1)", (char *)(&hst.accept_passive_checks) - ref, indirect_offset));
+    #endif // NAGIOS4
     table->addColumn(new OffsetIntColumn(prefix + "event_handler_enabled",
                 "Whether event handling is enabled (0/1)", (char *)(&hst.event_handler_enabled) - ref, indirect_offset));
     table->addColumn(new OffsetIntColumn(prefix + "acknowledgement_type",
@@ -148,10 +166,17 @@ void TableHosts::addColumns(Table *table, string prefix, int indirect_offset)
                 "Last hard state", (char *)(&hst.last_hard_state) - ref, indirect_offset));
     table->addColumn(new OffsetIntColumn(prefix + "current_attempt",
                 "Number of the current check attempts", (char *)(&hst.current_attempt) - ref, indirect_offset));
+    #ifndef NAGIOS4
     table->addColumn(new OffsetTimeColumn(prefix + "last_notification",
                 "Time of the last notification (Unix timestamp)", (char *)(&hst.last_host_notification) - ref, indirect_offset));
     table->addColumn(new OffsetTimeColumn(prefix + "next_notification",
                 "Time of the next notification (Unix timestamp)", (char *)(&hst.next_host_notification) - ref, indirect_offset));
+    #else
+    table->addColumn(new OffsetTimeColumn(prefix + "last_notification",
+                "Time of the last notification (Unix timestamp)", (char *)(&hst.last_notification) - ref, indirect_offset));
+    table->addColumn(new OffsetTimeColumn(prefix + "next_notification",
+                "Time of the next notification (Unix timestamp)", (char *)(&hst.next_notification) - ref, indirect_offset));
+    #endif // NAGIOS4
     table->addColumn(new OffsetTimeColumn(prefix + "next_check",
                 "Scheduled time for the next check (Unix timestamp)", (char *)(&hst.next_check) - ref, indirect_offset));
     table->addColumn(new OffsetTimeColumn(prefix + "last_hard_state_change",
@@ -164,6 +189,7 @@ void TableHosts::addColumns(Table *table, string prefix, int indirect_offset)
                 "Whether a flex downtime is pending (0/1)", (char *)(&hst.pending_flex_downtime) - ref, indirect_offset));
     table->addColumn(new OffsetIntColumn(prefix + "total_services",
                 "The total number of services of the host", (char *)(&hst.total_services) - ref, indirect_offset));
+    // Note: this is redundant with "active_checks_enabled". Nobody noted this before...
     table->addColumn(new OffsetIntColumn(prefix + "checks_enabled",
                 "Whether checks of the host are enabled (0/1)", (char *)(&hst.checks_enabled) - ref, indirect_offset));
     table->addColumn(new OffsetIntColumn(prefix + "notifications_enabled",
@@ -200,8 +226,13 @@ void TableHosts::addColumns(Table *table, string prefix, int indirect_offset)
                 "Whether active checks are enabled for the host (0/1)", (char *)(&hst.checks_enabled) - ref, indirect_offset));
     table->addColumn(new OffsetIntColumn(prefix + "check_options",
                 "The current check option, forced, normal, freshness... (0-2)", (char *)(&hst.check_options) - ref, indirect_offset));
+    #ifndef NAGIOS4
     table->addColumn(new OffsetIntColumn(prefix + "obsess_over_host",
                 "The current obsess_over_host setting... (0/1)", (char *)(&hst.obsess_over_host) - ref, indirect_offset));
+    #else
+    table->addColumn(new OffsetIntColumn(prefix + "obsess_over_host",
+                "The current obsess_over_host setting... (0/1)", (char *)(&hst.obsess) - ref, indirect_offset));
+    #endif // NAGIOS4
     table->addColumn(new AttributelistColumn(prefix + "modified_attributes",
                 "A bitmask specifying which attributes have been modified", (char *)(&hst.modified_attributes) - ref, indirect_offset, false));
     table->addColumn(new AttributelistColumn(prefix + "modified_attributes_list",
@@ -237,6 +268,8 @@ void TableHosts::addColumns(Table *table, string prefix, int indirect_offset)
                 "Whether this host is currently in its notification period (0/1)", (char *)(&hst.notification_period_ptr) - ref, indirect_offset));
     table->addColumn(new OffsetTimeperiodColumn(prefix + "in_check_period",
                 "Whether this host is currently in its check period (0/1)", (char *)(&hst.check_period_ptr) - ref, indirect_offset));
+    table->addColumn(new CustomTimeperiodColumn(prefix + "in_service_period",
+                "Whether this host is currently in its service period (0/1)", (char *)(&hst.custom_variables) - ref, indirect_offset, "SERVICE_PERIOD"));
 
     table->addColumn(new HostContactsColumn(prefix + "contacts",
                 "A list of all contacts of this host, either direct or via a contact group", indirect_offset));
@@ -298,7 +331,16 @@ void TableHosts::addColumns(Table *table, string prefix, int indirect_offset)
     table->addColumn(new HostSpecialIntColumn(prefix + "hard_state",
                 "The effective hard state of the host (eliminates a problem in hard_state)", HSIC_REAL_HARD_STATE, indirect_offset));
     table->addColumn(new HostSpecialIntColumn(prefix + "pnpgraph_present",
-                "Whether there is a PNP4Nagios graph present for this host (0/1)", HSIC_PNP_GRAPH_PRESENT, indirect_offset));
+                "Whether there is a PNP4Nagios graph present for this host (-1/0/1)", HSIC_PNP_GRAPH_PRESENT, indirect_offset));
+    table->addColumn(new HostSpecialIntColumn(prefix + "mk_inventory_last",
+                "The timestamp of the last Check_MK HW/SW-Inventory for this host. 0 means that no inventory data is present", HSIC_MK_INVENTORY_LAST, indirect_offset));
+    table->addColumn(new HostFileColumn(prefix + "mk_inventory",
+                "The file content content of the Check_MK HW/SW-Inventory", g_mk_inventory_path, "", indirect_offset));
+    table->addColumn(new HostFileColumn(prefix + "mk_inventory_gz",
+                "The gzipped file content content of the Check_MK HW/SW-Inventory", g_mk_inventory_path, ".gz", indirect_offset));
+
+    table->addColumn(new HostSpecialDoubleColumn(prefix + "staleness",
+                    "Staleness indicator for this host", HSDC_STALENESS, indirect_offset));
 
     table->addColumn(new HostgroupsColumn(prefix + "groups",
                 "A list of all host groups this host is in", (char *)(&hst.hostgroups_ptr) - ref, indirect_offset));
@@ -311,6 +353,9 @@ void TableHosts::addColumns(Table *table, string prefix, int indirect_offset)
                 "A list of all services of the host together with state and has_been_checked",    (char *)(&hst.services) - ref, indirect_offset, false, 1));
     table->addColumn(new ServicelistColumn(prefix + "services_with_info",
                 "A list of all services including detailed information about each service",    (char *)(&hst.services) - ref, indirect_offset, false, 2));
+    table->addColumn(new ServicelistColumn(prefix + "services_with_fullstate",
+                "A list of all services including full state information. The list of entries can grow in future versions.",
+                   (char *)(&hst.services) - ref, indirect_offset, false, 3));
 }
 
 void *TableHosts::findObject(char *objectspec)
@@ -325,14 +370,34 @@ void TableHosts::answerQuery(Query *query)
     if (_by_group) {
         hostgroup *hgroup = hostgroup_list;
         hostbygroup hg;
+        bool show_hgroup;
+
+        // When g_group_authorization is set to AUTH_STRICT we need to pre-check
+        // if every host of this group is visible to the _auth_user
+        bool requires_precheck = query->authUser() && g_group_authorization == AUTH_STRICT;
+
         while (hgroup) {
+            show_hgroup = true;
             hg._hostgroup = hgroup;
             hostsmember *mem = hgroup->members;
-            while (mem) {
-                memcpy(&hg._host, mem->host_ptr, sizeof(host));
-                if (!query->processDataset(&hg))
-                    break;
-                mem = mem->next;
+            if (requires_precheck) {
+                while (mem) {
+                    if (!is_authorized_for(query->authUser(), mem->host_ptr, 0)) {
+                        show_hgroup = false;
+                        break;
+                    }
+                    mem = mem->next;
+                }
+            }
+
+            if (show_hgroup) {
+                mem = hgroup->members;
+                while (mem) {
+                    memcpy(&hg._host, mem->host_ptr, sizeof(host));
+                    if (!query->processDataset(&hg))
+                        break;
+                    mem = mem->next;
+                }
             }
             hgroup = hgroup->next;
         }
