@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -42,7 +42,7 @@
 #include "nagios.h"
 #include "livestatus.h"
 #include "downtime.h"
-#include "store.h"
+#include "store_c.h"
 #include "logger.h"
 #include "config.h"
 #include "global_counters.h"
@@ -50,7 +50,6 @@
 #include "auth.h"
 #include "data_encoding.h"
 #include "waittriggers.h"
-#include "livechecking.h"
 
 
 #ifndef AF_LOCAL
@@ -65,8 +64,19 @@
 # define SUN_LEN(ptr) ((size_t) (((struct sockaddr_un *) 0)->sun_path) + strlen ((ptr)->sun_path))
 #endif
 
+void TRIGGER(int what)
+{
+    pthread_cond_broadcast(&g_wait_cond[WT_ALL]);
+    pthread_cond_broadcast(&g_wait_cond[what]);
+}
+
+
 NEB_API_VERSION(CURRENT_NEB_API_VERSION)
+#ifndef NAGIOS4
 extern int event_broker_options;
+#else
+extern unsigned long event_broker_options;
+#endif // NAGIOS4
 extern int enable_environment_macros;
 extern char *log_file;
 
@@ -77,6 +87,7 @@ int g_num_clientthreads = 10;     /* allow 10 concurrent connections per default
 int g_num_queued_connections = 0;     /* current number of queued connections (for statistics) */
 int g_num_active_connections = 0;     /* current number of active connections (for statistics) */
 size_t g_thread_stack_size = 65536; /* stack size of threads */
+extern int g_disable_statehist_filtering;
 
 #define false 0
 #define true 1
@@ -86,14 +97,15 @@ void *g_nagios_handle;
 int g_unix_socket = -1;
 int g_max_fd_ever = 0;
 char g_socket_path[4096];
-char g_pnp_path[4096];
-char g_livecheck_path[4096];
+char g_pnp_path[4096]; // base path for PNP4Nagios graphs
+char g_mk_inventory_path[4096]; // base path of Check_MK inventor files
 char g_logfile_path[4096];
 int g_debug_level = 0;
 int g_should_terminate = false;
 pthread_t g_mainthread_id;
 pthread_t *g_clientthread_id;
 unsigned long g_max_cached_messages = 500000;
+unsigned long g_max_lines_per_logfile = 1000000; // do never read more than that number of lines from a logfile
 unsigned long g_max_response_size = 100 * 1024 * 1024; // limit answer to 10 MB
 int g_thread_running = 0;
 int g_thread_pid = 0;
@@ -368,25 +380,18 @@ int broker_check(int event_type, void *data)
 {
     int result = NEB_OK;
     if (event_type == NEBCALLBACK_SERVICE_CHECK_DATA) {
-        if (g_livecheck_enabled)
-            result = broker_service_livecheck(event_type, data);
-
         nebstruct_service_check_data *c = (nebstruct_service_check_data *)data;
         if (c->type == NEBTYPE_SERVICECHECK_PROCESSED) {
             g_counters[COUNTER_SERVICE_CHECKS]++;
         }
     }
     else if (event_type == NEBCALLBACK_HOST_CHECK_DATA) {
-        if (g_livecheck_enabled)
-            result = broker_host_livecheck(event_type, data);
-
         nebstruct_host_check_data *c = (nebstruct_host_check_data *)data;
         if (c->type == NEBTYPE_HOSTCHECK_PROCESSED) {
             g_counters[COUNTER_HOST_CHECKS]++;
         }
     }
-    pthread_cond_broadcast(&g_wait_cond[WT_ALL]);
-    pthread_cond_broadcast(&g_wait_cond[WT_CHECK]);
+    TRIGGER(WT_CHECK);
     return result;
 }
 
@@ -396,8 +401,7 @@ int broker_comment(int event_type __attribute__ ((__unused__)), void *data)
     nebstruct_comment_data *co = (nebstruct_comment_data *)data;
     store_register_comment(co);
     g_counters[COUNTER_NEB_CALLBACKS]++;
-    pthread_cond_broadcast(&g_wait_cond[WT_ALL]);
-    pthread_cond_broadcast(&g_wait_cond[WT_COMMENT]);
+    TRIGGER(WT_COMMENT);
     return 0;
 }
 
@@ -406,8 +410,7 @@ int broker_downtime(int event_type __attribute__ ((__unused__)), void *data)
     nebstruct_downtime_data *dt = (nebstruct_downtime_data *)data;
     store_register_downtime(dt);
     g_counters[COUNTER_NEB_CALLBACKS]++;
-    pthread_cond_broadcast(&g_wait_cond[WT_ALL]);
-    pthread_cond_broadcast(&g_wait_cond[WT_DOWNTIME]);
+    TRIGGER(WT_DOWNTIME);
     return 0;
 }
 
@@ -416,8 +419,7 @@ int broker_log(int event_type __attribute__ ((__unused__)), void *data __attribu
 
     g_counters[COUNTER_NEB_CALLBACKS]++;
     g_counters[COUNTER_LOG_MESSAGES]++;
-    pthread_cond_broadcast(&g_wait_cond[WT_ALL]);
-    pthread_cond_broadcast(&g_wait_cond[WT_LOG]);
+    TRIGGER(WT_LOG);
     return 0;
 }
 
@@ -427,24 +429,21 @@ int broker_command(int event_type __attribute__ ((__unused__)), void *data)
     if (sc->type == NEBTYPE_EXTERNALCOMMAND_START)
         g_counters[COUNTER_COMMANDS]++;
     g_counters[COUNTER_NEB_CALLBACKS]++;
-    pthread_cond_broadcast(&g_wait_cond[WT_ALL]);
-    pthread_cond_broadcast(&g_wait_cond[WT_COMMAND]);
+    TRIGGER(WT_COMMAND);
     return 0;
 }
 
 int broker_state(int event_type __attribute__ ((__unused__)), void *data __attribute__ ((__unused__)))
 {
     g_counters[COUNTER_NEB_CALLBACKS]++;
-    pthread_cond_broadcast(&g_wait_cond[WT_ALL]);
-    pthread_cond_broadcast(&g_wait_cond[WT_STATE]);
+    TRIGGER(WT_STATE);
     return 0;
 }
 
 int broker_program(int event_type __attribute__ ((__unused__)), void *data __attribute__ ((__unused__)))
 {
     g_counters[COUNTER_NEB_CALLBACKS]++;
-    pthread_cond_broadcast(&g_wait_cond[WT_ALL]);
-    pthread_cond_broadcast(&g_wait_cond[WT_PROGRAM]);
+    TRIGGER(WT_PROGRAM);
     return 0;
 }
 
@@ -508,8 +507,8 @@ int broker_event(int event_type __attribute__ ((__unused__)), void *data)
     {
         if (g_thread_running == 1 )
             livestatus_log_initial_states();
-        else if (log_initial_states == 1) // initi
-            write_to_all_logs("logging intitial states", LG_INFO); // initial info during startup
+        else if (log_initial_states == 1)
+            write_to_all_logs("logging initial states", LG_INFO); // initial info during startup
     }
     update_timeperiods_cache(ts->timestamp.tv_sec);
     return 0;
@@ -522,7 +521,6 @@ int broker_process(int event_type __attribute__ ((__unused__)), void *data)
     struct nebstruct_process_struct *ps = (struct nebstruct_process_struct *)data;
     if (ps->type == NEBTYPE_PROCESS_EVENTLOOPSTART) {
         update_timeperiods_cache(time(0));
-        init_livecheck();
         start_threads();
     }
     return 0;
@@ -610,19 +608,18 @@ void deregister_callbacks()
     neb_deregister_callback(NEBCALLBACK_TIMED_EVENT_DATA,      broker_event);
 }
 
-
-void check_pnp_path()
+void check_path(const char *name, char *path)
 {
     struct stat st;
-    if (0 == stat(g_pnp_path, &st)) {
-        if (0 != access(g_pnp_path, R_OK)) {
-            logger(LG_ERR, "PNP perfdata directory '%s' not readable. Please fix permissions.", g_pnp_path);
-            g_pnp_path[0] = 0; // disable
+    if (0 == stat(path, &st)) {
+        if (0 != access(path, R_OK)) {
+            logger(LG_ERR, "%s '%s' not readable. Please fix permissions.", name, path);
+            path[0] = 0; // disable
         }
     }
     else {
-        logger(LG_ERR, "PNP perfdata directory '%s' not existing. Please check pnp_path option!", g_pnp_path);
-        g_pnp_path[0] = 0; // disable
+        logger(LG_ERR, "%s '%s' not existing!", name, path);
+        path[0] = 0; // disable
     }
 }
 
@@ -643,9 +640,6 @@ void livestatus_parse_arguments(const char *args_orig)
     /* there is no default PNP path */
     g_pnp_path[0] = 0;
 
-    /* also livecheck is disabled per default */
-    g_livecheck_path[0] = 0;
-
     if (!args_orig)
         return; // no arguments, use default options
 
@@ -672,6 +666,10 @@ void livestatus_parse_arguments(const char *args_orig)
                 g_max_cached_messages = strtoul(right, 0, 10);
                 logger(LG_INFO, "Setting max number of cached log messages to %lu", g_max_cached_messages);
             }
+            else if (!strcmp(left, "max_lines_per_logfile")) {
+                g_max_lines_per_logfile = strtoul(right, 0, 10);
+                logger(LG_INFO, "Setting max number lines per logfile to %lu", g_max_lines_per_logfile);
+            }
             else if (!strcmp(left, "thread_stack_size")) {
                 g_thread_stack_size = strtoul(right, 0, 10);
                 logger(LG_INFO, "Setting size of thread stacks to %lu", g_thread_stack_size);
@@ -736,7 +734,13 @@ void livestatus_parse_arguments(const char *args_orig)
                 strncpy(g_pnp_path, right, sizeof(g_pnp_path) - 1);
                 if (right[strlen(right) - 1] != '/')
                     strncat(g_pnp_path, "/",  sizeof(g_pnp_path) - strlen(g_pnp_path) - 1 ); // make sure, that trailing slash is always there
-                check_pnp_path();
+                check_path("PNP perfdata directory", g_pnp_path);
+            }
+            else if (!strcmp(left, "mk_inventory_path")) {
+                strncpy(g_mk_inventory_path, right, sizeof(g_mk_inventory_path) - 1);
+                if (right[strlen(right) - 1] != '/')
+                    strncat(g_mk_inventory_path, "/",  sizeof(g_mk_inventory_path) - strlen(g_mk_inventory_path) - 1 ); // make sure, that trailing slash is always there
+                check_path("Check_MK Inventory directory", g_mk_inventory_path);
             }
             else if (!strcmp(left, "data_encoding")) {
                 if (!strcmp(right, "utf8"))
@@ -750,15 +754,11 @@ void livestatus_parse_arguments(const char *args_orig)
                 }
             }
             else if (!strcmp(left, "livecheck")) {
-                strncpy(g_livecheck_path, right, sizeof(g_livecheck_path) - 1);
-                if (0 != access(g_livecheck_path, X_OK)) {
-                    logger(LG_INFO, "ERROR: %s is not executable. Disabling livecheck.", g_livecheck_path);
-                    g_livecheck_path[0] = 0;
-                }
-                g_livecheck_enabled = true;
+                logger(LG_INFO, "Livecheck has been removed from Livestatus. Sorry.");
             }
-            else if (!strcmp(left, "num_livecheck_helpers")) {
-                g_num_livehelpers = atoi(right);
+            else if (!strcmp(left, "disable_statehist_filtering"))
+            {
+                g_disable_statehist_filtering = atoi(right);
             }
             else {
                 logger(LG_INFO, "Ignoring invalid option %s=%s", left, right);
@@ -829,7 +829,6 @@ int nebmodule_deinit(int flags __attribute__ ((__unused__)), int reason __attrib
     close_unix_socket();
     store_deinit();
     deregister_callbacks();
-    deinit_livecheck();
     close_logfile();
     return 0;
 }
