@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -27,14 +27,27 @@
 
 #include "config.h"
 
-#define NSCORE
-#include "nagios/objects.h"
-#include "nagios/nagios.h"
-#include "nagios/nebstructs.h"
-#include "nagios/neberrors.h"
-#include "nagios/broker.h"
-#include "nagios/nebmodules.h"
-#include "nagios/nebcallbacks.h"
-
+#ifdef CMC
+#include "../cmc.h"
+#else
+    #define NSCORE
+    #ifdef NAGIOS4
+    #include "nagios4/objects.h"
+    #include "nagios4/nagios.h"
+    #include "nagios4/nebstructs.h"
+    #include "nagios4/neberrors.h"
+    #include "nagios4/broker.h"
+    #include "nagios4/nebmodules.h"
+    #include "nagios4/nebcallbacks.h"
+    #else
+    #include "nagios/objects.h"
+    #include "nagios/nagios.h"
+    #include "nagios/nebstructs.h"
+    #include "nagios/neberrors.h"
+    #include "nagios/broker.h"
+    #include "nagios/nebmodules.h"
+    #include "nagios/nebcallbacks.h"
+    #endif // NAGIOS4
+#endif // CMC
 #endif // nagios_h
 
