@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -38,7 +38,7 @@ void cleanup_pnpname(char *p)
     }
 }
 
-int pnpgraph_present(char *host, char *service)
+int pnpgraph_present(const char *host, const char *service)
 {
     if (!g_pnp_path[0])
         return -1;
