@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -25,7 +25,7 @@
 #ifndef pnp4nagios_h
 #define pnp4nagios_h
 
-int pnpgraph_present(char *host, char *service);
+int pnpgraph_present(const char *host, const char *service);
 
 #endif // pnp4nagios_h
 
