@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -40,6 +40,16 @@
 
 #define WT_ALLNAMES "all, check, state, log, downtime, comment, command and program"
 
+#ifdef __cplusplus
+extern "C"
+{
+#endif
+void trigger(int what);
+#ifdef __cplusplus
+}
+#endif
+
+
 extern const char *wt_names[];
 extern pthread_cond_t g_wait_cond[];
 extern pthread_mutex_t g_wait_mutex;
