@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -34,17 +34,35 @@ loaded_with_language = False
 
 # Custom user attributes
 user_attributes = {}
+builtin_user_attribute_names = []
 
-# Load all login plugins
+# Load all userdb plugins
 def load_plugins():
+    global user_attributes
+    global multisite_user_connectors
+    global builtin_user_attribute_names
+
+    # Do not cache the custom user attributes. They can be created by the user
+    # during runtime, means they need to be loaded during each page request.
+    # But delete the old definitions before to also apply removals of attributes
+    if user_attributes:
+        for attr_name in user_attributes.keys():
+            if attr_name not in builtin_user_attribute_names:
+                del user_attributes[attr_name]
+        declare_custom_user_attrs()
+
     global loaded_with_language
     if loaded_with_language == current_language:
         return
 
     # declare & initialize global vars
-    global multisite_user_connectors ; multisite_user_connectors = []
+    user_attributes = {}
+    multisite_user_connectors = []
 
     load_web_plugins("userdb", globals())
+    builtin_user_attribute_names = user_attributes.keys()
+    declare_custom_user_attrs()
+    hook_load()
 
     # This must be set after plugin loading to make broken plugins raise
     # exceptions all the time and not only the first time (when the plugins
@@ -104,7 +122,7 @@ def new_user_template(connector_id):
     return new_user
 
 def create_non_existing_user(connector_id, username):
-    users = load_users()
+    users = load_users(lock = True)
     if username in users:
         return # User exists. Nothing to do...
 
@@ -114,16 +132,67 @@ def create_non_existing_user(connector_id, username):
     # Call the sync function for this new user
     hook_sync(connector_id = connector_id, only_username = username)
 
+# FIXME: Can we improve this easily? Would be nice not to have to call "load_users".
+# Maybe a directory listing of profiles or a list of a small file would perform better
+# than having to load the users, contacts etc. during each http request to multisite
+def user_exists(username):
+    return username in load_users().keys()
+
 def user_locked(username):
     users = load_users()
     return users[username].get('locked', False)
 
+def update_user_access_time(username):
+    if not config.save_user_access_times:
+        return
+    save_custom_attr(username, 'last_seen', repr(time.time()))
 
+def on_succeeded_login(username):
+    num_failed = load_custom_attr(username, 'num_failed', saveint)
+    if num_failed != None and num_failed != 0:
+        save_custom_attr(username, 'num_failed', '0')
+
+    update_user_access_time(username)
+
+# userdb.need_to_change_pw returns either False or the reason description why the
+# password needs to be changed
+def need_to_change_pw(username):
+    if load_custom_attr(username, 'enforce_pw_change', saveint) == 1:
+        return 'enforced'
+
+    last_pw_change = load_custom_attr(username, 'last_pw_change', saveint)
+    max_pw_age = config.password_policy.get('max_age')
+    if max_pw_age:
+        if not last_pw_change:
+            # The age of the password is unknown. Assume the user has just set
+            # the password to have the first access after enabling password aging
+            # as starting point for the password period. This bewares all users
+            # from needing to set a new password after enabling aging.
+            save_custom_attr(username, 'last_pw_change', str(int(time.time())))
+            return False
+        elif time.time() - last_pw_change > max_pw_age:
+            return 'expired'
+    return False
+
+def on_failed_login(username):
+    users = load_users(lock = True)
+    if username in users:
+        if "num_failed" in users[username]:
+            users[username]["num_failed"] += 1
+        else:
+            users[username]["num_failed"] = 1
+
+        if config.lock_on_logon_failures:
+            if users[username]["num_failed"] >= config.lock_on_logon_failures:
+                users[username]["locked"] = True
+
+        save_users(users)
 
 root_dir      = defaults.check_mk_configdir + "/wato/"
 multisite_dir = defaults.default_config_dir + "/multisite.d/wato/"
 
-#   .--Users---------------------------------------------------------------.
+#.
+#   .-Users----------------------------------------------------------------.
 #   |                       _   _                                          |
 #   |                      | | | |___  ___ _ __ ___                        |
 #   |                      | | | / __|/ _ \ '__/ __|                       |
@@ -132,20 +201,39 @@ multisite_dir = defaults.default_config_dir + "/multisite.d/wato/"
 #   |                                                                      |
 #   +----------------------------------------------------------------------+
 
+def declare_user_attribute(name, vs, user_editable = True, permission = None,
+                           show_in_table = False, topic = None, add_custom_macro = False,
+                           domain = "multisite"):
+
+    user_attributes[name] = {
+        'valuespec'         : vs,
+        'user_editable'     : user_editable,
+        'show_in_table'     : show_in_table,
+        'topic'             : topic and topic or 'personal',
+        'add_custom_macro'  : add_custom_macro,
+        'domain'            : domain,
+    }
+
+    # Permission needed for editing this attribute
+    if permission:
+        user_attributes[name]["permission"] = permission
+
 def get_user_attributes():
     return user_attributes.items()
 
-def reset_user_attributes():
-    global user_attributes
-    user_attributes = {}
-
-def load_users():
+def load_users(lock = False):
     filename = root_dir + "contacts.mk"
 
-    # Make sure that the file exists without modifying it, *if* it exists.
-    # Note the lock will be released at end of page request automatically.
-    file(filename, "a")
-    aquire_lock(filename)
+    if lock:
+        # Make sure that the file exists without modifying it, *if* it exists
+        # to be able to lock and realease the file properly.
+        # Note: the lock will be released on next save_users() call or at
+        #       end of page request automatically.
+        file(filename, "a")
+        aquire_lock(filename)
+
+    if html.is_cached('users'):
+        return html.get_cached('users')
 
     # First load monitoring contacts from Check_MK's world. If this is
     # the first time, then the file will be empty, which is no problem.
@@ -154,6 +242,8 @@ def load_users():
         vars = { "contacts" : {} }
         execfile(filename, vars, vars)
         contacts = vars["contacts"]
+    except IOError:
+        contacts = {} # a not existing file is ok, start with empty data
     except Exception, e:
         if config.debug:
             raise MKGeneralException(_("Cannot read configuration file %s: %s" %
@@ -165,20 +255,19 @@ def load_users():
 
     # Now add information about users from the Web world
     filename = multisite_dir + "users.mk"
-    if os.path.exists(filename):
-        try:
-            vars = { "multisite_users" : {} }
-            execfile(filename, vars, vars)
-            users = vars["multisite_users"]
-        except Exception, e:
-            if config.debug:
-                raise MKGeneralException(_("Cannot read configuration file %s: %s" %
-                              (filename, e)))
-            else:
-                html.log('load_users: Problem while loading users (%s - %s). '
-                         'Initializing structure...' % (filename, e))
-            users = {}
-    else:
+    try:
+        vars = { "multisite_users" : {} }
+        execfile(filename, vars, vars)
+        users = vars["multisite_users"]
+    except IOError:
+        users = {} # not existing is ok -> empty structure
+    except Exception, e:
+        if config.debug:
+            raise MKGeneralException(_("Cannot read configuration file %s: %s" %
+                          (filename, e)))
+        else:
+            html.log('load_users: Problem while loading users (%s - %s). '
+                     'Initializing structure...' % (filename, e))
         users = {}
 
     # Merge them together. Monitoring users not known to Multisite
@@ -206,41 +295,45 @@ def load_users():
     # they are getting according to the multisite old-style
     # configuration variables.
 
+    def readlines(f):
+        try:
+            return file(f)
+        except IOError:
+            return []
+
     filename = defaults.htpasswd_file
-    if os.path.exists(filename):
-        for line in file(filename):
-            line = line.strip()
-            if ':' in line:
-                id, password = line.strip().split(":")[:2]
-                if password.startswith("!"):
-                    locked = True
-                    password = password[1:]
-                else:
-                    locked = False
-                if id in result:
-                    result[id]["password"] = password
-                    result[id]["locked"] = locked
-                else:
-                    # Create entry if this is an admin user
-                    new_user = {
-                        "roles"    : config.roles_of_user(id),
-                        "password" : password,
-                        "locked"   : False,
-                    }
-                    result[id] = new_user
-                # Make sure that the user has an alias
-                result[id].setdefault("alias", id)
-            # Other unknown entries will silently be dropped. Sorry...
+    for line in readlines(filename):
+        line = line.strip()
+        if ':' in line:
+            id, password = line.strip().split(":")[:2]
+            if password.startswith("!"):
+                locked = True
+                password = password[1:]
+            else:
+                locked = False
+            if id in result:
+                result[id]["password"] = password
+                result[id]["locked"] = locked
+            else:
+                # Create entry if this is an admin user
+                new_user = {
+                    "roles"    : config.roles_of_user(id),
+                    "password" : password,
+                    "locked"   : False,
+                }
+                result[id] = new_user
+            # Make sure that the user has an alias
+            result[id].setdefault("alias", id)
+        # Other unknown entries will silently be dropped. Sorry...
 
     # Now read the serials, only process for existing users
     serials_file = '%s/auth.serials' % os.path.dirname(defaults.htpasswd_file)
-    if os.path.exists(serials_file):
-        for line in file(serials_file):
-            line = line.strip()
-            if ':' in line:
-                user_id, serial = line.split(':')[:2]
-                if user_id in result:
-                    result[user_id]['serial'] = saveint(serial)
+    for line in readlines(serials_file):
+        line = line.strip()
+        if ':' in line:
+            user_id, serial = line.split(':')[:2]
+            if user_id in result:
+                result[user_id]['serial'] = saveint(serial)
 
     # Now read the user specific files
     dir = defaults.var_dir + "/web/"
@@ -248,11 +341,25 @@ def load_users():
         if d[0] != '.':
             id = d
 
+            # read special values from own files
+            if id in result:
+                for attr, conv_func in [
+                        ('num_failed',        saveint),
+                        ('last_pw_change',    saveint),
+                        ('last_seen',         savefloat),
+                        ('enforce_pw_change', lambda x: bool(saveint(x))),
+                    ]:
+                    val = load_custom_attr(id, attr, conv_func)
+                    if val != None:
+                        result[id][attr] = val
+
             # read automation secrets and add them to existing
             # users or create new users automatically
-            secret_file = dir + d + "/automation.secret"
-            if os.path.exists(secret_file):
-                secret = file(secret_file).read().strip()
+            try:
+                secret = file(dir + d + "/automation.secret").read().strip()
+            except IOError:
+                secret = None
+            if secret:
                 if id in result:
                     result[id]["automation_secret"] = secret
                 else:
@@ -261,14 +368,44 @@ def load_users():
                         "automation_secret" : secret,
                     }
 
+    # populate the users cache
+    html.set_cache('users', result)
+
     return result
 
+def load_custom_attr(userid, key, conv_func, default = None):
+    basedir = defaults.var_dir + "/web/" + userid
+    try:
+        return conv_func(file(basedir + '/' + key + '.mk').read().strip())
+    except IOError:
+        return default
+
+def save_custom_attr(userid, key, val):
+    basedir = defaults.var_dir + "/web/" + userid
+    make_nagios_directory(basedir)
+    create_user_file('%s/%s.mk' % (basedir, key), 'w').write('%s\n' % val)
+
+def get_online_user_ids():
+    online_threshold = time.time() - config.user_online_maxage
+    users = []
+    for user_id, user in load_users(lock = False).items():
+        if user.get('last_seen', 0) >= online_threshold:
+            users.append(user_id)
+    return users
+
 def split_dict(d, keylist, positive):
     return dict([(k,v) for (k,v) in d.items() if (k in keylist) == positive])
 
-
 def save_users(profiles):
-    custom_values = user_attributes.keys()
+
+    # Add custom macros
+    core_custom_macros =  [ k for k,o in user_attributes.items() if o.get('add_custom_macro') ]
+    for user in profiles.keys():
+        for macro in core_custom_macros:
+            if profiles[user].get(macro):
+                profiles[user]['_'+macro] = profiles[user][macro]
+
+    multisite_custom_values = [ k for k,v in user_attributes.items() if v["domain"] == "multisite" ]
 
     # Keys not to put into contact definitions for Check_MK
     non_contact_keys = [
@@ -279,7 +416,11 @@ def save_users(profiles):
         "language",
         "serial",
         "connector",
-    ] + custom_values
+        "num_failed",
+        "enforce_pw_change",
+        "last_pw_change",
+        "last_seen",
+    ] + multisite_custom_values
 
     # Keys to put into multisite configuration
     multisite_keys   = [
@@ -289,7 +430,7 @@ def save_users(profiles):
         "alias",
         "language",
         "connector",
-    ] + custom_values
+    ] + multisite_custom_values
 
     # Remove multisite keys in contacts.
     contacts = dict(
@@ -305,31 +446,46 @@ def save_users(profiles):
                             for p, val in profile.items()
                             if p in multisite_keys + multisite_attributes(profile.get('connector'))])
 
-    filename = root_dir + "contacts.mk"
 
     # Check_MK's monitoring contacts
+    filename = root_dir + "contacts.mk.new"
     out = create_user_file(filename, "w")
     out.write("# Written by Multisite UserDB\n# encoding: utf-8\n\n")
     out.write("contacts.update(\n%s\n)\n" % pprint.pformat(contacts))
     out.close()
+    os.rename(filename, filename[:-4])
 
     # Users with passwords for Multisite
+    filename = multisite_dir + "users.mk.new"
     make_nagios_directory(multisite_dir)
-    filename = multisite_dir + "users.mk"
     out = create_user_file(filename, "w")
     out.write("# Written by Multisite UserDB\n# encoding: utf-8\n\n")
     out.write("multisite_users = \\\n%s\n" % pprint.pformat(users))
     out.close()
+    os.rename(filename, filename[:-4])
 
     # Execute user connector save hooks
     hook_save(profiles)
 
     # Write out the users serials
-    serials_file = '%s/auth.serials' % os.path.dirname(defaults.htpasswd_file)
-    out = create_user_file(serials_file, "w")
+    serials_file = '%s/auth.serials.new' % os.path.dirname(defaults.htpasswd_file)
+    rename_file = True
+    try:
+        out = create_user_file(serials_file, "w")
+    except:
+        rename_file = False
+        out = create_user_file(serials_file[:-4], "w")
+
+    def encode_utf8(value):
+        if type(value) == unicode:
+            value = value.encode("utf-8")
+        return value
+
     for user_id, user in profiles.items():
-        out.write('%s:%d\n' % (user_id, user.get('serial', 0)))
+        out.write('%s:%d\n' % (encode_utf8(user_id), user.get('serial', 0)))
     out.close()
+    if rename_file:
+        os.rename(serials_file, serials_file[:-4])
 
     # Write user specific files
     for id, user in profiles.items():
@@ -343,9 +499,17 @@ def save_users(profiles):
         elif os.path.exists(auth_file):
             os.remove(auth_file)
 
-        # Write out the users serial
-        serial_file = user_dir + '/serial.mk'
-        create_user_file(serial_file, 'w').write('%d\n' % user.get('serial', 0))
+        # Write out user attributes which are written to dedicated files in the user
+        # profile directory. The primary reason to have separate files, is to reduce
+        # the amount of data to be loaded during regular page processing
+        save_custom_attr(id, 'serial', str(user.get('serial', 0)))
+        save_custom_attr(id, 'num_failed', str(user.get('num_failed', 0)))
+        save_custom_attr(id, 'enforce_pw_change', str(int(user.get('enforce_pw_change', False))))
+        save_custom_attr(id, 'last_pw_change', str(user.get('last_pw_change', int(time.time()))))
+
+        # Write out the last seent time
+        if 'last_seen' in user:
+            save_custom_attr(id, 'last_seen', repr(user['last_seen']))
 
     # Remove settings directories of non-existant users.
     # Beware: we removed this since it leads to violent destructions
@@ -357,10 +521,27 @@ def save_users(profiles):
     #         entry = dir + "/" + e
     #         if os.path.isdir(entry):
     #             shutil.rmtree(entry)
+    # But for the automation.secret this is ok, since automation users are not
+    # created by other sources in common cases
+    dir = defaults.var_dir + "/web"
+    for user_dir in os.listdir(defaults.var_dir + "/web"):
+        if user_dir not in ['.', '..'] and user_dir not in profiles:
+            entry = dir + "/" + user_dir
+            if os.path.isdir(entry) and os.path.exists(entry + '/automation.secret'):
+                os.unlink(entry + '/automation.secret')
+
+    # Release the lock to make other threads access possible again asap
+    # This lock is set by load_users() only in the case something is expected
+    # to be written (like during user syncs, wato, ...)
+    release_lock(root_dir + "contacts.mk")
+
+    # populate the users cache
+    html.set_cache('users', profiles)
 
     # Call the users_saved hook
-    hooks.call("users-saved", users)
+    hooks.call("users-saved", profiles)
 
+#.
 #   .-Roles----------------------------------------------------------------.
 #   |                       ____       _                                   |
 #   |                      |  _ \ ___ | | ___  ___                         |
@@ -384,9 +565,6 @@ def load_roles():
                   for id in config.builtin_role_ids ])
 
     filename = multisite_dir + "roles.mk"
-    if not os.path.exists(filename):
-        return roles
-
     try:
         vars = { "roles" : roles }
         execfile(filename, vars, vars)
@@ -406,6 +584,8 @@ def load_roles():
 
         return vars["roles"]
 
+    except IOError:
+        return roles # Use empty structure, not existing file is ok!
     except Exception, e:
         if config.debug:
             raise MKGeneralException(_("Cannot read configuration file %s: %s" %
@@ -415,6 +595,7 @@ def load_roles():
                      'Initializing structure...' % (filename, e))
         return roles
 
+#.
 #   .-Groups---------------------------------------------------------------.
 #   |                    ____                                              |
 #   |                   / ___|_ __ ___  _   _ _ __  ___                    |
@@ -426,18 +607,39 @@ def load_roles():
 
 def load_group_information():
     try:
-        filename = root_dir + "groups.mk"
-        if not os.path.exists(filename):
-            return {}
-
+        # Load group information from Check_MK world
         vars = {}
         for what in ["host", "service", "contact" ]:
             vars["define_%sgroups" % what] = {}
 
-        execfile(filename, vars, vars)
+        filename = root_dir + "groups.mk"
+        try:
+            execfile(filename, vars, vars)
+        except IOError:
+            return {} # skip on not existing file
+
+        # Now load information from the Web world
+        multisite_vars = {}
+        for what in ["host", "service", "contact" ]:
+            multisite_vars["multisite_%sgroups" % what] = {}
+
+        filename = multisite_dir + "groups.mk"
+        try:
+            execfile(filename, multisite_vars, multisite_vars)
+        except IOError:
+            pass
+
+        # Merge information from Check_MK and Multisite worlds together
         groups = {}
         for what in ["host", "service", "contact" ]:
-            groups[what] = vars.get("define_%sgroups" % what, {})
+            groups[what] = {}
+            for id, alias in vars['define_%sgroups' % what].items():
+                groups[what][id] = {
+                    'alias': alias
+                }
+                if id in multisite_vars['multisite_%sgroups' % what]:
+                    groups[what][id].update(multisite_vars['multisite_%sgroups' % what][id])
+
         return groups
 
     except Exception, e:
@@ -449,7 +651,57 @@ def load_group_information():
                      'Initializing structure...' % (filename, e))
         return {}
 
-#   .----------------------------------------------------------------------.
+#.
+#   .-Custom-Attrs.--------------------------------------------------------.
+#   |   ____          _                          _   _   _                 |
+#   |  / ___|   _ ___| |_ ___  _ __ ___         / \ | |_| |_ _ __ ___      |
+#   | | |  | | | / __| __/ _ \| '_ ` _ \ _____ / _ \| __| __| '__/ __|     |
+#   | | |__| |_| \__ \ || (_) | | | | | |_____/ ___ \ |_| |_| |  \__ \_    |
+#   |  \____\__,_|___/\__\___/|_| |_| |_|    /_/   \_\__|\__|_|  |___(_)   |
+#   |                                                                      |
+#   +----------------------------------------------------------------------+
+#   | Mange custom attributes of users (in future hosts etc.)              |
+#   '----------------------------------------------------------------------'
+
+def load_custom_attrs():
+    try:
+        filename = multisite_dir + "custom_attrs.mk"
+        if not os.path.exists(filename):
+            return {}
+
+        vars = {
+            'wato_user_attrs': [],
+        }
+        execfile(filename, vars, vars)
+
+        attrs = {}
+        for what in [ "user" ]:
+            attrs[what] = vars.get("wato_%s_attrs" % what, [])
+        return attrs
+
+    except Exception, e:
+        if config.debug:
+            raise MKGeneralException(_("Cannot read configuration file %s: %s" %
+                          (filename, e)))
+        else:
+            html.log('load_custom_attrs: Problem while loading custom attributes (%s - %s). '
+                     'Initializing structure...' % (filename, e))
+        return {}
+
+def declare_custom_user_attrs():
+    all_attrs = load_custom_attrs()
+    attrs = all_attrs.setdefault('user', [])
+    for attr in attrs:
+        vs = globals()[attr['type']](title = attr['title'], help = attr['help'])
+        declare_user_attribute(attr['name'], vs,
+            user_editable = attr['user_editable'],
+            show_in_table = attr.get('show_in_table', False),
+            topic = attr.get('topic', 'personal'),
+            add_custom_macro = attr.get('add_custom_macro', False )
+        )
+
+#.
+#   .-Hooks----------------------------------------------------------------.
 #   |                     _   _             _                              |
 #   |                    | | | | ___   ___ | | _____                       |
 #   |                    | |_| |/ _ \ / _ \| |/ / __|                      |
@@ -496,35 +748,44 @@ def hook_login(username, password):
 # Is called on:
 #   a) before rendering the user management page in WATO
 #   b) a user is created during login (only for this user)
-def hook_sync(connector_id = None, add_to_changelog = False, only_username = None):
+#   c) Before activating the changes in WATO
+def hook_sync(connector_id = None, add_to_changelog = False, only_username = None, raise_exc = False):
     if connector_id:
         connectors = [ get_connector(connector_id) ]
     else:
         connectors = enabled_connectors()
 
+    no_errors = True
     for connector in connectors:
         handler = connector.get('sync', None)
         if handler:
             try:
                 handler(add_to_changelog, only_username)
             except MKLDAPException, e:
+                if raise_exc:
+                    raise
                 if config.debug:
                     import traceback
                     html.show_error(
-                        "<h3>" + _("Error executing sync hook") + "</h3>"
+                        "<h3>" + _("Error during sync") + "</h3>"
                         "<pre>%s</pre>" % (traceback.format_exc())
                     )
                 else:
                     html.show_error(
-                        "<h3>" + _("Error executing sync hook") + "</h3>"
+                        "<h3>" + _("Error during sync") + "</h3>"
                         "<pre>%s</pre>" % (e)
                     )
+                no_errors = False
             except:
+                if raise_exc:
+                    raise
                 import traceback
                 html.show_error(
-                    "<h3>" + _("Error executing sync hook") + "</h3>"
+                    "<h3>" + _("Error during sync") + "</h3>"
                     "<pre>%s</pre>" % (traceback.format_exc())
                 )
+                no_errors = False
+    return no_errors
 
 # Hook function can be registered here to be executed during saving of the
 # new user construct
@@ -539,12 +800,18 @@ def hook_save(users):
             if config.debug:
                 import traceback
                 html.show_error(
-                    "<h3>" + _("Error executing sync hook") + "</h3>"
+                    "<h3>" + _("Error during saving") + "</h3>"
                     "<pre>%s</pre>" % (traceback.format_exc())
                 )
             else:
                 raise
 
+def hook_load():
+    for connector in enabled_connectors():
+        handler = connector.get('load', None)
+        if handler:
+            handler()
+
 # This function registers general stuff, which is independet of the single
 # connectors to each page load. It is exectued AFTER all other page hooks.
 def general_page_hook():
@@ -555,18 +822,21 @@ def general_page_hook():
     # This is a good place to replace old api based files in the future.
     auth_php = defaults.var_dir + '/wato/auth/auth.php'
     if not os.path.exists(auth_php) or os.path.getsize(auth_php) == 0:
-        create_auth_file(load_users())
+        create_auth_file("page_hook", load_users())
 
     # Create initial auth.serials file, same issue as auth.php above
     serials_file = '%s/auth.serials' % os.path.dirname(defaults.htpasswd_file)
     if not os.path.exists(serials_file) or os.path.getsize(serials_file) == 0:
-        save_users(load_users())
+        save_users(load_users(lock = True))
 
 # Hook function can be registered here to execute actions on a "regular" base without
 # user triggered action. This hook is called on each page load.
 # Catch all exceptions and log them to apache error log. Let exceptions raise trough
 # when debug mode is enabled.
 def hook_page():
+    if 'page' not in config.userdb_automatic_sync:
+        return
+
     for connector in enabled_connectors():
         handler = connector.get('page', None)
         if not handler:
@@ -582,3 +852,13 @@ def hook_page():
                             (connector['id'], traceback.format_exc()))
 
     general_page_hook()
+
+def ajax_sync():
+    try:
+        hook_sync(add_to_changelog = False, raise_exc = True)
+        html.write('OK\n')
+    except Exception, e:
+        if config.debug:
+            raise
+        else:
+            html.write('ERROR %s\n' % e)
