@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -37,7 +37,7 @@ import bi
 multisite_datasources["bi_aggregations"] = {
     "title"       : _("BI Aggregations"),
     "table"       : bi.table,
-    "infos"       : [ "aggr" ],
+    "infos"       : [ "aggr", "aggr_group", ],
     "keys"        : [],
     "idkeys"      : [ 'aggr_name' ],
 }
@@ -45,7 +45,7 @@ multisite_datasources["bi_aggregations"] = {
 multisite_datasources["bi_host_aggregations"] = {
     "title"       : _("BI Aggregations affected by one host"),
     "table"       : bi.host_table,
-    "infos"       : [ "host", "aggr" ],
+    "infos"       : [ "host", "aggr", "aggr_group" ],
     "keys"        : [],
     "idkeys"      : [ 'aggr_name' ],
 }
@@ -55,7 +55,16 @@ multisite_datasources["bi_host_aggregations"] = {
 multisite_datasources["bi_hostname_aggregations"] = {
     "title"       : _("BI Hostname Aggregations"),
     "table"       : bi.hostname_table,
-    "infos"       : [ "host", "aggr" ],
+    "infos"       : [ "host", "aggr", "aggr_group" ],
+    "keys"        : [],
+    "idkeys"      : [ 'aggr_name' ],
+}
+
+# The same but with group information
+multisite_datasources["bi_hostnamebygroup_aggregations"] = {
+    "title"       : _("BI Aggregations for Hosts by Hostgroups"),
+    "table"       : bi.hostname_by_group_table,
+    "infos"       : [ "host", "aggr", "hostgroup", "aggr_group" ],
     "keys"        : [],
     "idkeys"      : [ 'aggr_name' ],
 }
@@ -68,6 +77,30 @@ multisite_datasources["bi_hostname_aggregations"] = {
 #    |_|   \__,_|_|_| |_|\__\___|_|  |___/
 #
 
+def paint_bi_icons(row):
+    html.plug()
+    single_url = "view.py?" + html.urlencode_vars([
+            ("view_name", "aggr_single"),
+            ("aggr_name", row["aggr_name"])])
+    html.icon_button(single_url, _("Show only this aggregation"), "showbi")
+    avail_url = single_url + "&mode=availability"
+    html.icon_button(avail_url, _("Analyse availability of this aggregation"), "availability")
+    if row["aggr_effective_state"]["in_downtime"]:
+	html.icon(_("This aggregation is currently in a scheduled downtime"), "downtime")
+    if row["aggr_effective_state"]["acknowledged"]:
+	html.icon(_("The critical problems that make this aggregation non-OK have been acknowledged"), "ack")
+    code = html.drain()
+    html.unplug()
+    return "buttons", code
+
+multisite_painters["aggr_icons"] = {
+    "title" : _("Links"),
+    "columns" : [ "aggr_group", "aggr_name", "aggr_effective_state" ],
+    "printable" : False,
+    "paint" : paint_bi_icons,
+}
+
+
 def paint_aggr_state_short(state, assumed = False):
     if state == None:
         return "", ""
@@ -150,38 +183,55 @@ multisite_painters["aggr_hosts_services"] = {
 }
 
 multisite_painter_options["aggr_expand"] = {
- "title"   : _("Initial expansion of aggregations"),
- "default" : "0",
- "values"  : [ ("0", "collapsed"), ("1", "first level"), ("2", "two levels"), ("3", "three levels"), ("999", "complete")]
+    'valuespec' : DropdownChoice(
+        title = _("Initial expansion of aggregations"),
+        default_value = "0",
+        choices = [
+            ("0",   _("collapsed")),
+            ("1",   _("first level")),
+            ("2",   _("two levels")),
+            ("3",   _("three levels")),
+            ("999", _("complete"))
+        ]
+    )
 }
 
 multisite_painter_options["aggr_onlyproblems"] = {
- "title"   : _("Show only problems"),
- "default" : "0",
- "values"  : [ ("0", "show all"), ("1", "show only problems")]
+    'valuespec' : DropdownChoice(
+        title = _("Show only problems"),
+        default_value = "0",
+        choices = [
+            ("0", _("show all")),
+            ("1", _("show only problems"))
+        ],
+    )
 }
 
 multisite_painter_options["aggr_treetype"] = {
- "title"   : _("Type of tree layout"),
- "default" : "foldable",
- "values"  : [
-    ("foldable",     _("foldable")),
-    ("boxes",        _("boxes")),
-    ("boxes-omit-root", _("boxes (omit root)")),
-    ("bottom-up",    _("bottom up")),
-    ("top-down",     _("top down"))]
+    'valuespec' : DropdownChoice(
+        title = _("Type of tree layout"),
+        default_value = "foldable",
+        choices = [
+            ("foldable",        _("foldable")),
+            ("boxes",           _("boxes")),
+            ("boxes-omit-root", _("boxes (omit root)")),
+            ("bottom-up",       _("bottom up")),
+            ("top-down",        _("top down")),
+        ],
+    )
 }
 
 multisite_painter_options["aggr_wrap"] = {
- "title"   : _("Handling of too long texts"),
- "default" : "wrap",
- "values"  : [ ("wrap", "wrap"), ("nowrap", "don't wrap")]
+    'valuespec' : DropdownChoice(
+         title = _("Handling of too long texts"),
+         default_value = "wrap",
+         choices = [
+            ("wrap",   _("wrap")),
+            ("nowrap", _("don't wrap")),
+        ],
+    )
 }
 
-
-
-
-
 def paint_aggr_tree_ltr(row, mirror):
     wrap = get_painter_option("aggr_wrap")
 
@@ -265,185 +315,3 @@ multisite_painters["aggr_treestate_boxed"] = {
     "paint"   : lambda row: bi.render_tree_foldable(row, boxes=True, omit_root=True,
                 expansion_level=bi.load_ex_level(), only_problems=False, lazy=True),
 }
-
-#     _____ _ _ _
-#    |  ___(_) | |_ ___ _ __ ___
-#    | |_  | | | __/ _ \ '__/ __|
-#    |  _| | | | ||  __/ |  \__ \
-#    |_|   |_|_|\__\___|_|  |___/
-#
-
-class BIGroupFilter(Filter):
-    def __init__(self):
-        self.column = "aggr_group"
-        Filter.__init__(self, self.column, _("Aggregation group"), "aggr", [self.column], [self.column])
-
-    def variable_settings(self, row):
-        return [ (self.htmlvars[0], row[self.column]) ]
-
-    def display(self):
-        htmlvar = self.htmlvars[0]
-        html.select(htmlvar, [ ("", "") ] + [(g, g) for g in bi.aggregation_groups()])
-
-    def selected_group(self):
-        return html.var(self.htmlvars[0])
-
-    def filter_table(self, rows):
-        group = self.selected_group()
-        if not group:
-            return rows
-        else:
-            return [ row for row in rows if row[self.column] == group ]
-
-    def heading_info(self, infoname):
-        return html.var(self.htmlvars[0])
-
-declare_filter( 90,  BIGroupFilter())
-
-class BITextFilter(Filter):
-    def __init__(self, what):
-        self.column = "aggr_" + what
-        label = ''
-        if what == 'name':
-            label = _('Aggregation name')
-        elif what == 'output':
-            label = _('Aggregation output')
-        Filter.__init__(self, self.column, label, "aggr", [self.column], [self.column])
-
-    def variable_settings(self, row):
-        return [ (self.htmlvars[0], row[self.column]) ]
-
-    def display(self):
-        html.text_input(self.htmlvars[0])
-
-    def heading_info(self, infoname):
-        return html.var(self.htmlvars[0])
-
-    def filter_table(self, rows):
-        val = html.var(self.htmlvars[0])
-        if not val:
-            return rows
-        reg = re.compile(val.lower())
-        return [ row for row in rows if reg.search(row[self.column].lower()) ]
-
-declare_filter(120, BITextFilter("name"))
-declare_filter(121, BITextFilter("output"))
-
-class BIHostFilter(Filter):
-    def __init__(self):
-        self.column = "aggr_hosts"
-        Filter.__init__(self, self.column, _("Affected hosts contain"), "aggr", ["site", "host"], [])
-
-    def display(self):
-        html.text_input(self.htmlvars[1])
-
-    def heading_info(self, infoname):
-        return html.var(self.htmlvars[1])
-
-    def find_host(self, host, hostlist):
-        for s, h in hostlist:
-            if h == host:
-                return True
-        return False
-
-    # Used for linking
-    def variable_settings(self, row):
-        return [ ("host", row["host_name"]), ("site", row["site"]) ]
-
-    def filter_table(self, rows):
-        val = html.var(self.htmlvars[1])
-        if not val:
-            return rows
-        return [ row for row in rows if self.find_host(val, row["aggr_hosts"]) ]
-
-declare_filter(130, BIHostFilter(), _("Filter for all aggregations that base on status information of that host. Exact match (no regular expression)"))
-
-class BIServiceFilter(Filter):
-    def __init__(self):
-        Filter.__init__(self, "aggr_service", _("Affected by service"), "aggr", ["site", "host", "service"], [])
-
-    def double_height(self):
-        return True
-
-    def display(self):
-        html.write(_("Host") + ": ")
-        html.text_input("host")
-        html.write(_("Service") + ": ")
-        html.text_input("service")
-
-    def heading_info(self, infoname):
-        return html.var_utf8("host") + " / " + html.var_utf8("service")
-
-    def service_spec(self):
-        return html.var_utf8("site"), html.var_utf8("host"), html.var_utf8("service")
-
-    # Used for linking
-    def variable_settings(self, row):
-        return [ ("site", row["site"]), ("host", row["host_name"]), ("service", row["service_description"]) ]
-
-declare_filter(131, BIServiceFilter(), _("Filter for all aggregations that are affected by one specific service on a specific host (no regular expression)"))
-
-class BIStatusFilter(Filter):
-    def __init__(self, what):
-        title = (what.replace("_", " ") + " state").title()
-        self.column = "aggr_" + what + "state"
-        if what == "":
-            self.code = 'r'
-        else:
-            self.code = what[0]
-        self.prefix = "bi%ss" % self.code
-        vars = [ self.prefix + str(x) for x in [ -1, 0, 1, 2, 3 ] ]
-        if self.code == 'a':
-            vars.append(self.prefix + "n")
-        Filter.__init__(self, self.column, title, "aggr", vars, [])
-
-    def filter(self, tablename):
-        return ""
-
-    def double_height(self):
-        return self.column == "aggr_assumed_state"
-
-    def display(self):
-        if html.var("filled_in"):
-            defval = ""
-        else:
-            defval = "on"
-        for varend, text in [('0', 'OK'), ('1', 'WARN'), ('2', 'CRIT'),
-                             ('3', 'UNKN'), ('-1', 'PENDING'), ('n', _('no assumed state set'))]:
-            if self.code != 'a' and varend == 'n':
-                continue # no unset for read and effective state
-            if varend == 'n':
-                html.write("<br>")
-            var = self.prefix + varend
-            html.checkbox(var, defval, label = text)
-
-    def filter_table(self, rows):
-        jeaders = []
-        if html.var("filled_in"):
-            defval = ""
-        else:
-            defval = "on"
-
-        allowed_states = []
-        for i in ['0','1','2','3','-1','n']:
-            if html.var(self.prefix + i, defval) == "on":
-                if i == 'n':
-                    s = None
-                else:
-                    s = int(i)
-                allowed_states.append(s)
-        newrows = []
-        for row in rows:
-            if row[self.column] != None:
-                s = row[self.column]["state"]
-            else:
-                s = None
-            if s in allowed_states:
-                newrows.append(row)
-        return newrows
-
-declare_filter(150,  BIStatusFilter(""))
-declare_filter(151,  BIStatusFilter("effective_"))
-declare_filter(152,  BIStatusFilter("assumed_"))
-
-
