@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -36,25 +36,40 @@
 # - function that outputs the HTML input fields
 # - function that creates the nagios command and title
 
-#import datetime, traceback
-#file('/tmp/1', 'a').write('%s %s\n%s' % (datetime.datetime.now(), current_language, ''.join(traceback.format_stack())))
-
 # RESCHEDULE ACTIVE CHECKS
+def command_reschedule(cmdtag, spec, row, row_nr, total_rows):
+    if html.var("_resched_checks"):
+        spread = saveint(html.var("_resched_spread"))
+        text = "<b>" + _("reschedule an immediate check")
+        if spread:
+            text += _(" spread over %d minutes ") % spread
+
+        text += "</b>" + _("of")
+
+        t = time.time()
+        if spread:
+            t += spread * 60.0 * row_nr / total_rows
+
+        command = "SCHEDULE_FORCED_" + cmdtag + "_CHECK;%s;%d" % (spec, int(t))
+        return command, text
+
 config.declare_permission("action.reschedule",
         _("Reschedule checks"),
         _("Reschedule host and service checks"),
         [ "user", "admin" ])
 
+
 multisite_commands.append({
     "tables"      : [ "host", "service" ],
     "permission"  : "action.reschedule",
-    "title"       : _("Reschedule"),
+    "title"       : _("Reschedule active checks"),
     "render"      : lambda: \
-        html.button("_resched_checks", _("Reschedule active checks")),
-    "action"      : lambda cmdtag, spec, row:
-        html.var("_resched_checks") and (
-            "SCHEDULE_FORCED_" + cmdtag + "_CHECK;%s;%d" % (spec, int(time.time())),
-            _("<b>reschedule an immediate check</b> of"))
+        html.button("_resched_checks", _("Reschedule")) == \
+        html.write(_("and spread over") + " ") == \
+        html.number_input("_resched_spread", 0, size=3) == \
+        html.write(" " + _("minutes") + " "),
+    "action"      : command_reschedule,
+    "row_stats"   : True, # Get information about number of rows and current row nr.
 })
 
 
@@ -131,8 +146,8 @@ multisite_commands.append({
 
 # CLEAR MODIFIED ATTRIBUTES
 config.declare_permission("action.clearmodattr",
-        _("Clear modified attributes"),
-        _("Remove the information that an attribute (like check enabling) has been changed"),
+        _("Reset modified attributes"),
+        _("Reset all manually modified attributes of a host or service (like disabled notifications)"),
         [ "admin" ])
 
 multisite_commands.append({
@@ -140,11 +155,11 @@ multisite_commands.append({
     "permission"  : "action.clearmodattr",
     "title"       : _("Modified attributes"),
     "render"      : lambda: \
-       html.button("_clear_modattr", _('Clear information about modified attributes')),
+       html.button("_clear_modattr", _('Clear modified attributes')),
     "action"      : lambda cmdtag, spec, row: (
         html.var("_clear_modattr") and (
         "CHANGE_" + cmdtag + "_MODATTR;%s;0" % spec,
-         _("<b>clear the information about modified attributes</b> of"))),
+         _("<b>clear the modified attributes</b> of"))),
 })
 
 # FAKE CHECKS
@@ -154,25 +169,51 @@ config.declare_permission("action.fakechecks",
         [ "admin" ])
 
 def command_fake_checks(cmdtag, spec, row):
-    for s in [0,1,2,3]:
+    for s in [0, 1, 2, 3]:
         statename = html.var("_fake_%d" % s)
         if statename:
-            pluginoutput = _("Manually set to %s by %s") % (statename, config.user_id)
+            pluginoutput = html.var_utf8("_fake_output").strip()
+            if not pluginoutput:
+                pluginoutput = _("Manually set to %s by %s") % (html.attrencode(statename), config.user_id)
+            perfdata = html.var("_fake_perfdata")
+            if perfdata:
+                pluginoutput += "|" + perfdata
             if cmdtag == "SVC":
                 cmdtag = "SERVICE"
-            command = "PROCESS_%s_CHECK_RESULT;%s;%s;%s" % (cmdtag, spec, s, pluginoutput)
-            title = _("<b>manually set check results to %s</b> for") % statename
+            command = "PROCESS_%s_CHECK_RESULT;%s;%s;%s" % (cmdtag, spec, s, lqencode(pluginoutput))
+            title = _("<b>manually set check results to %s</b> for") % html.attrencode(statename)
             return command, title
 
 
+def render_fake_form(what):
+    html.write("<table><tr><td>")
+    html.write("%s: " % _("Plugin output"))
+    html.write("</td><td>")
+    html.text_input("_fake_output", "", size=50)
+    html.write("</td></tr><tr><td>")
+    html.write("%s: " % _("Performance data"))
+    html.write("</td><td>")
+    html.text_input("_fake_perfdata", "", size=50)
+    html.write("</td></tr><tr><td>")
+    html.write(_("Set to:"))
+    html.write("</td><td>")
+    if what == "host":
+        html.button("_fake_0", _("Up"))
+        html.button("_fake_1", _("Down"))
+        html.button("_fake_2", _("Unreachable"))
+    else:
+        html.button("_fake_0", _("OK"))
+        html.button("_fake_1", _("Warning"))
+        html.button("_fake_2", _("Critical"))
+        html.button("_fake_3", _("Unknown"))
+    html.write("</td></tr></table>")
+
 multisite_commands.append({
     "tables"      : [ "host" ],
     "permission"  : "action.fakechecks",
     "title"       : _("Fake check results"),
-    "render"      : lambda: \
-       html.button("_fake_0", _("Up")) == \
-       html.button("_fake_1", _("Down")) == \
-       html.button("_fake_2", _("Unreachable")),
+    "group"       : _("Fake check results"),
+    "render"      : lambda: render_fake_form("host"),
     "action"      : command_fake_checks,
 })
 
@@ -180,11 +221,8 @@ multisite_commands.append({
     "tables"      : [ "service" ],
     "permission"  : "action.fakechecks",
     "title"       : _("Fake check results"),
-    "render"      : lambda: \
-       html.button("_fake_0", _("OK")) == \
-       html.button("_fake_1", _("Warning")) == \
-       html.button("_fake_2", _("Critical")) == \
-       html.button("_fake_3", _("Unknown")),
+    "group"       : _("Fake check results"),
+    "render"      : lambda: render_fake_form("service"),
     "action"      : command_fake_checks,
 })
 
@@ -201,7 +239,7 @@ def command_custom_notification(cmdtag, spec, row):
         broadcast = html.get_checkbox("_cusnot_broadcast") and 1 or 0
         forced = html.get_checkbox("_cusnot_forced") and 2 or 0
         command = "SEND_CUSTOM_%s_NOTIFICATION;%s;%s;%s;%s" % \
-                ( cmdtag, spec, broadcast + forced, config.user_id, comment)
+                ( cmdtag, spec, broadcast + forced, config.user_id, lqencode(comment))
         title = _("<b>send a custom notification</b> regarding")
         return command, title
 
@@ -232,12 +270,23 @@ def command_acknowledgement(cmdtag, spec, row):
         comment = html.var_utf8("_ack_comment")
         if not comment:
             raise MKUserError("_ack_comment", _("You need to supply a comment."))
+        if ";" in comment:
+            raise MKUserError("_ack_comment", _("The comment must not contain semicolons."))
         sticky = html.var("_ack_sticky") and 2 or 0
         sendnot = html.var("_ack_notify") and 1 or 0
         perscomm = html.var("_ack_persistent") and 1 or 0
+
+        expire_secs = Age().from_html_vars("_ack_expire")
+        if expire_secs:
+            expire = int(time.time()) + expire_secs
+        else:
+            expire = 0
+
         command = "ACKNOWLEDGE_" + cmdtag + "_PROBLEM;%s;%d;%d;%d;%s" % \
-                      (spec, sticky, sendnot, perscomm, config.user_id) + (";%s" % comment)
-        title = _("<b>acknowledge the problems</b> of")
+                      (spec, sticky, sendnot, perscomm, config.user_id) + (";%s" % lqencode(comment)) \
+                      + (";%d" % expire)
+        title = _("<b>acknowledge the problems%s</b> of") % \
+                    (expire and (_(" for a period of %s") % Age().value_to_text(expire_secs)) or "")
         return command, title
 
     elif html.var("_remove_ack"):
@@ -249,15 +298,18 @@ def command_acknowledgement(cmdtag, spec, row):
 multisite_commands.append({
     "tables"      : [ "host", "service" ],
     "permission"  : "action.acknowledge",
-    "title"       : _("Acknowledge"),
+    "title"       : _("Acknowledge Problems"),
     "render"      : lambda: \
         html.button("_acknowledge", _("Acknowledge")) == \
-        html.button("_remove_ack", _("Remove Acknowledge")) == \
+        html.button("_remove_ack", _("Remove Acknowledgement")) == \
         html.write("<hr>") == \
         html.checkbox("_ack_sticky", True, label=_("sticky")) == \
         html.checkbox("_ack_notify", True, label=_("send notification")) == \
         html.checkbox("_ack_persistent", False, label=_('persistent comment')) == \
         html.write("<hr>") == \
+        Age(display=["days", "hours", "minutes"], label=_("Expire acknowledgement after")).render_input("_ack_expire", 0) == \
+        html.help(_("Note: Expiration of acknowledgements only works when using the Check_MK Micro Core.")) == \
+        html.write("<hr>") == \
         html.write(_("Comment") + ": ") == \
         html.text_input("_ack_comment", size=48, submit="_acknowledge"),
     "action"      : command_acknowledgement,
@@ -277,7 +329,7 @@ def command_comment(cmdtag, spec, row):
         if not comment:
             raise MKUserError("_comment", _("You need to supply a comment."))
         command = "ADD_" + cmdtag + "_COMMENT;%s;1;%s" % \
-                  (spec, config.user_id) + (";%s" % comment)
+                  (spec, config.user_id) + (";%s" % lqencode(comment))
         title = _("<b>add a comment to</b>")
         return command, title
 
@@ -348,6 +400,11 @@ def command_downtime(cmdtag, spec, row):
         down_to = time.time() + minutes * 60
         title = _("<b>schedule an immediate downtime for the next %d minutes</b> on" % minutes)
 
+    elif html.var("_down_adhoc"):
+        minutes = config.adhoc_downtime.get("duration",0)
+        down_to = time.time() + minutes * 60
+        title = _("<b>schedule an immediate downtime for the next %d minutes</b> on" % minutes)
+
     elif html.var("_down_custom"):
         down_from = html.get_datetime_input("_down_from")
         down_to   = html.get_datetime_input("_down_to")
@@ -360,6 +417,9 @@ def command_downtime(cmdtag, spec, row):
             time.asctime(time.localtime(down_to)))
 
     elif html.var("_down_remove"):
+        if html.var("_on_hosts"):
+            raise MKUserError("_on_hosts", _("The checkbox for setting host downtimes does not work when removing downtimes."))
+
         downtime_ids = []
         if cmdtag == "HOST":
             prefix = "host_"
@@ -376,7 +436,10 @@ def command_downtime(cmdtag, spec, row):
         return commands, title
 
     if down_to:
-        comment = html.var_utf8("_down_comment")
+        if html.var("_down_adhoc"):
+            comment = config.adhoc_downtime.get("comment","")
+        else:
+            comment = html.var_utf8("_down_comment")
         if not comment:
             raise MKUserError("_down_comment", _("You need to supply a comment for your downtime."))
         if html.var("_down_flexible"):
@@ -388,14 +451,19 @@ def command_downtime(cmdtag, spec, row):
 
         if html.var("_include_childs"): # only for hosts
             specs = [ spec ] + get_child_hosts(row["site"], [spec], recurse = not not html.var("_include_childs_recurse"))
+        elif html.var("_on_hosts"): # set on hosts instead of services
+            specs = [ spec.split(";")[0] ]
+            title += " the hosts of"
+            cmdtag = "HOST"
         else:
             specs = [ spec ]
 
         commands = [(("SCHEDULE_" + cmdtag + "_DOWNTIME;%s;" % spec ) \
                    + ("%d;%d;%d;0;%d;%s;" % (down_from, down_to, fixed, duration, config.user_id)) \
-                   + comment) for spec in specs]
+                   + lqencode(comment)) for spec in specs]
         return commands, title
 
+
 def get_child_hosts(site, hosts, recurse):
     hosts = set(hosts)
     html.live.set_only_sites([site])
@@ -413,9 +481,16 @@ def get_child_hosts(site, hosts, recurse):
         new_childs.update(rec_childs)
     return list(new_childs)
 
+
 def paint_downtime_buttons(what):
+
     html.write(_('Downtime Comment')+": ")
-    html.text_input("_down_comment", size=40, submit="")
+    html.text_input("_down_comment", "", size=60, submit="")
+    html.write("<hr>")
+    html.button("_down_from_now", _("From now for"))
+    html.write("&nbsp;")
+    html.number_input("_down_minutes", 60, size=4, submit="_down_from_now")
+    html.write("&nbsp; " + _("minutes"))
     html.write("<hr>")
     html.button("_down_2h", _("2 hours"))
     html.button("_down_today", _("Today"))
@@ -425,24 +500,31 @@ def paint_downtime_buttons(what):
     html.write(" &nbsp; - &nbsp;")
     html.button("_down_remove", _("Remove all"))
     html.write("<hr>")
+    if config.adhoc_downtime and config.adhoc_downtime.get("duration"):
+        adhoc_duration = config.adhoc_downtime.get("duration")
+        adhoc_comment  = config.adhoc_downtime.get("comment", "")
+        html.button("_down_adhoc", _("Adhoc for %d minutes") % adhoc_duration)
+        html.write("&nbsp;")
+        html.write(_('with comment')+": ")
+        html.write(adhoc_comment)
+        html.write("<hr>")
+
     html.button("_down_custom", _("Custom time range"))
     html.datetime_input("_down_from", time.time(), submit="_down_custom")
     html.write("&nbsp; "+_('to')+" &nbsp;")
     html.datetime_input("_down_to", time.time() + 7200, submit="_down_custom")
     html.write("<hr>")
-    html.button("_down_from_now", _("From now for"))
-    html.write("&nbsp;")
-    html.number_input("_down_minutes", 60, size=4, submit="_down_from_now")
-    html.write("&nbsp; " + _("minutes"))
-    html.write("<hr>")
     html.checkbox("_down_flexible", False, label=_('flexible with max. duration')+" ")
     html.time_input("_down_duration", 2, 0)
     html.write(" "+_('(HH:MM)'))
+    html.write("<hr>")
     if what == "host":
-        html.write("<hr>")
         html.checkbox("_include_childs", False, label=_('Also set downtime on child hosts'))
         html.write("  ")
         html.checkbox("_include_childs_recurse", False, label=_('Do this recursively'))
+    else:
+        html.checkbox("_on_hosts", False, label=_('Schedule downtimes on the affected <b>hosts</b> instead of their services'))
+
 
 
 multisite_commands.append({
@@ -488,3 +570,50 @@ multisite_commands.append({
       ( "DEL_%s_COMMENT;%d" % (cmdtag, spec),
         _("remove"))
 })
+
+#   .--Stars *-------------------------------------------------------------.
+#   |                   ____  _                                            |
+#   |                  / ___|| |_ __ _ _ __ ___  __/\__                    |
+#   |                  \___ \| __/ _` | '__/ __| \    /                    |
+#   |                   ___) | || (_| | |  \__ \ /_  _\                    |
+#   |                  |____/ \__\__,_|_|  |___/   \/                      |
+#   |                                                                      |
+#   '----------------------------------------------------------------------'
+
+def command_star(cmdtag, spec, row):
+    if html.var("_star") or html.var("_unstar"):
+        star = html.var("_star") and 1 or 0
+        if star:
+            title = _("<b>add to you favorites</b>")
+        else:
+            title = _("<b>remove from your favorites</b>")
+        return "STAR;%d;%s" % (star, spec), title
+
+
+def command_executor_star(command, site):
+    foo, star, spec = command.split(";", 2)
+    stars = config.load_stars()
+    if star == "0" and spec in stars:
+        stars.remove(spec)
+    elif star == "1":
+        stars.add(spec)
+    config.save_stars(stars)
+
+config.declare_permission("action.star",
+    _("Use favorites"),
+    _("This permission allows a user to make certain host and services "
+      "his personal favorites. Favorites can be used for a having a fast "
+      "access to items that are needed on a regular base."),
+    [ "user", "admin" ])
+
+multisite_commands.append({
+    "tables"         : [ "host", "service" ],
+    "permission"     : "action.star",
+    "title"          : _("Favorites"),
+    "render"         : lambda: \
+       html.button("_star",   _("Add to Favorites")) == \
+       html.button("_unstar", _("Remove from Favorites")),
+    "action"         : command_star,
+    "executor"       : command_executor_star,
+})
+
