@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -24,6 +24,9 @@
 # to the Free Software Foundation, Inc., 51 Franklin St,  Fifth Floor,
 # Boston, MA 02110-1301 USA.
 
+# FIXME: Can be removed once all dashboards have been converted
+# to have the view definitions right inside the dashboards
+
 multisite_builtin_views.update({
 
 # A similar view, used in the dashboard
@@ -46,7 +49,7 @@ multisite_builtin_views.update({
                                       ('is_host_acknowledged', '0'),
                                       ('is_summary_host', '0'),
                                       ],
-                  'hidden': False,
+                  'hidden': True,
                   'hidebutton': True,
                   'hide_filters': [],
                   'layout': 'table',
@@ -98,7 +101,7 @@ multisite_builtin_views.update({
                                      ('hst2', ''),
                                      ('hstp', 'on'),
                                      ('is_summary_host', '0')],
-                 'hidden': False,
+                 'hidden': True,
                  'hide_filters': [],
                  'layout': 'table',
                  'mustsearch': False,
@@ -135,7 +138,7 @@ multisite_builtin_views.update({
                                 ('logtime_from_range', '3600'),
                                 ('logtime_from', '4'),
                                 ],
-            'hidden': False,
+            'hidden': True,
             'hide_filters': [],
             'layout': 'table',
             'linktitle': 'Events',
