@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -184,7 +184,7 @@ def render_grouped_boxes(rows, view, group_painters, painters, num_columns, show
             html.write("</tr>\n")
 
         column_headers = view.get("column_headers")
-        if column_headers:
+        if column_headers != "off":
             show_header_line()
 
         visible_row_number = 0
@@ -200,11 +200,14 @@ def render_grouped_boxes(rows, view, group_painters, painters, num_columns, show
             else:
                 trclass = "odd"
             # state = row.get("service_state", row.get("aggr_state"))
-            state = row.get("service_state")
+            state = saveint(row.get("service_state"))
             if state == None:
-                state = row.get("host_state", 0)
+                state = saveint(row.get("host_state", 0))
                 if state > 0: state +=1 # 1 is critical for hosts
-            html.write('<tr class="data %s%d">' % (trclass, state))
+            stale = ''
+            if is_stale(row):
+                stale = ' stale'
+            html.write('<tr class="data %s%d%s">' % (trclass, state, stale))
             if show_checkboxes:
                 render_checkbox_td(view, row, len(painters))
             for p in painters:
