@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -74,44 +74,42 @@
 
 import bi # needed for aggregation icon
 
-multisite_painter_options["pnpview"] = {
- "title"   : _("PNP Timerange"),
- "default" : "1",
- "values"  : [ ("0", _("4 Hours")),  ("1", _("25 Hours")),
-               ("2", _("One Week")), ("3", _("One Month")),
-               ("4", _("One Year")), ("", _("All")) ]
+multisite_painter_options["pnp_timerange"] = {
+    'valuespec' : PNPTimerange(
+        title = _("PNP Timerange"),
+        default_value = None,
+        include_time = True,
+    )
 }
 
 multisite_painter_options["ts_format"] = {
- "title"   : _("Time stamp format"),
- "default" : config.default_ts_format,
- "values"  : [
-     ("mixed", _("Mixed")),
-     ("abs", _("Absolute")),
-     ("rel", _("Relative")),
-     ("both", _("Both")),
-  ]
+    'valuespec': DropdownChoice(
+        title = _("Time stamp format"),
+        default_value = config.default_ts_format,
+        choices = [
+            ("mixed", _("Mixed")),
+            ("abs",   _("Absolute")),
+            ("rel",   _("Relative")),
+            ("both",  _("Both")),
+            ("epoch", _("Unix Timestamp (Epoch)")),
+        ],
+    )
 }
 
 multisite_painter_options["ts_date"] = {
- "title" : _("Date format"),
- "default" : "%Y-%m-%d",
- "values" : [ ("%Y-%m-%d", "1970-12-18"),
-              ("%d.%m.%Y", "18.12.1970"),
-              ("%m/%d/%Y", "12/18/1970"),
-              ("%d.%m.",   "18.12."),
-              ("%m/%d",    "12/18") ]
+    'valuespec' : DateFormat(),
 }
 
 # This helper function returns the value of the given custom var
-def paint_custom_host_var(what, row):
-    custom_vars = dict(zip(row["host_custom_variable_names"],
-                           row["host_custom_variable_values"]))
-
-    if what in custom_vars:
-        return what, custom_vars[what]
-    return what,  ""
+def paint_custom_var(what, key, row):
+    if what:
+        what += '_'
+    custom_vars = dict(zip(row[what + "custom_variable_names"],
+                           row[what + "custom_variable_values"]))
 
+    if key in custom_vars:
+        return key, custom_vars[key]
+    return key,  ""
 
 #    ___
 #   |_ _|___ ___  _ __  ___
@@ -153,7 +151,23 @@ def paint_icons(what, row):
         except Exception, e:
             output += 'Exception in icon plugin!<br />' + traceback.format_exc()
 
-    return "icons", output
+    if html.output_format == "html":
+        return "icons", output
+    else:
+        # Strip icon names out of HTML code that is generated by htmllib.render_icon()
+        icon_rename_regex = regex('<img src="([^"]*)"[^>]*>')
+        icons = []
+        for n in icon_rename_regex.findall(output):
+            if n.startswith("images/"):
+                n = n[7:]
+            if n.startswith("icon_"):
+                n = n[5:]
+            if n.endswith(".png"):
+                n = n[:-4]
+            elif n.endswith(".gif"):
+                n = n[:-4]
+            icons.append(n.encode('utf-8'))
+        return "icons", " ".join(icons)
 
 def iconpainter_columns(what):
     cols = set(['site',
@@ -180,15 +194,19 @@ def iconpainter_columns(what):
 multisite_painters["service_icons"] = {
     "title":   _("Service icons"),
     "short":   _("Icons"),
+    "printable" : False, # does not contain printable text
     "columns": iconpainter_columns("service"),
-    "paint":   lambda row: paint_icons("service", row)
+    "groupby" : lambda row: "", # Do not account for in grouping
+    "paint":    lambda row: paint_icons("service", row)
 }
 
 multisite_painters["host_icons"] = {
     "title":   _("Host icons"),
     "short":   _("Icons"),
+    "printable" : False, # does not contain printable text
     "columns": iconpainter_columns("host"),
-    "paint":   lambda row: paint_icons("host", row)
+    "groupby" : lambda row: "", # Do not account for in grouping
+    "paint":    lambda row: paint_icons("host", row)
 }
 
 # -----------------------------------------------------------------------
@@ -197,26 +215,30 @@ def paint_nagios_link(row):
     # We need to use the Nagios-URL as configured
     # in sites.
     baseurl = config.site(row["site"])["url_prefix"] + "nagios/cgi-bin"
-    url = baseurl + "/extinfo.cgi?host=" + htmllib.urlencode(row["host_name"])
+    url = baseurl + "/extinfo.cgi?host=" + html.urlencode(row["host_name"])
     svc = row.get("service_description")
     if svc:
-        url += "&type=2&service=" + htmllib.urlencode(svc)
+        url += "&type=2&service=" + html.urlencode(svc)
         what = "service"
     else:
         url += "&type=1"
         what = "host"
-    return "singleicon", "<a href=\"%s\"><img title=\"%s\" src=\"images/icon_nagios.gif\"></a>" % (url, _('Show this %s in Nagios') % what)
+    return "singleicon", "<a href=\"%s\">%s</a>" % \
+        (url, html.render_icon('nagios', _('Show this %s in Nagios') % what))
 
-def paint_age(timestamp, has_been_checked, bold_if_younger_than, mode=None):
+def paint_age(timestamp, has_been_checked, bold_if_younger_than, mode=None, what='past'):
     if not has_been_checked:
         return "age", "-"
 
     if mode == None:
         mode = get_painter_option("ts_format")
 
+    if mode == "epoch":
+        return "", str(int(timestamp))
+
     if mode == "both":
-        css, h1 = paint_age(timestamp, has_been_checked, bold_if_younger_than, "abs")
-        css, h2 = paint_age(timestamp, has_been_checked, bold_if_younger_than, "rel")
+        css, h1 = paint_age(timestamp, has_been_checked, bold_if_younger_than, "abs", what=what)
+        css, h2 = paint_age(timestamp, has_been_checked, bold_if_younger_than, "rel", what=what)
         return css, "%s - %s" % (h1, h2)
 
     dateformat = get_painter_option("ts_date")
@@ -225,6 +247,12 @@ def paint_age(timestamp, has_been_checked, bold_if_younger_than, mode=None):
         (mode == "mixed" and age >= 48 * 3600 or age < -48 * 3600):
         return "age", time.strftime(dateformat + " %H:%M:%S", time.localtime(timestamp))
 
+    warn_txt = ''
+    if what == 'future' and age > 0:
+        warn_txt = ' <b>%s</b>' % _('in the past!')
+    elif what == 'past' and age < 0:
+        warn_txt = ' <b>%s</b>' % _('in the future!')
+
     # Time delta less than two days => make relative time
     if age < 0:
         age = -age
@@ -235,14 +263,15 @@ def paint_age(timestamp, has_been_checked, bold_if_younger_than, mode=None):
         age_class = "age recent"
     else:
         age_class = "age"
-    return age_class, prefix + html.age_text(age)
+
+    return age_class, prefix + html.age_text(age) + warn_txt
 
 
 def paint_future_time(timestamp):
     if timestamp <= 0:
         return "", "-"
     else:
-        return paint_age(timestamp, True, 0)
+        return paint_age(timestamp, True, 0, what='future')
 
 def paint_day(timestamp):
     return "", time.strftime("%A, %Y-%m-%d", time.localtime(timestamp))
@@ -254,7 +283,7 @@ def paint_site_icon(row):
         return None, ""
 
 multisite_painters["sitename_plain"] = {
-    "title"   : _("Site id"),
+    "title"   : _("Site ID"),
     "short"   : _("Site"),
     "columns" : ["site"],
     "paint"   : lambda row: (None, row["site"]),
@@ -267,30 +296,41 @@ multisite_painters["sitealias"] = {
     "paint"   : lambda row: (None, config.site(row["site"])["alias"]),
 }
 
-#    ____                  _
-#   / ___|  ___ _ ____   _(_) ___ ___  ___
-#   \___ \ / _ \ '__\ \ / / |/ __/ _ \/ __|
-#    ___) |  __/ |   \ V /| | (_|  __/\__ \
-#   |____/ \___|_|    \_/ |_|\___\___||___/
-#
+#.
+#   .--Services------------------------------------------------------------.
+#   |                ____                  _                               |
+#   |               / ___|  ___ _ ____   _(_) ___ ___  ___                 |
+#   |               \___ \ / _ \ '__\ \ / / |/ __/ _ \/ __|                |
+#   |                ___) |  __/ |   \ V /| | (_|  __/\__ \                |
+#   |               |____/ \___|_|    \_/ |_|\___\___||___/                |
+#   |                                                                      |
+#   +----------------------------------------------------------------------+
+#   | Painters for services                                                |
+#   '----------------------------------------------------------------------'
 
-def paint_service_state_short(row ):
+def paint_service_state_short(row):
     if row["service_has_been_checked"] == 1:
-        state = row["service_state"]
-        name = nagios_short_state_names[row["service_state"]]
+        state = str(row["service_state"])
+        name = nagios_short_state_names.get(row["service_state"], "")
     else:
         state = "p"
-        name = "PEND"
+        name = _("PEND")
+    if is_stale(row):
+        state = str(state) + " stale"
     return "state svcstate state%s" % state, name
 
 def paint_host_state_short(row):
 # return None, str(row)
     if row["host_has_been_checked"] == 1:
         state = row["host_state"]
-        name = nagios_short_host_state_names[row["host_state"]]
+        # A state of 3 is sent by livestatus in cases where no normal state
+        # information is avaiable, e.g. for "DOWNTIMESTOPPED (UP)"
+        name = nagios_short_host_state_names.get(row["host_state"], "")
     else:
         state = "p"
-        name = "PEND"
+        name = _("PEND")
+    if is_stale(row):
+        state = str(state) + " stale"
     return "state hstate hstate%s" % state, name
 
 multisite_painters["service_nagios_link"] = {
@@ -317,25 +357,24 @@ multisite_painters["site_icon"] = {
     "sorter"  : 'site',
 }
 
-
 multisite_painters["svc_plugin_output"] = {
     "title"   : _("Output of check plugin"),
     "short"   : _("Status detail"),
     "columns" : ["service_plugin_output"],
-    "paint"   : lambda row: ("", format_plugin_output(row["service_plugin_output"], row)),
+    "paint"   : lambda row: paint_stalified(row, format_plugin_output(row["service_plugin_output"], row)),
     "sorter"  : 'svcoutput',
 }
 multisite_painters["svc_long_plugin_output"] = {
     "title"   : _("Long output of check plugin (multiline)"),
     "short"   : _("Status detail"),
     "columns" : ["service_long_plugin_output"],
-    "paint"   : lambda row: (None, row["service_long_plugin_output"].replace('\\n', '<br>')),
+    "paint"   : lambda row: paint_stalified(row, format_plugin_output(row["service_long_plugin_output"], row).replace('\\n', '<br>').replace('\n', '<br>')),
 }
 multisite_painters["svc_perf_data"] = {
     "title" : _("Service performance data"),
     "short" : _("Perfdata"),
     "columns" : ["service_perf_data"],
-    "paint" : lambda row: (None, row["service_perf_data"])
+    "paint" : lambda row: paint_stalified(row, row["service_perf_data"])
 }
 
 def get_perfdata_nth_value(row, n, remove_unit = False):
@@ -357,7 +396,7 @@ def get_perfdata_nth_value(row, n, remove_unit = False):
         return str(e)
 
 def paint_perfdata_nth_value(row, n):
-    return "", get_perfdata_nth_value(row, n)
+    return paint_stalified(row, get_perfdata_nth_value(row, n))
 
 multisite_painters["svc_perf_val01"] = {
     "title" : _("Service performance data - value number  1"),
@@ -440,14 +479,14 @@ multisite_painters["svc_check_command"] = {
     "title"   : _("Service check command"),
     "short"   : _("Check command"),
     "columns" : ["service_check_command"],
-    "paint"   : lambda row: (None, row["service_check_command"]),
+    "paint"   : lambda row: (None, html.attrencode(row["service_check_command"])),
 }
 
 multisite_painters["svc_check_command_expanded"] = {
     "title"   : _("Service check command expanded"),
     "short"   : _("Check command expanded"),
     "columns" : ["service_check_command_expanded"],
-    "paint"   : lambda row: (None, row["service_check_command_expanded"]),
+    "paint"   : lambda row: (None, html.attrencode(row["service_check_command_expanded"])),
 }
 
 multisite_painters["svc_contacts"] = {
@@ -473,6 +512,14 @@ multisite_painters["service_description"] = {
     "sorter"  : 'svcdescr',
 }
 
+multisite_painters["service_display_name"] = {
+    "title"   : _("Service alternative display name"),
+    "short"   : _("Display name"),
+    "columns" : ["service_display_name"],
+    "paint"   : lambda row: (None, row["service_display_name"]),
+    "sorter"  : 'svcdispname',
+}
+
 multisite_painters["svc_state_age"] = {
     "title"   : _("The age of the current service state"),
     "short"   : _("Age"),
@@ -481,12 +528,19 @@ multisite_painters["svc_state_age"] = {
     "paint"   : lambda row: paint_age(row["service_last_state_change"], row["service_has_been_checked"] == 1, 60 * 10),
     "sorter"  : "stateage",
 }
+
+def paint_checked(what, row):
+    css, td = paint_age(row[what + "_last_check"], row[what + "_has_been_checked"] == 1, 0)
+    if is_stale(row):
+        css += " staletime"
+    return css, td
+
 multisite_painters["svc_check_age"] = {
     "title"   : _("The time since the last check of the service"),
     "short"   : _("Checked"),
     "columns" : [ "service_has_been_checked", "service_last_check" ],
     "options" : [ "ts_format", "ts_date" ],
-    "paint"   : lambda row: paint_age(row["service_last_check"], row["service_has_been_checked"] == 1, 0),
+    "paint"   : lambda row: paint_checked("service", row),
 }
 
 multisite_painters["svc_next_check"] = {
@@ -496,6 +550,13 @@ multisite_painters["svc_next_check"] = {
     "paint"   : lambda row: paint_future_time(row["service_next_check"]),
 }
 
+multisite_painters["svc_last_time_ok"] = {
+    "title"   : _("The last time the service was OK"),
+    "short"   : _("Last OK"),
+    "columns" : [ "service_last_time_ok", "service_has_been_checked" ],
+    "paint"   : lambda row: paint_age(row["service_last_time_ok"], row["service_has_been_checked"] == 1, 60 * 10),
+}
+
 multisite_painters["svc_next_notification"] = {
     "title"   : _("The time of the next service notification"),
     "short"   : _("Next notification"),
@@ -511,6 +572,14 @@ multisite_painters["svc_last_notification"] = {
     "paint"   : lambda row: paint_age(row["service_last_notification"], row["service_last_notification"], 0),
 }
 
+multisite_painters['svc_notification_number'] = {
+    "title"     : _("Service notification number"),
+    "short"     : _("N#"),
+    "columns"   : [ "service_current_notification_number" ],
+    "paint"     : lambda row: ("", str(row["service_current_notification_number"])),
+}
+
+
 multisite_painters["svc_check_latency"] = {
     "title"   : _("Service check latency"),
     "short"   : _("Latency"),
@@ -561,7 +630,7 @@ multisite_painters["svc_check_type"] = {
 
 def paint_nagiosflag(row, field, bold_if_nonzero):
     value = row[field]
-    yesno = {True:"yes", False:"no"}[value != 0]
+    yesno = {True: _("yes"), False: _("no")}[value != 0]
     if (value != 0) == bold_if_nonzero:
         return "badflag", yesno
     else:
@@ -631,24 +700,36 @@ def paint_pnpgraph(sitename, host, service = "_HOST_"):
         with_link = 'true'
     else:
         with_link = 'false'
-    pnpview = get_painter_option("pnpview")
+
+    pnp_timerange = get_painter_option("pnp_timerange")
+
+    pnpview = '1'
+    from_ts, to_ts = 'null', 'null'
+    if pnp_timerange != None:
+        if pnp_timerange[0] != 'pnp_view':
+            vs = multisite_painter_options["pnp_timerange"]['valuespec']
+            from_ts, to_ts = map(int, vs.compute_range(pnp_timerange)[0])
+        else:
+            pnpview = pnp_timerange[1]
+
     return "pnpgraph", "<div id=\"%s\"></div>" \
-                       "<script>render_pnp_graphs('%s', '%s', '%s', '%s', '%s', '%s', '%s', %s)</script>" % \
+                       "<script>render_pnp_graphs('%s', '%s', '%s', '%s', '%s', '%s', '%s', %s, '%s', %s, %s)</script>" % \
                           (container_id, container_id, sitename, host, service, pnpview,
-                           defaults.url_prefix + "check_mk/", pnp_url, with_link)
+                           defaults.url_prefix + "check_mk/", pnp_url, with_link, _('Add this graph to...'), from_ts, to_ts)
 
 multisite_painters["svc_pnpgraph" ] = {
     "title"   : _("PNP service graph"),
     "short"   : _("PNP graph"),
     "columns" : [ "host_name", "service_description" ],
-    "options" : [ "pnpview" ],
+    "options" : [ 'pnp_timerange' ],
     "paint"   : lambda row: paint_pnpgraph(row["site"], row["host_name"], row["service_description"]),
+    "printable" : False,
 }
 
 def paint_check_manpage(row):
     command = row["service_check_command"]
     if not command.startswith("check_mk-"):
-	return "", ""
+        return "", ""
     checktype = command[9:]
     # Honor man-pages in OMD's local structure
     p = None
@@ -659,23 +740,23 @@ def paint_check_manpage(row):
     if not p:
         p = defaults.check_manpages_dir + "/" + checktype
     if os.path.isfile(p):
-	description = None
-	for line in file(p):
-	    line = line.rstrip()
-	    if line == "description:":
-		description = ""
-	    elif line.strip() == "" and description != None:
-		description += "<p>"
-	    elif not line.startswith(' ') and line[-1] == ':':
-		break
-	    elif description != None:
-	        description += " " + line
-	if not description:
-	    return "", ""
-	else:
-	    return "", description.replace("{", "<b>").replace("}", "</b>")
+        description = None
+        for line in file(p):
+            line = line.rstrip()
+            if line == "description:":
+                description = ""
+            elif line.strip() == "" and description != None:
+                description += "<p>"
+            elif not line.startswith(' ') and line[-1] == ':':
+                break
+            elif description != None:
+                description += " " + line.replace("<", "&lt;").replace(">", "&gt;")
+        if not description:
+            return "", ""
+        else:
+            return "", description.replace("{", "<b>").replace("}", "</b>")
     else:
-	return "", _("Man-Page: %s not found.") % p
+        return "", _("Man-Page: %s not found.") % p
 
 multisite_painters["check_manpage"] = {
     "title"   : _("Check manual (for Check_MK based checks)"),
@@ -686,7 +767,7 @@ multisite_painters["check_manpage"] = {
 
 def paint_comments(prefix, row):
     comments = row[ prefix + "comments_with_info"]
-    text = ", ".join(["<i>%s</i>: %s" % (a, htmllib.attrencode(c)) for (id, a, c) in comments ])
+    text = ", ".join(["<i>%s</i>: %s" % (a, html.attrencode(c)) for (id, a, c) in comments ])
     return "", text
 
 multisite_painters["svc_comments"] = {
@@ -718,10 +799,10 @@ def notes_matching_pattern_entries(dirs, item):
                     matching.append(dir + "/" + pattern)
     return matching
 
-def paint_custom_notes(row):
+def paint_custom_notes(what, row):
     host = row["host_name"]
     svc = row.get("service_description")
-    if svc:
+    if what == "service":
         notes_dir = defaults.default_config_dir + "/notes/services"
         dirs = notes_matching_pattern_entries([notes_dir], host)
         item = svc
@@ -756,15 +837,67 @@ multisite_painters["svc_custom_notes"] = {
     "title"   : _("Custom services notes"),
     "short"   : _("Notes"),
     "columns" : [ "host_name", "host_address", "service_description", "service_plugin_output" ],
-    "paint"   : paint_custom_notes,
+    "paint"   : lambda row: paint_custom_notes("service", row),
 }
 
-#   _   _           _
-#  | | | | ___  ___| |_ ___
-#  | |_| |/ _ \/ __| __/ __|
-#  |  _  | (_) \__ \ |_\__ \
-#  |_| |_|\___/|___/\__|___/
-#
+multisite_painters["svc_staleness"] = {
+    "title"   : _("Service staleness value"),
+    "short"   : _("Staleness"),
+    "columns" : ["service_staleness"],
+    "paint"   : lambda row: ('', '%0.2f' % row.get('service_staleness', 0)),
+}
+
+def paint_is_stale(row):
+    if is_stale(row):
+        return "badflag", _('yes')
+    else:
+        return "goodflag", _('no')
+
+multisite_painters["svc_is_stale"] = {
+    "title"   : _("Service is stale"),
+    "short"   : _("Stale"),
+    "columns" : ["service_staleness"],
+    "paint"   : paint_is_stale,
+    "sorter"  : 'svc_staleness',
+}
+
+multisite_painters["svc_servicelevel"] = {
+    "title"   : _("Service service level"),
+    "short"   : _("Service Level"),
+    "columns" : [ "service_custom_variable_names", "service_custom_variable_values" ],
+    "paint"   : lambda row: paint_custom_var('service', 'EC_SL', row),
+    "sorter"  : 'servicelevel',
+}
+
+def paint_custom_vars(what, row, blacklist=[]):
+    items = row[what + "_custom_variables"].items()
+    items.sort()
+    code = '<table class=customvars>'
+    for varname, value in items:
+        if varname not in blacklist:
+            code += '<tr><td>%s</td><td>%s</td></tr>' % (varname, value)
+    code += '</table>'
+    return "", code
+
+multisite_painters["svc_custom_vars"] = {
+    "title"   : _("Service custom variables"),
+    "columns" : [ "service_custom_variables" ],
+    "paint"   : lambda row: paint_custom_vars('service', row),
+}
+
+
+#.
+#   .--Hosts---------------------------------------------------------------.
+#   |                       _   _           _                              |
+#   |                      | | | | ___  ___| |_ ___                        |
+#   |                      | |_| |/ _ \/ __| __/ __|                       |
+#   |                      |  _  | (_) \__ \ |_\__ \                       |
+#   |                      |_| |_|\___/|___/\__|___/                       |
+#   |                                                                      |
+#   +----------------------------------------------------------------------+
+#   | Painters for hosts                                                   |
+#   '----------------------------------------------------------------------'
+
 
 multisite_painters["host_state"] = {
     "title"   : _("Host state"),
@@ -823,7 +956,7 @@ multisite_painters["host_check_age"] = {
     "short"   : _("Checked"),
     "columns" : [ "host_has_been_checked", "host_last_check" ],
     "options" : [ "ts_format", "ts_date" ],
-    "paint"   : lambda row: paint_age(row["host_last_check"], row["host_has_been_checked"] == 1, 0),
+    "paint"   : lambda row: paint_checked("host", row),
 }
 
 multisite_painters["host_next_check"] = {
@@ -910,6 +1043,13 @@ multisite_painters["host_notifper"] = {
     "paint"   : lambda row: (None, row["host_notification_period"]),
 }
 
+multisite_painters['host_notification_number'] = {
+    "title"     : _("Host notification number"),
+    "short"     : _("N#"),
+    "columns"   : [ "host_current_notification_number" ],
+    "paint"     : lambda row: ("", str(row["host_current_notification_number"])),
+}
+
 multisite_painters["host_flapping"] = {
     "title"   : _("Host is flapping"),
     "short"   : _("Flap"),
@@ -923,13 +1063,20 @@ multisite_painters["host_is_active"] = {
     "columns" : [ "host_active_checks_enabled" ],
     "paint"   : lambda row: paint_nagiosflag(row, "host_active_checks_enabled", None),
 }
+multisite_painters["host_notifications_enabled"] = {
+    "title"   : _("Host notifications enabled"),
+    "short"   : _("Notif."),
+    "columns" : [ "host_notifications_enabled" ],
+    "paint"   : lambda row: paint_nagiosflag(row, "host_notifications_enabled", False),
+}
 
 multisite_painters["host_pnpgraph" ] = {
     "title"   : _("PNP host graph"),
     "short"   : _("PNP graph"),
     "columns" : [ "host_name" ],
-    "options" : [ "pnpview" ],
-    "paint"   : lambda row: paint_pnpgraph(row["site"], row["host_name"])
+    "options" : [ 'pnp_timerange' ],
+    "paint"   : lambda row: paint_pnpgraph(row["site"], row["host_name"]),
+    "printable" : False,
 }
 
 def paint_host_black(row):
@@ -950,7 +1097,7 @@ multisite_painters["host_black"] = {
 def paint_host_black_with_link_to_old_nagios_services(row):
     host = row["host_name"]
     baseurl = config.site(row["site"])["url_prefix"] + "nagios/cgi-bin"
-    url = baseurl + "/status.cgi?host=" + htmllib.urlencode(host)
+    url = baseurl + "/status.cgi?host=" + html.urlencode(host)
     state = row["host_state"]
     if state != 0:
         return None, '<div class=hostdown><a href="%s">%s</a></div>' % (url, host)
@@ -1090,9 +1237,9 @@ def paint_service_list(row, columnname):
             host = row["host_name"]
             text = svc
         link = "view.py?view_name=service&site=%s&host=%s&service=%s" % (
-                htmllib.urlencode(row["site"]),
-                htmllib.urlencode(host),
-                htmllib.urlencode(svc))
+                html.urlencode(row["site"]),
+                html.urlencode(host),
+                html.urlencode(svc))
         if checked:
             css = "state%d" % state
         else:
@@ -1127,7 +1274,7 @@ def paint_host_group_memberlist(row):
     for group in row["host_groups"]:
         link = "view.py?view_name=hostgroup&hostgroup=" + group
         if html.var("display_options"):
-            link += "&display_options=%s" % html.var("display_options")
+            link += "&display_options=%s" % html.attrencode(html.var("display_options"))
         links.append('<a href="%s">%s</a>' % (link, group))
     return "", ", ".join(links)
 
@@ -1156,21 +1303,11 @@ multisite_painters["host_custom_notes"] = {
     "title"   : _("Custom host notes"),
     "short"   : _("Notes"),
     "columns" : [ "host_name", "host_address", "host_plugin_output" ],
-    "paint"   : paint_custom_notes,
-}
-
-def paint_host_tags(row):
-    return "", get_host_tags(row)
-
-multisite_painters["host_tags"] = {
-    "title"   : _("Host Tags (Check_MK)"),
-    "short"   : _("Tags"),
-    "columns" : [ "host_custom_variable_names", "host_custom_variable_values" ],
-    "paint"   : paint_host_tags,
+    "paint"   : lambda row: paint_custom_notes("hosts", row),
 }
 
 multisite_painters["host_comments"] = {
-    "title"   : _("Host Comments"),
+    "title"   : _("Host comments"),
     "short"   : _("Comments"),
     "columns" : [ "host_comments_with_info" ],
     "paint"   : lambda row: paint_comments("host_", row),
@@ -1190,6 +1327,35 @@ multisite_painters["host_acknowledged"] = {
     "paint"   : lambda row: paint_nagiosflag(row, "host_acknowledged", False),
 }
 
+multisite_painters["host_staleness"] = {
+    "title"   : _("Host staleness value"),
+    "short"   : _("Staleness"),
+    "columns" : ["host_staleness"],
+    "paint"   : lambda row: ('', '%0.2f' % row.get('host_staleness', 0)),
+}
+
+multisite_painters["host_is_stale"] = {
+    "title"   : _("Host is stale"),
+    "short"   : _("Stale"),
+    "columns" : ["host_staleness"],
+    "paint"   : paint_is_stale,
+    "sorter"  : 'svc_staleness',
+}
+
+multisite_painters["host_servicelevel"] = {
+    "title"   : _("Host service level"),
+    "short"   : _("Service Level"),
+    "columns" : [ "host_custom_variable_names", "host_custom_variable_values" ],
+    "paint"   : lambda row: paint_custom_var('host', 'EC_SL', row),
+    "sorter"  : 'servicelevel',
+}
+
+multisite_painters["host_custom_vars"] = {
+    "title"   : _("Host custom variables"),
+    "columns" : [ "host_custom_variables" ],
+    "paint"   : lambda row: paint_custom_vars('host', row, [ 'FILENAME', 'TAGS']),
+}
+
 
 #    _   _           _
 #   | | | | ___  ___| |_ __ _ _ __ ___  _   _ _ __  ___
@@ -1202,8 +1368,8 @@ def paint_hg_host_list(row):
     h = "<div class=objectlist>"
     for host, state, checked in row["hostgroup_members_with_state"]:
         link = "view.py?view_name=host&site=%s&host=%s" % (
-                htmllib.urlencode(row["site"]),
-                htmllib.urlencode(host))
+                html.urlencode(row["site"]),
+                html.urlencode(host))
         if checked:
             css = "hstate%d" % state
         else:
@@ -1390,7 +1556,7 @@ multisite_painters["comment_id"] = {
     "title"   : _("Comment id"),
     "short"   : _("ID"),
     "columns" : ["comment_id"],
-    "paint"   : lambda row: (None, row["comment_id"]),
+    "paint"   : lambda row: (None, str(row["comment_id"])),
 }
 multisite_painters["comment_author"] = {
     "title"   : _("Comment author"),
@@ -1402,7 +1568,7 @@ multisite_painters["comment_author"] = {
 multisite_painters["comment_comment"] = {
     "title"   : _("Comment text"),
     "columns" : ["comment_comment"],
-    "paint"   : lambda row: (None, htmllib.attrencode(row["comment_comment"])),
+    "paint"   : lambda row: (None, html.attrencode(row["comment_comment"])),
 }
 
 multisite_painters["comment_what"] = {
@@ -1425,25 +1591,32 @@ multisite_painters["comment_expires"] = {
     "short"   : _("Expires"),
     "columns" : ["comment_expire_time"],
     "options" : [ "ts_format", "ts_date" ],
-    "paint"   : lambda row: paint_age(row["comment_expire_time"], row["comment_expire_time"] != 0, 3600),
+    "paint"   : lambda row: paint_age(row["comment_expire_time"], row["comment_expire_time"] != 0, 3600, what='future'),
 }
 
 def paint_comment_entry_type(row):
     t = row["comment_entry_type"]
     linkview = None
-    if t == 1:   icon = "comment"
+    if t == 1:
+        icon = "comment"
+        help = _("Comment")
     elif t == 2:
         icon = "downtime"
+        help = _("Downtime")
         if row["service_description"]:
             linkview = "downtimes_of_service"
         else:
             linkview = "downtimes_of_host"
 
-    elif t == 3: icon = "flapping"
-    elif t == 4: icon = "ack"
+    elif t == 3:
+        icon = "flapping"
+        help = _("Flapping")
+    elif t == 4:
+        icon = "ack"
+        help = _("Acknowledgement")
     else:
         return "", ""
-    code = '<img class=icon src="images/icon_%s.gif">' % icon
+    code = html.render_icon(icon, help)
     if linkview:
         code = link_to_view(code, row, linkview)
     return "icons", code
@@ -1481,21 +1654,21 @@ multisite_painters["downtime_comment"] = {
     "title"   : _("Downtime comment"),
     "short"   : _("Comment"),
     "columns" : ["downtime_comment"],
-    "paint"   : lambda row: (None, htmllib.attrencode(row["downtime_comment"])),
+    "paint"   : lambda row: (None, html.attrencode(row["downtime_comment"])),
 }
 
 multisite_painters["downtime_fixed"] = {
-    "title"   : _("Downtime is fixed"),
-    "short"   : _("Fixed"),
+    "title"   : _("Downtime start mode"),
+    "short"   : _("Mode"),
     "columns" : ["downtime_fixed"],
     "paint"   : lambda row: (None, row["downtime_fixed"] == 0 and _("flexible") or _("fixed")),
 }
 
 multisite_painters["downtime_what"] = {
-    "title"   : _("Downtime type (host/service)"),
-    "short"   : _("Type"),
-    "columns" : ["is_service"],
-    "paint"   : lambda row: (None, row["is_service"] and _("Service") or _("Host")),
+    "title"   : _("Downtime for host/service"),
+    "short"   : _("for"),
+    "columns" : ["downtime_is_service"],
+    "paint"   : lambda row: (None, row["downtime_is_service"] and _("Service") or _("Host")),
 }
 
 multisite_painters["downtime_type"] = {
@@ -1518,7 +1691,7 @@ multisite_painters["downtime_start_time"] = {
     "short"   : _("Start"),
     "columns" : ["downtime_start_time"],
     "options" : [ "ts_format", "ts_date" ],
-    "paint"   : lambda row: paint_age(row["downtime_start_time"], True, 3600),
+    "paint"   : lambda row: paint_age(row["downtime_start_time"], True, 3600, what=None),
 }
 
 multisite_painters["downtime_end_time"] = {
@@ -1526,19 +1699,16 @@ multisite_painters["downtime_end_time"] = {
     "short"   : _("End"),
     "columns" : ["downtime_end_time"],
     "options" : [ "ts_format", "ts_date" ],
-    "paint"   : lambda row: paint_age(row["downtime_end_time"], True, 3600),
+    "paint"   : lambda row: paint_age(row["downtime_end_time"], True, 3600, what=None),
 }
 
 def paint_downtime_duration(row):
-    if row["downtime_fixed"] == 1:
-        return None, ""
-    else:
-        return None, "%02d:%02d" % divmod(row["downtime_duration"] / 60, 60)
+    return "number", "%02d:%02d:00" % divmod(row["downtime_duration"] / 60, 60)
 
 multisite_painters["downtime_duration"] = {
     "title"   : _("Downtime duration (if flexible)"),
     "short"   : _("Duration"),
-    "columns" : ["downtime_duration", "downtime_fixed"],
+    "columns" : ["downtime_duration", ], # "downtime_fixed"],
     "paint"   : paint_downtime_duration,
 }
 
@@ -1553,13 +1723,16 @@ multisite_painters["log_message"] = {
     "title"   : _("Log: complete message"),
     "short"   : _("Message"),
     "columns" : ["log_message"],
-    "paint"   : lambda row: ("", htmllib.attrencode(row["log_message"])),
+    "paint"   : lambda row: ("", html.attrencode(row["log_message"])),
 }
 
 def paint_log_plugin_output(row):
     output = row["log_plugin_output"]
+    comment = row["log_comment"]
     if output:
         return "", format_plugin_output(output, row)
+    elif comment:
+        return "", comment
     else:
         log_type = row["log_type"]
         lst = row["log_state_type"]
@@ -1573,21 +1746,43 @@ def paint_log_plugin_output(row):
             else:
                 return "", _("The %s started flapping") % what
 
-        return "", (lst + " - " + log_type)
+        elif lst:
+            return "", (lst + " - " + log_type)
+        else:
+            return "", ""
 
 
 multisite_painters["log_plugin_output"] = {
     "title"   : _("Log: output of check plugin"),
     "short"   : _("Check output"),
-    "columns" : ["log_plugin_output", "log_type", "log_state_type" ],
+    "columns" : ["log_plugin_output", "log_type", "log_state_type", "log_comment" ],
     "paint"   : paint_log_plugin_output,
 }
 
+
+def paint_log_type(row):
+    lt = row["log_type"]
+    if "HOST" in lt:
+        return "", _("Host")
+    elif "SERVICE" in lt or "SVC" in lt:
+        return "", _("Service")
+    else:
+        return "", _("Program")
+
+
+multisite_painters["log_what"] = {
+    "title"   : _("Log: host or service"),
+    "short"   : _("Host/Service"),
+    "columns" : [ "log_type" ],
+    "paint"   : paint_log_type,
+}
+
+
 multisite_painters["log_attempt"] = {
     "title"   : _("Log: number of check attempt"),
     "short"   : _("Att."),
     "columns" : ["log_attempt"],
-    "paint"   : lambda row: ("", row["log_attempt"]),
+    "paint"   : lambda row: ("", str(row["log_attempt"])),
 }
 multisite_painters["log_state_type"] = {
     "title"   : _("Log: type of state (hard/soft/stopped/started)"),
@@ -1607,6 +1802,12 @@ multisite_painters["log_contact_name"] = {
     "columns" : ["log_contact_name"],
     "paint"   : lambda row: ("nowrap", row["log_contact_name"]),
 }
+multisite_painters["log_command"] = {
+    "title"   : _("Log: command/plugin"),
+    "short"   : _("Command"),
+    "columns" : ["log_command_name"],
+    "paint"   : lambda row: ("nowrap", row["log_command_name"]),
+}
 def paint_log_icon(row):
     img = None
     log_type = row["log_type"]
@@ -1615,7 +1816,7 @@ def paint_log_icon(row):
     elif log_type == "HOST ALERT":
         img = { 0: "up", 1: "down", 2:"unreach" }.get(row["log_state"])
     elif "DOWNTIME" in log_type:
-        if row["log_state_type"] == "STOPPED":
+        if row["log_state_type"] in [ "END", "STOPPED" ]:
             img = "downtimestop"
         else:
             img = "downtime"
@@ -1627,13 +1828,18 @@ def paint_log_icon(row):
         img = "restart"
     elif "starting..." in log_type:
         img = "start"
-    elif "shutdown..." in log_type:
+    elif "shutdown..." in log_type or "shutting down" in log_type:
         img = "stop"
     elif " FLAPPING " in log_type:
         img = "flapping"
+    elif "ACKNOWLEDGE ALERT" in log_type:
+        if row["log_state_type"] == "STARTED":
+            img = "ack"
+        else:
+            img = "ackstop"
 
     if img:
-        return "icon", '<img src="images/alert_%s.png">' % img
+        return "icon", '<img class=icon src="images/alert_%s.png">' % img
     else:
         return "icon", ""
 
@@ -1648,14 +1854,14 @@ multisite_painters["log_options"] = {
     "title"   : _("Log: informational part of message"),
     "short"   : _("Info"),
     "columns" : ["log_options"],
-    "paint"   : lambda row: ("", htmllib.attrencode(row["log_options"])),
+    "paint"   : lambda row: ("", html.attrencode(row["log_options"])),
 }
 
 def paint_log_comment(msg):
     if ';' in msg:
         parts = msg.split(';')
         if len(parts) > 6:
-          return ("", htmllib.attrencode(parts[-1]))
+          return ("", html.attrencode(parts[-1]))
     return ("", "")
 
 multisite_painters["log_comment"] = {
@@ -1738,3 +1944,85 @@ multisite_painters["alert_stats_problem"] = {
     "columns" : [ "alerts_problem" ],
     "paint" : lambda row: paint_svc_count('s', row["alerts_problem"])
 }
+
+#
+# HOSTTAGS
+#
+
+def paint_host_tags(row):
+    return "", get_host_tags(row)
+
+multisite_painters["host_tags"] = {
+    "title"   : _("Host tags (raw)"),
+    "short"   : _("Tags"),
+    "columns" : [ "host_custom_variable_names", "host_custom_variable_values" ],
+    "paint"   : paint_host_tags,
+    "sorter"  : 'host',
+}
+
+def paint_host_tags_with_titles(row):
+    output = ''
+    misc_tags = []
+    for tag in get_host_tags(row).split():
+        group_title = config.tag_group_title(tag)
+        if group_title:
+            output += group_title + ': ' + (config.tag_alias(tag) or tag) + '<br />\n'
+        else:
+            misc_tags.append(tag)
+
+    if misc_tags:
+        output += _('Misc:') + ' ' + ', '.join(misc_tags)
+
+    return "", output
+
+multisite_painters["host_tags_with_titles"] = {
+    "title"   : _("Host tags (with titles)"),
+    "short"   : _("Tags"),
+    "columns" : [ "host_custom_variable_names", "host_custom_variable_values" ],
+    "paint"   : paint_host_tags_with_titles,
+    "sorter"  : 'host',
+}
+
+
+def paint_host_tag(row, tgid):
+    tags_of_host = get_host_tags(row).split()
+
+    for t in get_tag_group(tgid)[1]:
+        if t[0] in tags_of_host:
+            return "", t[1]
+    return "", _("N/A")
+
+# Use title of the tag value for grouping, not the complete
+# dictionary of custom variables!
+def groupby_host_tag(row, tgid):
+    cssclass, title = paint_host_tag(row, tgid)
+    return title
+
+def load_host_tag_painters():
+    # first remove all old painters to reflect delted painters during runtime
+    for key in multisite_painters.keys():
+        if key.startswith('host_tag_'):
+            del multisite_painters[key]
+
+    for entry in config.wato_host_tags:
+        tgid = entry[0]
+        tit  = entry[1]
+        ch   = entry[2]
+
+        long_tit = tit
+        if '/' in tit:
+            topic, tit = tit.split('/', 1)
+            if topic:
+                long_tit = topic + ' / ' + tit
+            else:
+                long_tit = tit
+
+        multisite_painters["host_tag_" + tgid] = {
+            "title"   : _("Host tag:") + ' ' + long_tit,
+            "name"    : "host_tag_" + tgid,
+            "short"   : tit,
+            "columns" : [ "host_custom_variables" ],
+            "paint"   : paint_host_tag,
+            "groupby" : groupby_host_tag,
+            "args"    : [ tgid ],
+        }
