@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -96,11 +96,8 @@ multisite_sorters["hoststate"] = {
 }
 
 def cmp_site_host(r1, r2):
-    c = cmp(r1["site"], r2["site"])
-    if c != 0:
-        return c
-    else:
-        return cmp_simple_string("host_name", r1, r2)
+    return cmp(r1["site"], r2["site"]) or \
+           cmp_num_split("host_name", r1, r2)
 
 multisite_sorters["site_host"] = {
     "title"   : _("Host"),
@@ -121,13 +118,39 @@ def cmp_host_tags(r1, r2):
     return cmp(get_host_tags(r1), get_host_tags(r2))
 
 multisite_sorters["host"] = {
-    "title"   : _("Host Tags (Check_MK)"),
+    "title"   : _("Host Tags (raw)"),
     "columns" : [ "host_custom_variable_names", "host_custom_variable_values" ],
     "cmp"     : cmp_host_tags,
 }
 
+multisite_sorters['servicelevel'] = {
+    'title'   : _("Servicelevel"),
+    'columns' : [ 'custom_variable_names', 'custom_variable_values' ],
+    'cmp'     : lambda r1, r2: cmp_custom_variable(r1, r2, 'EC_SL', cmp_simple_number)
+}
+
+def cmp_service_name_equiv(r):
+    if r == "Check_MK":
+        return -5
+    elif r == "Check_MK Discovery":
+        return -4
+    elif r == "Check_MK inventory":
+        return -3 # FIXME: Remove old name one day
+    elif r == "Check_MK HW/SW Inventory":
+        return -2
+    else:
+        return 0
+
+def cmp_service_name(column, r1, r2):
+    o = cmp(cmp_service_name_equiv(r1[column]), cmp_service_name_equiv(r2[column]))
+    if o == 0:
+        return cmp_simple_string(column, r1, r2)
+    else:
+        return o
+
 #                      name           title                    column                       sortfunction
-declare_simple_sorter("svcdescr",                _("Service description"),         "service_description",        cmp_simple_string)
+declare_simple_sorter("svcdescr",                _("Service description"),         "service_description",        cmp_service_name)
+declare_simple_sorter("svcdispname",             _("Service alternative display name"),   "service_display_name",  cmp_simple_string)
 declare_simple_sorter("svcoutput",               _("Service plugin output"),       "service_plugin_output",      cmp_simple_string)
 declare_simple_sorter("svc_long_plugin_output",  _("Long output of check plugin"), "service_long_plugin_output", cmp_simple_string)
 declare_simple_sorter("site",                    _("Site"),                        "site",                       cmp_simple_string)
@@ -155,6 +178,8 @@ declare_1to1_sorter("svc_notifications_enabled",  cmp_simple_number)
 declare_1to1_sorter("svc_is_active",              cmp_simple_number)
 declare_1to1_sorter("svc_group_memberlist",       cmp_string_list)
 declare_1to1_sorter("svc_acknowledged",           cmp_simple_number)
+declare_1to1_sorter("svc_staleness",              cmp_simple_number)
+declare_1to1_sorter("svc_servicelevel",           cmp_simple_number)
 
 def cmp_perfdata_nth_value(r1, r2, n):
     return cmp(savefloat(get_perfdata_nth_value(r1, n)), savefloat(get_perfdata_nth_value(r2, n)))
@@ -212,8 +237,8 @@ multisite_sorters['svc_perf_val10'] = {
 
 
 # Host
-declare_1to1_sorter("alias",                  cmp_simple_string)
-declare_1to1_sorter("host_address",           cmp_simple_string)
+declare_1to1_sorter("alias",                  cmp_num_split)
+declare_1to1_sorter("host_address",           cmp_ip_address)
 declare_1to1_sorter("host_plugin_output",     cmp_simple_string)
 declare_1to1_sorter("host_perf_data",         cmp_simple_string)
 declare_1to1_sorter("host_check_command",     cmp_simple_string)
@@ -243,6 +268,7 @@ declare_1to1_sorter("host_childs",            cmp_string_list)
 declare_1to1_sorter("host_group_memberlist",  cmp_string_list)
 declare_1to1_sorter("host_contacts",          cmp_string_list)
 declare_1to1_sorter("host_contact_groups",    cmp_string_list)
+declare_1to1_sorter("host_servicelevel",      cmp_simple_number)
 
 def cmp_host_problems(r1, r2):
     return cmp(r1["host_num_services"] - r1["host_num_services_ok"] - r1["host_num_services_pending"],
@@ -293,7 +319,7 @@ declare_1to1_sorter("downtime_author",         cmp_simple_string)
 declare_1to1_sorter("downtime_comment",        cmp_simple_string)
 declare_1to1_sorter("downtime_fixed",          cmp_simple_number)
 declare_1to1_sorter("downtime_type",           cmp_simple_number)
-declare_simple_sorter("downtime_what",         _("Downtime type (host/service)"),  "is_service",            cmp_simple_number)
+declare_simple_sorter("downtime_what",         _("Downtime for host/service"),     "downtime_is_service",   cmp_simple_number)
 declare_simple_sorter("downtime_start_time",   _("Downtime start"),                "downtime_start_time",   cmp_simple_number)
 declare_simple_sorter("downtime_end_time",     _("Downtime end"),                  "downtime_end_time",     cmp_simple_number)
 declare_simple_sorter("downtime_entry_time",   _("Downtime entry time"),           "downtime_entry_time",   cmp_simple_number)
@@ -307,6 +333,19 @@ declare_1to1_sorter("log_contact_name",        cmp_simple_string)
 declare_1to1_sorter("log_time",                cmp_simple_number)
 declare_1to1_sorter("log_lineno",              cmp_simple_number)
 
+def cmp_log_what(col, a, b):
+    return cmp(log_what(a[col]), log_what(b[col]))
+
+def log_what(t):
+    if "HOST" in t:
+        return 1
+    elif "SERVICE" in t or "SVC" in t:
+        return 2
+    else:
+        return 0
+
+declare_1to1_sorter("log_what",                cmp_log_what)
+
 import time
 def get_day_start_timestamp(t):
     st    = time.localtime(int(t))
@@ -333,3 +372,34 @@ declare_simple_sorter("alerts_problem",  _("Number of problem alerts"),  "alerts
 # Aggregations
 declare_simple_sorter("aggr_name",   _("Aggregation name"),  "aggr_name",       cmp_simple_string)
 declare_simple_sorter("aggr_group",  _("Aggregation group"),  "aggr_group",       cmp_simple_string)
+
+#
+# SINGLE HOSTTAG FIELDS
+#
+
+def cmp_host_tag(r1, r2, tgid):
+    tags1 = get_host_tags(r1).split()
+    tags2 = get_host_tags(r2).split()
+
+    val1 = _('N/A')
+    val2 = _('N/A')
+    for t in get_tag_group(tgid)[1]:
+        if t[0] in tags1:
+            val1 = t[1]
+        if t[0] in tags2:
+            val2 = t[1]
+
+    return cmp(val1, val2)
+
+for entry in config.wato_host_tags:
+    tgid = entry[0]
+    tit  = entry[1]
+
+    declare_simple_sorter("host_tag_" + tgid,  _("Host tag:") + ' ' + tit,  "host_tag_" + tgid, cmp_simple_string)
+
+    multisite_sorters["host_tag_" + tgid] = {
+        "title"   : _("Host tag:") + ' ' + tit,
+        "columns" : [ "host_custom_variable_names", "host_custom_variable_values" ],
+        "cmp"     : cmp_host_tag,
+        "args"    : [ tgid ],
+    }
