@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -25,7 +25,7 @@
 # Boston, MA 02110-1301 USA.
 
 declare_host_attribute(ContactGroupsAttribute(),
-                       show_in_table = True,
+                       show_in_table = False,
                        show_in_folder = True)
 
 declare_host_attribute(NagiosTextAttribute("alias", "alias", _("Alias"),
@@ -37,15 +37,32 @@ declare_host_attribute(NagiosTextAttribute("alias", "alias", _("Alias"),
 declare_host_attribute(TextAttribute("ipaddress", _("IP address"),
                        _("In case the name of the host is not resolvable via <tt>/etc/hosts</tt> "
                          "or DNS by your monitoring server, you can specify an explicit IP "
-                         "address or a resolvable DNS name of the host here. <b>Note</b>: If you leave "
-                         "this attribute empty, then DNS resolution will be done when you activate "
-                         "the configuration. When you enter a DNS name here, the DNS resolution will "
-                         "be done each time the host is checked. Use this only for hosts with "
-                         "dynamic IP addresses."),
+                         "address or a resolvable DNS name of the host here.<br> <b>Notes</b>:<br> "
+                         "1. If you leave this attribute empty, hostname resolution will be done when "
+                         "you activate the configuration. "
+                         "Check_MKs builtin DNS cache is activated per default in the global "
+                         "configuration to speed up the activation process. The cache is normally "
+                         "updated daily with a cron job. You can manually update the cache with the "
+                         "command <tt>cmk -v --update-dns-cache</tt>.<br>"
+                         "2. If you enter a DNS name here, the DNS resolution will be carried out "
+                         "each time the host is checked. Check_MKs DNS cache will NOT be queried. "
+                         "Use this only for hosts with dynamic IP addresses."
+                         ),
                          allow_empty = False),
                          show_in_table = True,
                          show_in_folder = False)
 
+declare_host_attribute(TextAttribute("snmp_community", _("SNMP Community"),
+                       _("Using this option you can configure the community which should be used when "
+                         "contacting this host via SNMP v1 or v2. It is possible to configure the SNMP community by "
+                         "using the <a href=\"%s\">SNMP Communities</a> ruleset, but when you configure "
+                         "a community here, this will override the community defined by the rules.") % \
+                             html.makeuri([('mode', 'edit_ruleset'), ('varname', 'snmp_communities')]),
+                         allow_empty = False),
+                         show_in_table = False,
+                         show_in_folder = True,
+                         depends_on_tags = ['snmp'])
+
 # Attribute for configuring parents
 class ParentsAttribute(ValueSpecAttribute):
     def __init__(self):
@@ -70,7 +87,7 @@ class ParentsAttribute(ValueSpecAttribute):
 
     def paint(self, value, hostname):
         parts = [ '<a href="%s">%s</a>' % (
-                   "wato.py?" + htmllib.urlencode_vars([("mode", "edithost"), ("host", hn)]), hn)
+                   "wato.py?" + html.urlencode_vars([("mode", "edithost"), ("host", hn)]), hn)
                   for hn in value ]
         return "", ", ".join(parts)
 
@@ -94,5 +111,5 @@ def validate_host_parents(effective_host):
                   "relation is used to describe the reachability of hosts by one monitoring daemon.") %
                     (parentname, parent["site"], effective_host["site"]))
 
-api.register_hook('validate-host', validate_host_parents)
+register_hook('validate-host', validate_host_parents)
 
