@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -26,58 +26,20 @@
 
 import config
 import lib
-
-#   .--Treestates----------------------------------------------------------.
-#   |            _____                   _        _                        |
-#   |           |_   _| __ ___  ___  ___| |_ __ _| |_ ___  ___             |
-#   |             | || '__/ _ \/ _ \/ __| __/ _` | __/ _ \/ __|            |
-#   |             | || | |  __/  __/\__ \ || (_| | ||  __/\__ \            |
-#   |             |_||_|  \___|\___||___/\__\__,_|\__\___||___/            |
-#   |                                                                      |
-#   +----------------------------------------------------------------------+
-#   | Saves and loads the current states of foldertrees for the user       |
-#   +----------------------------------------------------------------------+
-
-treestates = {}
-treestates_for_id = None
-
-def load_tree_states():
-    global treestates
-    global treestates_for_id
-    if html.id is not treestates_for_id:
-        treestates = config.load_user_file("treestates", {})
-        treestates_for_id = html.id
-
-def save_tree_states():
-    config.save_user_file("treestates", treestates)
-
-def get_tree_states(tree):
-    load_tree_states()
-    return treestates.get(tree, {})
-
-def set_tree_state(tree, key, val):
-    load_tree_states()
-
-    if tree not in treestates:
-        treestates[tree] = {}
-
-    treestates[tree][key] = val
-
-def set_tree_states(tree, val):
-    load_tree_states()
-    treestates[tree] = val
+import re
 
 def ajax_tree_openclose():
-    load_tree_states()
+    html.load_tree_states()
 
     tree = html.var("tree")
-    name = html.var("name")
+    name = html.var_utf8("name")
 
     if not tree or not name:
         MKUserError('tree or name parameter missing')
 
-    set_tree_state(tree, name, html.var("state"))
-    save_tree_states()
+    html.set_tree_state(tree, name, html.var("state"))
+    html.save_tree_states()
+    html.write('OK') # Write out something to make debugging easier
 
 #   .--Row Selector--------------------------------------------------------.
 #   |      ____                 ____       _           _                   |
@@ -116,16 +78,25 @@ def cleanup_old_selections():
 # Generates a selection id or uses the given one
 def selection_id():
     if not html.has_var('selection'):
-        sel_id = file('/proc/sys/kernel/random/uuid').read().strip()
-        html.add_var('selection', sel_id)
-    return html.var('selection')
+        sel_id = lib.gen_id()
+        html.set_var('selection', sel_id)
+        return sel_id
+    else:
+        sel_id = html.var('selection')
+        # Avoid illegal file access by introducing .. or /
+        if not re.match("^[-0-9a-zA-Z]+$", sel_id):
+            new_id = lib.gen_id()
+            html.set_var('selection', new_id)
+            return new_id
+        else:
+            return sel_id
 
 def get_rowselection(ident):
     vo = config.load_user_file("rowselection/%s" % selection_id(), {})
     return vo.get(ident, [])
 
 def set_rowselection(ident, rows, action):
-    vo = config.load_user_file("rowselection/%s" % selection_id(), {})
+    vo = config.load_user_file("rowselection/%s" % selection_id(), {}, True)
 
     if action == 'set':
         vo[ident] = rows
@@ -142,7 +113,7 @@ def set_rowselection(ident, rows, action):
     if not os.path.exists(config.user_confdir + '/rowselection'):
         make_nagios_directory(config.user_confdir + '/rowselection')
 
-    config.save_user_file("rowselection/%s" % selection_id(), vo)
+    config.save_user_file("rowselection/%s" % selection_id(), vo, True)
 
 def ajax_set_rowselection():
     ident = html.var('id')
